searchData={"items":[{"type":"module","title":"DateConverter","doc":"Convert `String` to one of the `Date`, `DateTime`, `Time` or `NaiveDateTime` types.","ref":"DateConverter.html"},{"type":"function","title":"DateConverter.convert/1","doc":"Convert the date `String` parameter to the `Date`/`DateTime`/`NaiveDateTime`/`Time` based on the format of the provided string.","ref":"DateConverter.html#convert/1"},{"type":"function","title":"Examples - DateConverter.convert/1","doc":"A simple date in the format `YYYY-MM-DD` (iso8601) will be converted to the `Date` type.\n\n      iex> DateConverter.convert(\"2020-10-11\")\n      Date.new(2020, 10, 11)\n\n      iex> DateConverter.convert(\"2020-22-11\")\n      {:error, :invalid_date}\n\n  String in the form of `YYYY-MM-DD hh:mm::ss` will be converted to `NaiveDateTime` struct.\n\n      iex> DateConverter.convert(\"2020-10-11 11:34:48\")\n      NaiveDateTime.new(2020, 10, 11, 11, 34, 48)\n\n      iex> DateConverter.convert(\"2020-10-11 11:34:78\")\n      {:error, :invalid_time}\n\n  String in the form of `YYYY-MM-DDThh:mm::ss` will be converted to `DateTime` struct.\n\n      iex> DateConverter.convert(\"2020-10-11T11:34:48.00Z\")\n      DateTime.new(~D[2020-10-11], ~T[11:34:48.00])\n\n      iex> DateConverter.convert(\"2020-10-11T11:34:78.00Z\")\n      {:error, :invalid_time}\n\n  String in the form of `hh:mm:ss.` will be converted to `Time` struct.\n\n      iex> DateConverter.convert(\"11:34:48\")\n      Time.new(11, 34, 48)","ref":"DateConverter.html#convert/1-examples"},{"type":"module","title":"Dsv","doc":"The `Dsv` module provides a set of functions for validating user data of various types.\nIn addition to simple data types such as strings, numbers, and dates, this module offers functionality for validating complex data structures like maps, lists, and structs.","ref":"Dsv.html"},{"type":"module","title":"Getting Started - Dsv","doc":"The simplest way to utilize this library is by using the `Dsv.validate/2` function with specified validation options.\nThis function supports various validators, which you can explore in the \"Basic Validators\" section of the documentation.\nThese validators include `:length`, `:number`, `:format`, and more.\n\n#","ref":"Dsv.html#module-getting-started"},{"type":"module","title":"Example - validate a simple value with multiple validators. - Dsv","doc":"\"This is an example of validating a simple string value\"\n    |> Dsv.validate(length: [min: 10, max: 100], format: ~r/[A-Z][a-z]+/)\n\nIn this example, the string value is checked against several constraints: its length must be at least 10 and at most 100 graphemes, and it must start with an uppercase letter followed by at least one lowercase letter.\n\n#","ref":"Dsv.html#module-example-validate-a-simple-value-with-multiple-validators"},{"type":"module","title":"Basic validators - Dsv","doc":"Here is a list of basic validators and their corresponding modules (the name on the left is also a name that need to be used in the `Dsv.validate/2` and `Dsv.valid?/2` functions):\n\n- length: `Dsv.Length`\n- number: `Dsv.Number`\n- format: `Dsv.Format`\n- any   : `Dsv.Any`\n- all   : `Dsv.All`\n- none  : `Dsv.None`\n- date  : `Dsv.Date`\n- in    : `Dsv.Inclusion`\n- not_in: `Dsv.Exclusion`\n- email : `Dsv.Email`\n- not_empty: `Dsv.NotEmpty`\n- equal: `Dsv.Equal`\n- custom: `Dsv.Custom`\n- position: `Dsv.At`\n- or: `Dsv.Or`\n- type: `Dsv.Type`\n\nFor detailed descriptions of each validator and their available options, refer to the respective validator's documentation.","ref":"Dsv.html#module-basic-validators"},{"type":"module","title":"Validating complex data structures - Dsv","doc":"The preferred way to validate complex data structures is by following these steps:\n1. Call `Dsv.validation/1` with the data to validate as the first argument.\n2. Add validators for each field by specifying the path in the second argument of `Dsv.add_validator/3` function and validators options in the third argument.\n3. Finally, run the validation by calling `Dsv.validate/1`\n\n\n#","ref":"Dsv.html#module-validating-complex-data-structures"},{"type":"module","title":"Example - validate nested maps with multiple validators. - Dsv","doc":"%{a: \"First value\", b: %{c: \"This is a nested map\", d: 10, e: [1, 2, 3]}}\n    |> Dsv.validation\n    |> Dsv.add_validator([:a], length: [min: 1, max: 20], format: ~r/.*e/)\n    |> Dsv.add_validator([:b, :d], number: [gt: 4])\n    |> Dsv.add_validator([:b, :e], position: [\"2\": [equal: 2]])\n    |> Dsv.validate\n\nYou can also validate the same data by specifying map with validators that reflects the structure of the validated data.\n\n#","ref":"Dsv.html#module-example-validate-nested-maps-with-multiple-validators"},{"type":"module","title":"Example - validate nested maps with multiple validators with validators as a map. - Dsv","doc":"Same validator as in the above example can be written in the form of a map that reflects the validated data structure.\n\n    %{a: \"First value\", b: %{c: \"This is a nested map\", d: 10, e: [1, 2, 3]}}\n    |> Dsv.validate(%{\n      :a => [length: [min: 1, max: 20], format: ~r/.*e/],\n      :b => %{\n        :d => [number: [gt: 4]],\n        :e => [position: [\"2\": [equal: 2]]]\n      }\n    })\n\n#","ref":"Dsv.html#module-example-validate-nested-maps-with-multiple-validators-with-validators-as-a-map"},{"type":"module","title":"Interchangeable approaches - Dsv","doc":"The two approaches mentioned above are interchangeable, and it's up to the user to choose which one to use.\n\nFor more ways of writing validators look at: [**How to validate**](how_to_validate.md)","ref":"Dsv.html#module-interchangeable-approaches"},{"type":"function","title":"Dsv.add_validator/3","doc":"Add a validator definition for the input data.","ref":"Dsv.html#add_validator/3"},{"type":"function","title":"Example - Dsv.add_validator/3","doc":"validators = %Validators{}\n    add_validator(validators, [:path, :to, :the, :validated, :field], length: [min: 1, max: 10], not_empty: :true)","ref":"Dsv.html#add_validator/3-example"},{"type":"function","title":"Dsv.add_validator/4","doc":"Add a validator definition for the input data, with a second path for value that will be used as an argument in the validator option.","ref":"Dsv.html#add_validator/4"},{"type":"function","title":"Example - Dsv.add_validator/4","doc":"validators = %Validators{}\n    add_validator(validators, [:path, :to, :the, :validated, :field], [:path, :to, :expected, :length], length: [min: 1, max: 10], not_empty: :true)","ref":"Dsv.html#add_validator/4-example"},{"type":"function","title":"Dsv.bind/3","doc":"Add binded value to the map of binded values.\nThose values can be used during the validation by calling `Dsv.binded/1` with the name provided in this function.","ref":"Dsv.html#bind/3"},{"type":"function","title":"Dsv.binded/1","doc":"Use binded value in the validator option.","ref":"Dsv.html#binded/1"},{"type":"function","title":"Dsv.paths/1","doc":"Transform a list of paths and validator descriptions into a map of validators.","ref":"Dsv.html#paths/1"},{"type":"function","title":"Example - Dsv.paths/1","doc":"iex> Dsv.paths([\n      [\"name\", length: [min: 1, max: 3], format: ~r/(B|b)r[a-z]{3,5}k/],\n      [\"age\", not_empty: true, number: [gt: 10, lt: 30]]\n    ])\n    %{\n      name: [length: [min: 1, max:3], format: ~r/(B|b)r[a-z]{3,5}k/],\n      age: [not_empty: true, number: [gt: 10, lt: 30]]\n    }","ref":"Dsv.html#paths/1-example"},{"type":"function","title":"Dsv.set_custom_message/2","doc":"Set a custom message for the validation.","ref":"Dsv.html#set_custom_message/2"},{"type":"function","title":"Dsv.valid?/1","doc":"","ref":"Dsv.html#valid?/1"},{"type":"function","title":"Dsv.valid?/2","doc":"Run all validators defined in the second argument agains data provided as the first argument.\n\n#","ref":"Dsv.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.valid?/2","doc":"* data (`data()`) - data provided by the user\n* options (`keyword()`) - validation options, describe the rules that will be checked against input `data`\n\n#","ref":"Dsv.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.valid?/2","doc":"* :true Represents successful validation, where all validation rules are met by the validated data.\n* :false is returned when any of the validation criteria are not met by the validated data.\n\n#","ref":"Dsv.html#valid?/2-returns"},{"type":"function","title":"Example - Dsv.valid?/2","doc":"Dsv.valid?(\"Simple string\", length: [min: 2, max: 20])\n    :true\n\n    Dsv.valid?(\"Simple string\", length: [min: 2, max: 10])\n    :false","ref":"Dsv.html#valid?/2-example"},{"type":"function","title":"Dsv.valid?/3","doc":"Same as `Dsv.valid?/2` but with additional argument containing binded values.\n\n#","ref":"Dsv.html#valid?/3"},{"type":"function","title":"Parameters - Dsv.valid?/3","doc":"* data (`data()`) - data provided by the user\n* options (`keyword()`) - validation options, describe the rules that will be checked against input `data`\n* binded_values (`map()`) - map containing names that can be used in validation options and paths from where value should be taken.\n\n#","ref":"Dsv.html#valid?/3-parameters"},{"type":"function","title":"Returns - Dsv.valid?/3","doc":"* :true Represents successful validation, where all validation rules are met by the validated data.\n* :false is returned when any of the validation criteria are not met by the validated data.","ref":"Dsv.html#valid?/3-returns"},{"type":"function","title":"Dsv.validate/1","doc":"Run validation based on the data in the `validators()` struct.\n\n#","ref":"Dsv.html#validate/1"},{"type":"function","title":"Parameters - Dsv.validate/1","doc":"* `validators()` - `%Validators{}` struct containing all the information needed to run validation.\n\n#","ref":"Dsv.html#validate/1-parameters"},{"type":"function","title":"Return `validation_result()` - Dsv.validate/1","doc":"* `:ok` - on successful validation, where all validation rules are met by the validated data.\n* `{:error, errrors()}` - on failure, when any fo the validation ciriteria are not met by the validated data.\n\n#","ref":"Dsv.html#validate/1-return-validation_result"},{"type":"function","title":"Example - Dsv.validate/1","doc":"%Validators{data: %{\"user\" => %{\"name\" => \"User name\"}}, validators: %{\"user\" => \"name\" => [length: [min: 2]]}} |> Dsv.validate()\n    :ok","ref":"Dsv.html#validate/1-example"},{"type":"function","title":"Dsv.validate/2","doc":"Validate input data according to the provided validation options.\n\n#","ref":"Dsv.html#validate/2"},{"type":"function","title":"Parameters - Dsv.validate/2","doc":"* `data()` - data to validate\n* `options()` - validation options\n\n#","ref":"Dsv.html#validate/2-parameters"},{"type":"function","title":"Returns `validation_result()` - Dsv.validate/2","doc":"* `:ok` - on successful validation, where all validation rules are met by the validated data.\n* `{:error, errrors()}` - on failure, when any fo the validation ciriteria are not met by the validated data.\n\n#","ref":"Dsv.html#validate/2-returns-validation_result"},{"type":"function","title":"Example - Dsv.validate/2","doc":"defmodule Person do\n      defstruct [:name, :age, :eyes]\n    end\n\n    person = %Person{\n      name: \"Martin\",\n      age: 30,\n      eyes: :blue\n    }\n\n    iex> Dsv.validate(person , %{name: [length: [min: 2, max: 20], format: ~r/[A-Z][0-9]+/]})\n    [\n      errors: %{\n        name: [error: \"Value Martin does not match pattern [A-Z][0-9]+\"]\n      }\n    ]\n\nAs in the example, validators can be joined to create a more complex check (in this case length and format).\n`Dsv.validate/2` allow to validate any kind of data from numbers and strings to lists, maps and structs.\n\nFor more information how to use `Dsv.validate/2` function go to [**How to validate**](how_to_validate.md)","ref":"Dsv.html#validate/2-example"},{"type":"function","title":"Dsv.validate/3","doc":"This function works like `validate/2` with an additional list of comparators that define validators using values from other input fields as their option values.","ref":"Dsv.html#validate/3"},{"type":"function","title":"Dsv.validate/4","doc":"Validate input data according to the provided validation options and return a custom message in case of failure.\n\n\n#","ref":"Dsv.html#validate/4"},{"type":"function","title":"Parameters - Dsv.validate/4","doc":"* data - data to validate\n* options - validation options\n* comparators - list of comparators to use in validators\n* message - custom message that will be return in case of failure\n\n#","ref":"Dsv.html#validate/4-parameters"},{"type":"function","title":"Returns `validation_result()` - Dsv.validate/4","doc":"* `:ok` - on successful validation, where all validation rules are met by the validated data.\n* `{:error, message}` - on failure, when any fo the validation ciriteria are not met by the validated data.\n\n#","ref":"Dsv.html#validate/4-returns-validation_result"},{"type":"function","title":"Example - Dsv.validate/4","doc":"defmodule Person do\n      defstruct [:name, :age, :eyes]\n    end\n\n    person = %Person{\n      name: \"Martin\",\n      age: 30,\n      eyes: :blue\n    }\n\n    iex> Dsv.validate(person , %{name: [length: [min: 2, max: 20], format: ~r/[A-Z][0-9]+/]}, [[:name], [:eyes], [:equal]], message: \"Something goes wrong! Fix your data and try again.\")\n    {:error, \"Something goes wrong! Fix your data and try again.\"}\n\n\nAs in the example, validators can be joined to create a more complex check (in this case length and format).\n`Dsv.validate/2` allow to validate any kind of data from numbers and strings to lists, maps and structs.\n\nFor more information how to use `Dsv.validate/2` function go to [**How to validate**](how_to_validate.md)","ref":"Dsv.html#validate/4-example"},{"type":"function","title":"Dsv.validation/0","doc":"","ref":"Dsv.html#validation/0"},{"type":"function","title":"Dsv.validation/1","doc":"Run validation for `%Validators{}`","ref":"Dsv.html#validation/1"},{"type":"type","title":"Dsv.data/0","doc":"Data to validate of any type.","ref":"Dsv.html#t:data/0"},{"type":"type","title":"Dsv.errors/0","doc":"A map of validation errors.","ref":"Dsv.html#t:errors/0"},{"type":"type","title":"Dsv.options/0","doc":"A keyword list or map representing validation options.\n\nOptions can be provided in one of the two forms:\n* `map()` - this form is intended for input data where input data is treated as multiple fields to validate.\n* `keyword()` - this for is intended for input data where input data is treated as one whole value to validate","ref":"Dsv.html#t:options/0"},{"type":"type","title":"Dsv.path/0","doc":"A non-empty list containing elements of the path for the field in the data to validate.\n\n#","ref":"Dsv.html#t:path/0"},{"type":"type","title":"Example - Dsv.path/0","doc":"%{\"user\" => %{\n        \"name\" => \"User name\", \"last_name\" => \"User last name\", \"age\" => 30, \"address\" => %{\n          \"street\" => \"Street\", \"postal_code\" => \"00900\"\n        }\n      }\n    }\n\nPath to the postal code field is the list of all keys in the map that carry on to the posta_code field. In this case the path will look like this:\n    [\"user\", \"address\", \"postal_code\"]","ref":"Dsv.html#t:path/0-example"},{"type":"type","title":"Dsv.validation_result/0","doc":"A result of the validation, which can be :ok for success or {:error, errors} for failure.","ref":"Dsv.html#t:validation_result/0"},{"type":"type","title":"Dsv.validators/0","doc":"A structure containing data for validation alongside the related validators.","ref":"Dsv.html#t:validators/0"},{"type":"type","title":"Fields - Dsv.validators/0","doc":"* data - input data to validate\n* validators - map containing paths to data in `data` field with related validators\n* comparators - list with comparators definitions\n* message - custom messaege returned in case of validation failure\n* binded_values - names of the binded values with path to those values","ref":"Dsv.html#t:validators/0-fields"},{"type":"module","title":"Dsv.Validators","doc":"","ref":"Dsv.Validators.html"},{"type":"module","title":"FieldComparator","doc":"Prepare validators to compare the input fields","ref":"FieldComparator.html"},{"type":"function","title":"FieldComparator.prepare/2","doc":"","ref":"FieldComparator.html#prepare/2"},{"type":"module","title":"ValidatorRegistry","doc":"","ref":"ValidatorRegistry.html"},{"type":"function","title":"ValidatorRegistry.get_validator/1","doc":"","ref":"ValidatorRegistry.html#get_validator/1"},{"type":"module","title":"Dsv.All","doc":"`Dsv.All` validator provides functions to validate that all elements in the given `List` or `String` meets all validation criteria.\n\n\n> ##","ref":"Dsv.All.html"},{"type":"module","title":"Validate other types of data {: .tip} - Dsv.All","doc":">\n> To validate arguments of other data types `FindAll` protocol must be implemented for this type.","ref":"Dsv.All.html#module-validate-other-types-of-data-tip"},{"type":"function","title":"Dsv.All.valid?/2","doc":"Ensure that all validators pass the check for the given data.\n\nIn the case of a string, this validator will iterate through all graphemes and check if each one passes the validation for all validators.\nIn the case of a list, this validator will iterate through all elements in the list and check if each one passes the validation for all validators.\n\nReturns `:true` if all validators pass for all elements; otherwise, returns `:false`.","ref":"Dsv.All.html#valid?/2"},{"type":"function","title":"Example - Dsv.All.valid?/2","doc":"iex> Dsv.All.valid?(\"abcd\", format: ~r/^[a-z]$/, equal: \"w\")\n    :false\n\n    iex> Dsv.All.valid?(\"abcd\", format: ~r/^[a-z]$/, equal: \"a\")\n    :false\n\n    iex> Dsv.All.valid?(\"abcd\", format: ~r/^[a-d]$/, custom: &is_bitstring/1)\n    :true\n\n    iex> Dsv.All.valid?(\"abcd\", format: ~r/^[a-c]$/, custom: &is_bitstring/1)\n    :false\n\n    iex> Dsv.All.valid?([1, 2, 3, \"hello\"], custom: &is_number/1, number: [gt: 2])\n    :false\n\n    iex> Dsv.All.valid?([1, 2, 3, \"hello\"], number: [gt: 0])\n    :false\n\n    iex> Dsv.All.valid?([1, 2, 3], custom: &is_number/1, number: [gte: 0, lte: 10])\n    :true","ref":"Dsv.All.html#valid?/2-example"},{"type":"function","title":"Dsv.All.valid?/3","doc":"","ref":"Dsv.All.html#valid?/3"},{"type":"function","title":"Dsv.All.validate/2","doc":"Ensure that all validators pass the check for the given data.\n\nIn the case of a string, this validator will iterate through all graphemes and check if each one passes the validation for all validators.\nIn the case of a list, this validator will iterate through all elements in the list and check if each one passes the validation for all validators.\n\nReturns `:ok` if all validators pass for all elements; otherwise, returns error tuple with the error message or custome message if provided.","ref":"Dsv.All.html#validate/2"},{"type":"function","title":"Options - Dsv.All.validate/2","doc":"* any validator name like\n    * `:format`\n    * `:custom`\n    * `:number`\n    * `:length`\n    * `:equal`\n    * `:data`\n  * `:message` - (EEx string) message used in error tuple in case of validation failure.","ref":"Dsv.All.html#validate/2-options"},{"type":"function","title":"Example - Dsv.All.validate/2","doc":"iex> Dsv.All.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"w\")\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.All.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"a\")\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.All.validate(\"abcd\", format: ~r/^[a-d]$/, custom: &is_bitstring/1)\n    :ok\n\n    iex> Dsv.All.validate(\"abcd\", format: ~r/^[a-c]$/, custom: &is_bitstring/1)\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.All.validate([1, 2, 3, \"hello\"], custom: &is_number/1, number: [gt: 2])\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.All.validate([1, 2, 3, \"hello\"], number: [gt: 0])\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.All.validate([1, 2, 3], custom: &is_number/1, number: [gte: 0, lte: 10])\n    :ok\n\n    iex> Dsv.All.validate([1, 2, 3, \"hello\"], number: [gt: 0], format: ~r/^[a-z]$/)\n    {:error, \"Not all elements match all validators.\"}","ref":"Dsv.All.html#validate/2-example"},{"type":"function","title":"Example with custom message - Dsv.All.validate/2","doc":"iex> Dsv.All.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"w\", message: \"All elements must be small letter 'w'.\")\n    {:error, \"All elements must be small letter 'w'.\"}\n\n    iex> Dsv.All.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"a\", message: \"All elements must be small letter 'a'.\")\n    {:error, \"All elements must be small letter 'a'.\"}\n\n    iex> Dsv.All.validate(\"abcd\", format: ~r/^[a-d]$/, custom: &is_bitstring/1, message: \"All elements must be one of the letter: 'a', 'b', 'c', 'd'.\")\n    :ok\n\n    iex> Dsv.All.validate(\"abcd\", format: ~r/^[a-c]$/, custom: &is_bitstring/1, message: \"All elements must be one of the letter: 'a', 'b', 'c'.\")\n    {:error, \"All elements must be one of the letter: 'a', 'b', 'c'.\"}\n\n    iex> Dsv.All.validate([1, 2, 3, \"hello\"], custom: &is_number/1, number: [gt: 2], message: \"All values must be a number greater that 2.\")\n    {:error, \"All values must be a number greater that 2.\"}\n\n    iex> Dsv.All.validate([1, 2, 3, \"hello\"], number: [gt: 0], message: \"All values must be a number greater that 0.\")\n    {:error, \"All values must be a number greater that 0.\"}\n\n    iex> Dsv.All.validate([1, 2, 3], custom: &is_number/1, number: [gte: 0, lte: 10], message: \"All values must be a number between 0 and 10.\")\n    :ok","ref":"Dsv.All.html#validate/2-example-with-custom-message"},{"type":"function","title":"Dsv.All.validate/3","doc":"","ref":"Dsv.All.html#validate/3"},{"type":"function","title":"Dsv.All.value/2","doc":"","ref":"Dsv.All.html#value/2"},{"type":"module","title":"Dsv.Any","doc":"Ensure that at least one element passes the check for all validators.\nIn the case of a string, this validator will iterate through all graphemes and check if at least one passes the validation for all validator.\nIn the case of a list, this validator will iterate through all elements and check if at least one passes the validation for all validators.\n\n\n> ##","ref":"Dsv.Any.html"},{"type":"module","title":"Validate other types of data {: .tip} - Dsv.Any","doc":">\n> To validate arguments of other data types `FindAany` protocol must be implemented for this type.","ref":"Dsv.Any.html#module-validate-other-types-of-data-tip"},{"type":"function","title":"Dsv.Any.valid?/2","doc":"Ensure that at least one element passes the check for all validators.\n\nIn the case of a string, this validator will iterate through all graphemes and check if at least one passes the validation for all validator.\nIn the case of a list, this validator will iterate through all elements and check if at least one passes the validation for all validators.\n\nReturns `:true` if all validators pass for at least one element; otherwise, returns `:false`.","ref":"Dsv.Any.html#valid?/2"},{"type":"function","title":"Example - Dsv.Any.valid?/2","doc":"iex> Dsv.Any.valid?(\"abcd\", format: ~r/^[a-z]$/, equal: \"w\")\n    :false\n\n    iex> Dsv.Any.valid?(\"abcd\", format: ~r/^[a-z]$/, equal: \"a\")\n    :true\n\n    iex> Dsv.Any.valid?(\"abcd\", format: ~r/^[k-z]$/, equal: \"a\")\n    :false\n\n    iex> Dsv.Any.valid?(\"abcd\", format: ~r/^[a-d]$/, custom: &is_bitstring/1)\n    :true\n\n    iex> Dsv.Any.valid?(\"abcd\", format: ~r/^[g-j]$/, custom: &is_bitstring/1)\n    :false\n\n    iex> Dsv.Any.valid?([1, 2, \"test\", 3], custom: &is_number/1, number: [gte: 3, lte: 5])\n    :true\n\n    iex> Dsv.Any.valid?([1, {:a, 1}, \"test\", 3], custom: &is_tuple/1, custom: &({:a, 1} == &1))\n    :true\n\n    iex> Dsv.Any.valid?([1, {:a, 2}, \"test\", 3], custom: &is_tuple/1, custom: &({:a, 1} == &1))\n    :false\n\n    iex> Dsv.Any.valid?([1, \"{:a, 1}\", \"test\", 3], custom: &is_tuple/1, custom: &({:a, 1} == &1))\n    :false\n\n    iex> Dsv.Any.valid?({1, 2, 3}, equal: 3)\n    :true","ref":"Dsv.Any.html#valid?/2-example"},{"type":"function","title":"Dsv.Any.valid?/3","doc":"","ref":"Dsv.Any.html#valid?/3"},{"type":"function","title":"Dsv.Any.validate/2","doc":"Ensure that at least one element passes the check for all validators.\n\nIn the case of a string, this validator will iterate through all graphemes and check if at least one passes the validation for all validator.\nIn the case of a list, this validator will iterate through all elements and check if at least one passes the validation for all validators.\n\nReturns `:ok` if all validators pass for at least one element; otherwise, returns error tuple with the error message.","ref":"Dsv.Any.html#validate/2"},{"type":"function","title":"Example - Dsv.Any.validate/2","doc":"iex> Dsv.Any.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"w\")\n    {:error, \"There is no element matching all validators.\"}\n\n    iex> Dsv.Any.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"a\")\n    :ok\n\n    iex> Dsv.Any.validate(\"abcd\", format: ~r/^[k-z]$/, equal: \"a\")\n    {:error, \"There is no element matching all validators.\"}\n\n    iex> Dsv.Any.validate(\"abcd\", format: ~r/^[a-d]$/, custom: &is_bitstring/1)\n    :ok\n\n    iex> Dsv.Any.validate(\"abcd\", format: ~r/^[g-j]$/, custom: &is_bitstring/1)\n    {:error, \"There is no element matching all validators.\"}\n\n    iex> Dsv.Any.validate([1, 2, \"test\", 3], custom: &is_number/1, number: [gte: 3, lte: 5])\n    :ok\n\n    iex> Dsv.Any.validate([1, {:a, 1}, \"test\", 3], custom: &is_tuple/1, custom: &({:a, 1} == &1))\n    :ok\n\n    iex> Dsv.Any.validate([1, {:a, 2}, \"test\", 3], custom: &is_tuple/1, custom: &({:a, 1} == &1))\n    {:error, \"There is no element matching all validators.\"}\n\n    iex> Dsv.Any.validate([1, \"{:a, 1}\", \"test\", 3], custom: &is_tuple/1, custom: &({:a, 1} == &1))\n    {:error, \"There is no element matching all validators.\"}\n\nTo return custom message, use :message key in the last argument.","ref":"Dsv.Any.html#validate/2-example"},{"type":"function","title":"Example - Dsv.Any.validate/2","doc":"iex> Dsv.Any.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"w\", message: \"At least one letter must be 'w'.\")\n    {:error, \"At least one letter must be 'w'.\"}\n\n    iex> Dsv.Any.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"a\", message: \"At least one letter must be 'a'.\")\n    :ok\n\n    iex> Dsv.Any.validate(\"abcd\", format: ~r/^[k-z]$/, equal: \"a\", message: \"At least one letter must be 'a' and any letter between k and z at the same moment. Sorry.\")\n    {:error, \"At least one letter must be 'a' and any letter between k and z at the same moment. Sorry.\"}\n\n    iex> Dsv.Any.validate(\"abcd\", format: ~r/^[a-d]$/, custom: &is_bitstring/1, message: \"At least one letter must be 'a', 'b', 'c' or 'd'.\")\n    :ok\n\n    iex> Dsv.Any.validate(\"abcd\", format: ~r/^[g-j]$/, custom: &is_bitstring/1, message: \"At least one letter must be 'g', 'h', 'i' or 'j'.\")\n    {:error, \"At least one letter must be 'g', 'h', 'i' or 'j'.\"}\n\n    iex> Dsv.Any.validate([1, 2, \"test\", 3], custom: &is_number/1, number: [gte: 3, lte: 5], message: \"At least one number must be between 3 and 5.\")\n    :ok\n\n    iex> Dsv.Any.validate([1, {:a, 1}, \"test\", 3], custom: &is_tuple/1, custom: &({:a, 1} == &1), message: \"At least on element must be a tuple `{:a, 1}`.\")\n    :ok\n\n    iex> Dsv.Any.validate([1, {:a, 2}, \"test\", 3], custom: &is_tuple/1, custom: &({:a, 1} == &1), message: \"At least on element must be a tuple `{:a, 1}`.\")\n    {:error, \"At least on element must be a tuple `{:a, 1}`.\"}\n\n    iex> Dsv.Any.validate([1, \"{:a, 1}\", \"test\", 3], custom: &is_tuple/1, custom: &({:a, 1} == &1), message: \"At least on element from the list `<%= inspect data %>` must be a tuple `{:a, 1}`.\")\n    {:error, ~s(At least on element from the list `[1, \"{:a, 1}\", \"test\", 3]` must be a tuple `{:a, 1}`.)}","ref":"Dsv.Any.html#validate/2-example"},{"type":"function","title":"Dsv.Any.validate/3","doc":"","ref":"Dsv.Any.html#validate/3"},{"type":"function","title":"Dsv.Any.value/2","doc":"","ref":"Dsv.Any.html#value/2"},{"type":"module","title":"Dsv.At","doc":"Run validator for an element at a given position.\n\n> ##","ref":"Dsv.At.html"},{"type":"module","title":"Validate other types of data {: .tip} - Dsv.At","doc":">\n> By default, String, and List are accepted, but other types can be added by implementing the `ValueAt` protocol for this type.\n\nTo validate the third letter in the string, run:\n\n    iex> Dsv.At.validate(\"string to validate\", \"2\", equal: \"r\")\n    :ok\n\nElements are numbered as in the elixir List, starting from 0.","ref":"Dsv.At.html#module-validate-other-types-of-data-tip"},{"type":"function","title":"Dsv.At.valid?/2","doc":"Run validators for elements on specified positions.\nGet data to validate as the first argument and keyword list as a second.\nKeys in second arguments are positions of elements to validates and values are validators to run for those elements.","ref":"Dsv.At.html#valid?/2"},{"type":"function","title":"Example - Dsv.At.valid?/2","doc":"iex> Dsv.At.valid?(\"abcd\", \"2\": [format: ~r/^[a-z]$/, equal: \"w\"])\n    :false\n\n    iex> Dsv.At.valid?(\"abcd\", \"2\": [format: ~r/^[a-z]$/, equal: \"w\"])\n    :false\n\n    iex> Dsv.At.valid?(\"abcd\", \"2\": [format: ~r/^[a-z]$/, equal: \"c\"])\n    :true\n\n    iex> Dsv.At.valid?(\"abcd\", \"2\": [format: ~r/^[k-z]$/, equal: \"a\"])\n    :false\n\n    iex> Dsv.At.valid?(\"abcd\", \"2\": [format: ~r/^[a-d]$/, custom: [function: &is_bitstring/1]])\n    :true\n\n    iex> Dsv.At.valid?(\"abcd\", \"2\": [format: ~r/^[g-j]$/, custom: [function: &is_bitstring/1]])\n    :false\n\n    iex> Dsv.At.valid?(\"abcd\", \"0\": [format: ~r/^[a-z]$/, equal: \"c\"], \"2\": [format: ~r/^[a-z]$/, equal: \"c\"], \"3\": [format: ~r/^[a-z]$/, equal: \"w\"])\n    :false\n\n    iex> Dsv.At.valid?(\"abcd\", \"0\": [format: ~r/^[a-z]$/, equal: \"c\"], \"2\": [format: ~r/^[a-z]$/, equal: \"c\"], \"3\": [format: ~r/^[a-z]$/, equal: \"w\"])\n    :false","ref":"Dsv.At.html#valid?/2-example"},{"type":"function","title":"Dsv.At.valid?/3","doc":"Run validator for an element at a given position.","ref":"Dsv.At.html#valid?/3"},{"type":"function","title":"Example - Dsv.At.valid?/3","doc":"Validator for string and list. Run all validators against element on provided position.\n\n    iex> Dsv.At.valid?(\"abcd\", \"2\", format: ~r/^[a-z]$/, equal: \"w\")\n    :false\n\n    iex> Dsv.At.valid?(\"abcd\", \"2\", format: ~r/^[a-z]$/, equal: \"c\")\n    :true\n\n    iex> Dsv.At.valid?(\"abcd\", \"2\", format: ~r/^[k-z]$/, equal: \"a\")\n    :false\n\n    iex> Dsv.At.valid?(\"abcd\", \"2\", format: ~r/^[a-d]$/, custom: [function: &is_bitstring/1])\n    :true\n\n    iex> Dsv.At.valid?(\"abcd\", \"2\", format: ~r/^[g-j]$/, custom: [function: &is_bitstring/1])\n    :false\n\n\n    iex> Dsv.At.valid?([\"test\", 1, ~D[2001-11-10]], \"2\", date: [min: ~D[1999-10-09], max: ~D[2020-10-10]])\n    :true\n\n    iex> Dsv.At.valid?([\"test\", 1, ~D[2001-11-10]], \"2\", date: [min: ~D[2002-10-09], max: ~D[2020-10-10]])\n    :false","ref":"Dsv.At.html#valid?/3-example"},{"type":"function","title":"Dsv.At.validate/2","doc":"Run validators for elements on specified positions.\nGet data to validate as the first argument and keyword list as a second.\nKeys in second arguments are positions of elements to validates and values are validators to run for those elements.","ref":"Dsv.At.html#validate/2"},{"type":"function","title":"Example - Dsv.At.validate/2","doc":"iex> Dsv.At.validate(\"abcd\", \"2\": [format: ~r/^[a-z]$/, equal: \"w\"])\n\n    iex> Dsv.At.validate(\"abcd\", \"2\": [format: ~r/^[a-z]$/, equal: \"w\"])\n    {:error, %{:\"2\" =>  [\"Values must be equal\"]}}\n\n    iex> Dsv.At.validate(\"abcd\", \"2\": [format: ~r/^[a-z]$/, equal: \"c\"])\n    :ok\n\n    iex> Dsv.At.validate(\"abcd\", \"2\": [format: ~r/^[k-z]$/, equal: \"a\"])\n    {:error, %{:\"2\" =>  [\"Value c does not match pattern ^[k-z]$\", \"Values must be equal\"]}}\n\n    iex> Dsv.At.validate(\"abcd\", \"2\": [format: ~r/^[a-d]$/, custom: [function: &is_bitstring/1, message: \"Data must be a bitstring\"]])\n    :ok\n\n    iex> Dsv.At.validate(\"abcd\", \"2\": [format: ~r/^[g-j]$/, custom: [function: &is_bitstring/1, message: \"Data must be a bitstring\"]])\n    {:error, %{:\"2\" =>  [\"Value c does not match pattern ^[g-j]$\"]}}\n\n    iex> Dsv.At.validate(\"abcd\", \"0\": [format: ~r/^[a-z]$/, equal: \"c\"], \"2\": [format: ~r/^[a-z]$/, equal: \"c\"], \"3\": [format: ~r/^[a-z]$/, equal: \"w\"])\n    {:error, %{:\"0\" =>   [\"Values must be equal\"], :\"3\" =>   [\"Values must be equal\"]}}\n\n    iex> Dsv.At.validate(\"abcd\",  \"0\": [format: ~r/^[a-z]$/, equal: \"c\"], \"2\": [format: ~r/^[a-z]$/, equal: \"c\"], \"3\": [format: ~r/^[a-z]$/, equal: \"w\"])\n    {:error, %{:\"0\" =>   [\"Values must be equal\"], :\"3\" =>   [\"Values must be equal\"]}}","ref":"Dsv.At.html#validate/2-example"},{"type":"function","title":"Dsv.At.validate/3","doc":"Run validator for an element at a given position.","ref":"Dsv.At.html#validate/3"},{"type":"function","title":"Example - Dsv.At.validate/3","doc":"Validator for string and list. Run all validators against element on provided position.\n\n    iex> Dsv.At.validate(\"abcd\", \"2\", format: ~r/^[a-z]$/, equal: \"w\")\n    {:error, %{:\"2\" =>  [\"Values must be equal\"]}}\n\n    iex> Dsv.At.validate(\"abcd\", \"2\", format: ~r/^[a-z]$/, equal: \"c\")\n    :ok\n\n    iex> Dsv.At.validate(\"abcd\", \"2\", format: ~r/^[k-z]$/, equal: \"a\")\n    {:error, %{:\"2\" =>  [\"Value c does not match pattern ^[k-z]$\", \"Values must be equal\"]}}\n\n    iex> Dsv.At.validate(\"abcd\", \"2\", format: ~r/^[a-d]$/, custom: [function: &is_bitstring/1, message: \"Data must be a bitstring\"])\n    :ok\n\n    iex> Dsv.At.validate(\"abcd\", \"2\", format: ~r/^[g-j]$/, custom: [function: &is_bitstring/1, message: \"Data must be a bitstring\"])\n    {:error, %{:\"2\" =>  [\"Value c does not match pattern ^[g-j]$\"]}}\n\n    iex> Dsv.At.validate([\"test\", 1, ~D[2001-11-10]], \"2\", date: [min: ~D[1999-10-09], max: ~D[2020-10-10]])\n    :ok\n\n    iex> Dsv.At.validate([\"test\", 1, ~D[2001-11-10]], \"2\", date: [min: ~D[2002-10-09], max: ~D[2020-10-10]])\n    {:error, %{:\"2\" =>  [\"Date must be between 2002-10-09 and 2020-10-10\"]}}\n\n    iex> Dsv.At.validate(\"abcd\", \"0\", format: ~r/^[a-z]$/, message: \"First letter must be lowercase.\")\n    :ok\n\n    iex> Dsv.At.validate(\"Abcd\", \"0\", format: ~r/^[a-z]$/, message: \"First letter must be lowercase.\")\n    {:error, \"First letter must be lowercase.\"}","ref":"Dsv.At.html#validate/3-example"},{"type":"function","title":"Dsv.At.value/2","doc":"","ref":"Dsv.At.html#value/2"},{"type":"module","title":"Dsv.Custom","doc":"Dsv.Custom module provides functions to validate data using a custom validation function.","ref":"Dsv.Custom.html"},{"type":"function","title":"Dsv.Custom.valid?/2","doc":"Validates the provided data using the given validation function.","ref":"Dsv.Custom.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.Custom.valid?/2","doc":"* `data` - The data value to validate.\n  * `func` - A validation function that takes the data as input and returns a boolean.\n    This function should return true if the data passes validation; otherwise, false.","ref":"Dsv.Custom.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.Custom.valid?/2","doc":"- `:true` if the data passes validation.\n- `:false` if the data fails validation.","ref":"Dsv.Custom.html#valid?/2-returns"},{"type":"function","title":"Examples - Dsv.Custom.valid?/2","doc":"iex> Dsv.Custom.valid?({:ok, \"hello\"}, function: fn data -> case data do\n    ...>   {:ok, _} -> :true\n    ...>   _ -> false\n    ...>  end\n    ...>  end)\n    :true\n\n    iex> Dsv.Custom.valid?({:error, \"hello\"}, function: fn data -> case data do\n    ...>   {:ok, _} -> :true\n    ...>   _ -> false\n    ...>  end\n    ...>  end)\n    :false\n\n    iex> Dsv.Custom.valid?(\"test\", &(&1 |> String.upcase() === \"TEST\"))\n    :true\n\n    iex> Dsv.Custom.valid?(\"tset\", &(&1 |> String.upcase() === \"TEST\"))\n    :false","ref":"Dsv.Custom.html#valid?/2-examples"},{"type":"function","title":"Dsv.Custom.valid?/3","doc":"","ref":"Dsv.Custom.html#valid?/3"},{"type":"function","title":"Dsv.Custom.validate/2","doc":"Validates the provided data using the given validation function.\n\n> ##","ref":"Dsv.Custom.html#validate/2"},{"type":"function","title":"Default error message {: .warning} - Dsv.Custom.validate/2","doc":">\n> This function will return \"Unknown validation error\" message in case of validation failure. Please consider using `Dsv.Custom.validate/3` with your own meaningful message.","ref":"Dsv.Custom.html#validate/2-default-error-message-warning"},{"type":"function","title":"Parameters - Dsv.Custom.validate/2","doc":"* `data` - The data value to validate.\n  * `options` - A validation function that takes the data as input and returns a boolean.\n    This function should return true if the data passes validation; otherwise, false.","ref":"Dsv.Custom.html#validate/2-parameters"},{"type":"function","title":"Returns - Dsv.Custom.validate/2","doc":"- `:ok` if the data passes validation.\n- `{:error, \"Unknown validation error\"}` if the data fails validation.\n\nTo get more significant message, use `Dsv.Custom.validate/3` function.","ref":"Dsv.Custom.html#validate/2-returns"},{"type":"function","title":"Examples - Dsv.Custom.validate/2","doc":"iex> Dsv.Custom.validate({:ok, \"hello\"}, function: fn data -> case data do\n    ...>   {:ok, _} -> :true\n    ...>   _ -> false\n    ...>  end\n    ...>  end)\n    :ok\n\n    iex> Dsv.Custom.validate({:error, \"hello\"}, function: fn data -> case data do\n    ...>   {:ok, _} -> :true\n    ...>   _ -> false\n    ...>  end\n    ...>  end)\n    {:error, \"Unknown validation error\"}","ref":"Dsv.Custom.html#validate/2-examples"},{"type":"function","title":"Dsv.Custom.validate/3","doc":"Validates the provided data using the given validation function and return a custom error message in case of validation failure.","ref":"Dsv.Custom.html#validate/3"},{"type":"function","title":"Parameters - Dsv.Custom.validate/3","doc":"* `data` (any) - The data value to be validated. This can encompass various data types such as strings,\n    numbers, lists, or more complex data structures.\n  * `function` (function) - A user-defined validation function that operates on the `data` parameter.\n    The validation function should return a boolean value, where `true` indicates successful validation\n    and `false` denotes validation failure.\n  * `message` (string) - A customized error message to be returned in the event of validation failure.","ref":"Dsv.Custom.html#validate/3-parameters"},{"type":"function","title":"Returns - Dsv.Custom.validate/3","doc":"- `:ok` - Represents a successful validation, where the data meets the criteria set by the validation function.\n- `{:error, message}` - Signifies validation failure, where the data does not fulfill the requirements\n  outlined by the validation function.","ref":"Dsv.Custom.html#validate/3-returns"},{"type":"function","title":"Examples - Dsv.Custom.validate/3","doc":"iex> Dsv.Custom.validate({:ok, \"hello\"}, function: fn data -> case data do\n    ...>   {:ok, _} -> :true\n    ...>   _ -> false\n    ...>  end\n    ...>  end, message: \"Tuple must contain :ok value\")\n    :ok\n\n    iex> Dsv.Custom.validate({:error, \"hello\"}, function: fn data -> case data do\n    ...>   {:ok, _} -> :true\n    ...>   _ -> false\n    ...>  end\n    ...>  end, message: \"Tuple must contain :ok value\")\n    {:error, \"Tuple must contain :ok value\"}\n\n\n    iex> Dsv.Custom.validate(\"test\", &(&1 |> String.upcase() === \"TEST\"), \"Value must be equal `TEST`\")\n    :ok\n\n    iex> Dsv.Custom.validate(\"tset\", &(&1 |> String.upcase() === \"TEST\"), \"Value must be equal `TEST`\")\n    {:error, \"Value must be equal `TEST`\"}","ref":"Dsv.Custom.html#validate/3-examples"},{"type":"function","title":"Dsv.Custom.value/2","doc":"","ref":"Dsv.Custom.html#value/2"},{"type":"module","title":"Dsv.Date","doc":"Dsv.Date module provides functions to validate a date value based on the specified rules (:min, :max, :range).","ref":"Dsv.Date.html"},{"type":"module","title":"Possible options are: - Dsv.Date","doc":"* `:min` - check if the date is later or the same as the date specified in the `:min` option.\n* `:max` - check if the date is earlier or the same as the date specified in the `:max` option.\n* `:range` - check if the date is between the dates specified in the `:range` options.","ref":"Dsv.Date.html#module-possible-options-are"},{"type":"function","title":"Dsv.Date.valid?/2","doc":"The `Dsv.Date.valid?/2` function is designed to validate a given date against a set of options provided by the user.\n\nIt checks whether the date falls within the specified range or exceeds certain boundaries defined by the min and max dates.\nIf the date meets all of the specified conditions, the function returns an `:true` response; otherwise, it returns `:false`.","ref":"Dsv.Date.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.Date.valid?/2","doc":"* `date` - The date value to validate.\n  * `rules` - A keyword list of validation rules.\n    Each rule is a tuple containing:\n    * The validation type (`:min`, `:max`, `:range`).\n    * The corresponding value or values for the validation type.","ref":"Dsv.Date.html#valid?/2-parameters"},{"type":"function","title":"Return - Dsv.Date.valid?/2","doc":"* `:true` - If the date is later than the min date provided in options, earlier than the max date provided in options, or falls between the dates specified in the range.\n  * `:false` - If the date does not meet any of the specified conditions.\n\n> ##","ref":"Dsv.Date.html#valid?/2-return"},{"type":"function","title":"min and max options {: .info} - Dsv.Date.valid?/2","doc":">\n> `:min` and `:max` are inclusive","ref":"Dsv.Date.html#valid?/2-min-and-max-options-info"},{"type":"function","title":"Dsv.Date.valid?/3","doc":"","ref":"Dsv.Date.html#valid?/3"},{"type":"function","title":"Dsv.Date.validate/2","doc":"The `Dsv.Date.validate/2` function is designed to validate a given date against a set of options provided by the user.\n\nIt checks whether the date falls within the specified range or exceeds certain boundaries defined by the min and max dates.\nIf the date meets all of the specified conditions, the function returns an `:ok` response; otherwise, it returns an appropriate error message.","ref":"Dsv.Date.html#validate/2"},{"type":"function","title":"Parameters - Dsv.Date.validate/2","doc":"* `date` - The date value to validate.\n  * `rules` - A keyword list of validation rules.\n    Each rule is a tuple containing:\n    * The validation type (`:min`, `:max`, `:range`).\n    * The corresponding value or values for the validation type.","ref":"Dsv.Date.html#validate/2-parameters"},{"type":"function","title":"Return - Dsv.Date.validate/2","doc":"* `:ok` - If the date is later than the min date provided in options, earlier than the max date provided in options, or falls between the dates specified in the range.\n  * `{:error, message}` - If the date does not meet any of the specified conditions, where message is an appropriate string message.\n\n> ##","ref":"Dsv.Date.html#validate/2-return"},{"type":"function","title":"min and max options {: .info} - Dsv.Date.validate/2","doc":">\n> `:min` and `:max` are inclusive\n\nExamples for different types:\n* [Date](#validate/2-examples-date)\n  * [min](#validate/2-min)\n  * [max](#validate/2-max)\n  * [min and max](#validate/2-min-max)\n  * [range](#validate/2-range)\n* [DateTime](#validate/2-examples-datetime)\n  * [min](#validate/2-min-1)\n  * [max](#validate/2-max-1)\n  * [min and max](#validate/2-min-max-1)\n  * [range](#validate/2-range-1)\n* [Time](#validate/2-examples-time)\n  * [min](#validate/2-min-2)\n  * [max](#validate/2-max-2)\n  * [min and max](#validate/2-min-max-2)\n  * [range](#validate/2-range-2)","ref":"Dsv.Date.html#validate/2-min-and-max-options-info"},{"type":"function","title":"Examples - date - Dsv.Date.validate/2","doc":"#","ref":"Dsv.Date.html#validate/2-examples-date"},{"type":"function","title":"min - Dsv.Date.validate/2","doc":"`:min` example with first parameter as a string:\n    iex> Dsv.Date.validate(\"2020-10-10\", min: ~D[2020-10-09])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10\", min: ~D[2020-10-10])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10\", min: ~D[2020-10-11])\n    {:error, \"Date is earlier than the minimum allowed date 2020-10-11\"}\n\n\n`:min` example with first parameter as a date:\n    iex> Dsv.Date.validate(~D[2020-10-10], min: ~D[2020-10-09])\n    :ok\n\n    iex> Dsv.Date.validate(~D[2020-10-10], min: ~D[2020-10-10])\n    :ok\n\n    iex> Dsv.Date.validate(~D[2020-10-10], min: ~D[2020-10-11])\n    {:error, \"Date is earlier than the minimum allowed date 2020-10-11\"}\n\n#","ref":"Dsv.Date.html#validate/2-min"},{"type":"function","title":"max - Dsv.Date.validate/2","doc":"`:max` example with first parameter as a string:\n    iex> Dsv.Date.validate(\"2020-10-10\", max: ~D[2020-10-11])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10\", max: ~D[2020-10-10])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10\", max: ~D[2020-10-09])\n    {:error, \"Date is later than the maximum allowed date 2020-10-09\"}\n\n`:max` example with first parameter as a date:\n    iex> Dsv.Date.validate(~D[2020-10-10], max: ~D[2020-10-11])\n    :ok\n\n    iex> Dsv.Date.validate(~D[2020-10-10], max: ~D[2020-10-10])\n    :ok\n\n    iex> Dsv.Date.validate(~D[2020-10-10], max: ~D[2020-10-09])\n    {:error, \"Date is later than the maximum allowed date 2020-10-09\"}\n\n#","ref":"Dsv.Date.html#validate/2-max"},{"type":"function","title":"min & max - Dsv.Date.validate/2","doc":"`:min & :max` example:\n    iex> Dsv.Date.validate(\"2020-10-10\", min: ~D[2020-10-09], max: ~D[2020-10-10])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-09\", min: ~D[2020-10-09], max: ~D[2020-10-10])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10\", min: ~D[2020-10-09], max: ~D[2020-10-11])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-07\", min: ~D[2020-10-08], max: ~D[2020-10-09])\n    {:error, \"Date must be between 2020-10-08 and 2020-10-09\"}\n\n    iex> Dsv.Date.validate(\"2020-10-10\", min: ~D[2020-10-08], max: ~D[2020-10-09])\n    {:error, \"Date must be between 2020-10-08 and 2020-10-09\"}\n\n    iex> Dsv.Date.validate(\"2020-10-10\", min: ~D[2020-10-10], max: ~D[2020-10-09])\n    ** (RuntimeError) Min date (2020-10-10) can't be after max date (2020-10-09)\n\n    iex> Dsv.Date.validate(~D[2020-10-10], min: ~D[2020-10-09], max: ~D[2020-10-10])\n    :ok\n\n    iex> Dsv.Date.validate(~D[2020-10-09], min: ~D[2020-10-09], max: ~D[2020-10-10])\n    :ok\n\n    iex> Dsv.Date.validate(~D[2020-10-10], min: ~D[2020-10-09], max: ~D[2020-10-11])\n    :ok\n\n    iex> Dsv.Date.validate(~D[2020-10-07], min: ~D[2020-10-08], max: ~D[2020-10-09])\n    {:error, \"Date must be between 2020-10-08 and 2020-10-09\"}\n\n    iex> Dsv.Date.validate(~D[2020-10-10], min: ~D[2020-10-08], max: ~D[2020-10-09])\n    {:error, \"Date must be between 2020-10-08 and 2020-10-09\"}\n\n    iex> Dsv.Date.validate(~D[2020-10-10], min: ~D[2020-10-10], max: ~D[2020-10-09])\n    ** (RuntimeError) Min date (2020-10-10) can't be after max date (2020-10-09)\n\n#","ref":"Dsv.Date.html#validate/2-min-max"},{"type":"function","title":"range - Dsv.Date.validate/2","doc":"`:range` example:\n    iex> Dsv.Date.validate(\"2020-10-10\", range: {~D[2020-10-09], ~D[2020-10-10]})\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-09\", range: {~D[2020-10-09], ~D[2020-10-10]})\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10\", range: {~D[2020-10-09], ~D[2020-10-11]})\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-07\", range: {~D[2020-10-08], ~D[2020-10-09]})\n    {:error, \"Date must be between 2020-10-08 and 2020-10-09\"}\n\n    iex> Dsv.Date.validate(\"2020-10-10\", range: {~D[2020-10-08], ~D[2020-10-09]})\n    {:error, \"Date must be between 2020-10-08 and 2020-10-09\"}\n\n    iex> Dsv.Date.validate(\"2020-10-10\", range: {~D[2020-10-10], ~D[2020-10-09]})\n    ** (RuntimeError) Min date (2020-10-10) can't be after max date (2020-10-09)\n\n    iex> Dsv.Date.validate(~D[2020-10-10], range: {~D[2020-10-09], ~D[2020-10-10]})\n    :ok\n\n    iex> Dsv.Date.validate(~D[2020-10-09], range: {~D[2020-10-09], ~D[2020-10-10]})\n    :ok\n\n    iex> Dsv.Date.validate(~D[2020-10-10], range: {~D[2020-10-09], ~D[2020-10-11]})\n    :ok\n\n    iex> Dsv.Date.validate(~D[2020-10-07], range: {~D[2020-10-08], ~D[2020-10-09]})\n    {:error, \"Date must be between 2020-10-08 and 2020-10-09\"}\n\n    iex> Dsv.Date.validate(~D[2020-10-10], range: {~D[2020-10-08], ~D[2020-10-09]})\n    {:error, \"Date must be between 2020-10-08 and 2020-10-09\"}\n\n    iex> Dsv.Date.validate(~D[2020-10-10], range: {~D[2020-10-10], ~D[2020-10-09]})\n    ** (RuntimeError) Min date (2020-10-10) can't be after max date (2020-10-09)","ref":"Dsv.Date.html#validate/2-range"},{"type":"function","title":"Examples DateTime - Dsv.Date.validate/2","doc":"#","ref":"Dsv.Date.html#validate/2-examples-datetime"},{"type":"function","title":"min - Dsv.Date.validate/2","doc":"`:min` example:\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:59Z\", min: ~U[2020-10-10 23:59:58Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:59Z\", min: ~U[2020-10-10 23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:59Z\", min: ~U[2020-10-11 00:00:00Z])\n    {:error, \"Date is earlier than the minimum allowed date 2020-10-11 00:00:00Z\"}\n\n    iex> Dsv.Date.validate(~U[2020-10-10T23:59:59Z], min: ~U[2020-10-10 23:59:58Z])\n    :ok\n\n    iex> Dsv.Date.validate(~U[2020-10-10T23:59:59Z], min: ~U[2020-10-10 23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~U[2020-10-10T23:59:59Z], min: ~U[2020-10-11 00:00:00Z])\n    {:error, \"Date is earlier than the minimum allowed date 2020-10-11 00:00:00Z\"}\n\n#","ref":"Dsv.Date.html#validate/2-min"},{"type":"function","title":"max - Dsv.Date.validate/2","doc":"`:max` example:\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:59Z\", max: ~U[2020-10-11 00:00:00Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:59Z\", max: ~U[2020-10-10 23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:59Z\", max: ~U[2020-10-10 23:59:58Z])\n    {:error, \"Date is later than the maximum allowed date 2020-10-10 23:59:58Z\"}\n\n    iex> Dsv.Date.validate(~U[2020-10-10T23:59:59Z], max: ~U[2020-10-11 00:00:00Z])\n    :ok\n\n    iex> Dsv.Date.validate(~U[2020-10-10T23:59:59Z], max: ~U[2020-10-10 23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~U[2020-10-10T23:59:59Z], max: ~U[2020-10-10 23:59:58Z])\n    {:error, \"Date is later than the maximum allowed date 2020-10-10 23:59:58Z\"}\n\n#","ref":"Dsv.Date.html#validate/2-max"},{"type":"function","title":"min & max - Dsv.Date.validate/2","doc":"`:min & :max` example:\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:58Z\", min: ~U[2020-10-10 23:59:58Z], max: ~U[2020-10-10 23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:59Z\", min: ~U[2020-10-10 23:59:58Z], max: ~U[2020-10-10 23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:58Z\", min: ~U[2020-10-10 23:59:57Z], max: ~U[2020-10-10 23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:57Z\", min: ~U[2020-10-10 23:59:58Z], max: ~U[2020-10-10 23:59:59Z])\n    {:error, \"Date must be between 2020-10-10 23:59:58Z and 2020-10-10 23:59:59Z\"}\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:59Z\", min: ~U[2020-10-10 23:59:57Z], max: ~U[2020-10-10 23:59:58Z])\n    {:error, \"Date must be between 2020-10-10 23:59:57Z and 2020-10-10 23:59:58Z\"}\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:58Z\", min: ~U[2020-10-10 23:59:59Z], max: ~U[2020-10-10 23:59:58Z])\n    ** (RuntimeError) Min date (2020-10-10 23:59:59Z) can't be after max date (2020-10-10 23:59:58Z)\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:58Z], min: ~U[2020-10-10 23:59:58Z], max: ~U[2020-10-10 23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:59Z], min: ~U[2020-10-10 23:59:58Z], max: ~U[2020-10-10 23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:58Z], min: ~U[2020-10-10 23:59:57Z], max: ~U[2020-10-10 23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:57Z], min: ~U[2020-10-10 23:59:58Z], max: ~U[2020-10-10 23:59:59Z])\n    {:error, \"Date must be between 2020-10-10 23:59:58Z and 2020-10-10 23:59:59Z\"}\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:59Z], min: ~U[2020-10-10 23:59:57Z], max: ~U[2020-10-10 23:59:58Z])\n    {:error, \"Date must be between 2020-10-10 23:59:57Z and 2020-10-10 23:59:58Z\"}\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:58Z], min: ~U[2020-10-10 23:59:59Z], max: ~U[2020-10-10 23:59:58Z])\n    ** (RuntimeError) Min date (2020-10-10 23:59:59Z) can't be after max date (2020-10-10 23:59:58Z)\n\n#","ref":"Dsv.Date.html#validate/2-min-max"},{"type":"function","title":"range - Dsv.Date.validate/2","doc":"`:range` example:\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:58Z\", range: {~U[2020-10-10 23:59:58Z], ~U[2020-10-10 23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:59Z\", range: {~U[2020-10-10 23:59:58Z], ~U[2020-10-10 23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:58Z\", range: {~U[2020-10-10 23:59:57Z], ~U[2020-10-10 23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:57Z\", range: {~U[2020-10-10 23:59:58Z], ~U[2020-10-10 23:59:59Z]})\n    {:error, \"Date must be between 2020-10-10 23:59:58Z and 2020-10-10 23:59:59Z\"}\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:59Z\", range: {~U[2020-10-10 23:59:57Z], ~U[2020-10-10 23:59:58Z]})\n    {:error, \"Date must be between 2020-10-10 23:59:57Z and 2020-10-10 23:59:58Z\"}\n\n    iex> Dsv.Date.validate(\"2020-10-10T23:59:58Z\", range: {~U[2020-10-10 23:59:59Z], ~U[2020-10-10 23:59:58Z]})\n    ** (RuntimeError) Min date (2020-10-10 23:59:59Z) can't be after max date (2020-10-10 23:59:58Z)\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:58Z], range: {~U[2020-10-10 23:59:58Z], ~U[2020-10-10 23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:59Z], range: {~U[2020-10-10 23:59:58Z], ~U[2020-10-10 23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:58Z], range: {~U[2020-10-10 23:59:57Z], ~U[2020-10-10 23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:57Z], range: {~U[2020-10-10 23:59:58Z], ~U[2020-10-10 23:59:59Z]})\n    {:error, \"Date must be between 2020-10-10 23:59:58Z and 2020-10-10 23:59:59Z\"}\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:59Z], range: {~U[2020-10-10 23:59:57Z], ~U[2020-10-10 23:59:58Z]})\n    {:error, \"Date must be between 2020-10-10 23:59:57Z and 2020-10-10 23:59:58Z\"}\n\n    iex> Dsv.Date.validate(~U[2020-10-10 23:59:58Z], [range: {~U[2020-10-10 23:59:59Z], ~U[2020-10-10 23:59:58Z]}])\n    ** (RuntimeError) Min date (2020-10-10 23:59:59Z) can't be after max date (2020-10-10 23:59:58Z)","ref":"Dsv.Date.html#validate/2-range"},{"type":"function","title":"Examples Time - Dsv.Date.validate/2","doc":"#","ref":"Dsv.Date.html#validate/2-examples-time"},{"type":"function","title":"min - Dsv.Date.validate/2","doc":"`:min` example:\n    iex> Dsv.Date.validate(\"23:59:59Z\", min: ~T[23:59:58Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"23:59:59Z\", min: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"23:59:58Z\", min: ~T[23:59:59Z])\n    {:error, \"Date is earlier than the minimum allowed date 23:59:59\"}\n\n    iex> Dsv.Date.validate(~T[23:59:59Z], min: ~T[23:59:58Z])\n    :ok\n\n    iex> Dsv.Date.validate(~T[23:59:59Z], min: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~T[23:59:58Z], min: ~T[23:59:59Z])\n    {:error, \"Date is earlier than the minimum allowed date 23:59:59\"}\n\n#","ref":"Dsv.Date.html#validate/2-min"},{"type":"function","title":"max - Dsv.Date.validate/2","doc":"`:max` example:\n    iex> Dsv.Date.validate(\"23:59:58Z\", max: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"23:59:59Z\", max: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"23:59:59Z\", max: ~T[23:59:58Z])\n    {:error, \"Date is later than the maximum allowed date 23:59:58\"}\n\n    iex> Dsv.Date.validate(~T[23:59:58Z], max: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~T[23:59:59Z], max: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~T[23:59:59Z], max: ~T[23:59:58Z])\n    {:error, \"Date is later than the maximum allowed date 23:59:58\"}\n\n#","ref":"Dsv.Date.html#validate/2-max"},{"type":"function","title":"min & max - Dsv.Date.validate/2","doc":"`:min & :max` example:\n    iex> Dsv.Date.validate(\"23:59:58Z\", min: ~T[23:59:58Z], max: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"23:59:59Z\", min: ~T[23:59:58Z], max: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"23:59:58Z\", min: ~T[23:59:57Z], max: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(\"23:59:57Z\", min: ~T[23:59:58Z], max: ~T[23:59:59Z])\n    {:error, \"Date must be between 23:59:58 and 23:59:59\"}\n\n    iex> Dsv.Date.validate(\"23:59:59Z\", min: ~T[23:59:57Z], max: ~T[23:59:58Z])\n    {:error, \"Date must be between 23:59:57 and 23:59:58\"}\n\n    iex> Dsv.Date.validate(\"23:59:58Z\", min: ~T[23:59:59Z], max: ~T[23:59:58Z])\n    ** (RuntimeError) Min date (23:59:59) can't be after max date (23:59:58)\n\n    iex> Dsv.Date.validate(~T[23:59:58Z], min: ~T[23:59:58Z], max: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~T[23:59:59Z], min: ~T[23:59:58Z], max: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~T[23:59:58Z], min: ~T[23:59:57Z], max: ~T[23:59:59Z])\n    :ok\n\n    iex> Dsv.Date.validate(~T[23:59:57Z], min: ~T[23:59:58Z], max: ~T[23:59:59Z])\n    {:error, \"Date must be between 23:59:58 and 23:59:59\"}\n\n    iex> Dsv.Date.validate(~T[23:59:59Z], min: ~T[23:59:57Z], max: ~T[23:59:58Z])\n    {:error, \"Date must be between 23:59:57 and 23:59:58\"}\n\n    iex> Dsv.Date.validate(~T[23:59:58Z], min: ~T[23:59:59Z], max: ~T[23:59:58Z])\n    ** (RuntimeError) Min date (23:59:59) can't be after max date (23:59:58)\n\n#","ref":"Dsv.Date.html#validate/2-min-max"},{"type":"function","title":"range - Dsv.Date.validate/2","doc":"`:range` example:\n    iex> Dsv.Date.validate(\"23:59:58Z\", range: {~T[23:59:58Z], ~T[23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(\"23:59:59Z\", range: {~T[23:59:58Z], ~T[23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(\"23:59:58Z\", range: {~T[23:59:57Z], ~T[23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(\"23:59:57Z\", range: {~T[23:59:58Z], ~T[23:59:59Z]})\n    {:error, \"Date must be between 23:59:58 and 23:59:59\"}\n\n    iex> Dsv.Date.validate(\"23:59:59Z\", range: {~T[23:59:57Z], ~T[23:59:58Z]})\n    {:error, \"Date must be between 23:59:57 and 23:59:58\"}\n\n    iex> Dsv.Date.validate(\"23:59:58Z\", range: {~T[23:59:59Z], ~T[23:59:58Z]})\n    ** (RuntimeError) Min date (23:59:59) can't be after max date (23:59:58)\n\n    iex> Dsv.Date.validate(~T[23:59:58Z], range: {~T[23:59:58Z], ~T[23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(~T[23:59:59Z], range: {~T[23:59:58Z], ~T[23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(~T[23:59:58Z], range: {~T[23:59:57Z], ~T[23:59:59Z]})\n    :ok\n\n    iex> Dsv.Date.validate(~T[23:59:57Z], range: {~T[23:59:58Z], ~T[23:59:59Z]})\n    {:error, \"Date must be between 23:59:58 and 23:59:59\"}\n\n    iex> Dsv.Date.validate(~T[23:59:59Z], range: {~T[23:59:57Z], ~T[23:59:58Z]})\n    {:error, \"Date must be between 23:59:57 and 23:59:58\"}\n\n    iex> Dsv.Date.validate(~T[23:59:58Z], range: {~T[23:59:59Z], ~T[23:59:58Z]})\n    ** (RuntimeError) Min date (23:59:59) can't be after max date (23:59:58)","ref":"Dsv.Date.html#validate/2-range"},{"type":"function","title":"Dsv.Date.validate/3","doc":"","ref":"Dsv.Date.html#validate/3"},{"type":"function","title":"Dsv.Date.value/2","doc":"","ref":"Dsv.Date.html#value/2"},{"type":"module","title":"Dsv.Email","doc":"Dsv.Email module offers functions to validate an email address and optionally checks specific parts of it.\n\n> ##","ref":"Dsv.Email.html"},{"type":"module","title":"Notes: {: .info} - Dsv.Email","doc":">  This function uses basic email format validation and does not guarantee\n>  that the email address is deliverable or exists.","ref":"Dsv.Email.html#module-notes-info"},{"type":"function","title":"Dsv.Email.valid?/2","doc":"The `valid?/2` function provides a robust mechanism to validate an email address\nwhile applying a set of associated validators based on provided criteria.","ref":"Dsv.Email.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.Email.valid?/2","doc":"* `email` (string) - The email address to be validated.\n  * `options` (keyword list) - A keyword list containing validation criteria for specific parts of the email.\n    Valid keys include:\n    - `:username` - Validators for the username part of the email (before \"@\").\n    - `:mail_server` - Validators for the mail server part of the email (between \"@\" and the last dot).\n    - `:top_level_domain` - Validators for the top-level domain part of the email (after the last dot).","ref":"Dsv.Email.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.Email.valid?/2","doc":"- `:true` Represents successful validation, where the email is correct and meets all the specified criteria.\n- `:false` if the data (email) fails validation.","ref":"Dsv.Email.html#valid?/2-returns"},{"type":"function","title":"Example - Dsv.Email.valid?/2","doc":"iex> Dsv.Email.valid?(\"mail.test@domain.com\")\n    :true\n\n    iex> Dsv.Email.valid?(\"not-existing-email-address@domain.com\")\n    :true\n\n    iex> Dsv.Email.valid?(\"this is not email address\")\n    :false\n\n    iex> Dsv.Email.valid?(\"mail.test@domain.com\", top_level_domain: [equal: \"domain\"], username: [format: ~r/m.*t.*/])\n    :false\n\n    iex> Dsv.Email.valid?(\"not-existing-email-address@domain.com\", top_level_domain: [equal: \"domain\"], username: [format: ~r/m.*t.*/])\n    :false\n\n    iex> Dsv.Email.valid?(\"mail.test@domain.com\", [])\n    :true\n\n    iex> Dsv.Email.valid?(\"this_is_not_real_email_address.com\", top_level_domain: [equal: \"com\"])\n    :false","ref":"Dsv.Email.html#valid?/2-example"},{"type":"function","title":"Dsv.Email.valid?/3","doc":"","ref":"Dsv.Email.html#valid?/3"},{"type":"function","title":"Dsv.Email.validate/2","doc":"The `validate/2` function provides a robust mechanism to validate an email address\nwhile applying a set of associated validators based on provided criteria.","ref":"Dsv.Email.html#validate/2"},{"type":"function","title":"Parameters - Dsv.Email.validate/2","doc":"* `email` (string) - The email address to be validated.\n  * `options` (keyword list) - A keyword list containing validation criteria for specific parts of the email.\n    Valid keys include:\n    - `:username` - Validators for the username part of the email (before \"@\").\n    - `:mail_server` - Validators for the mail server part of the email (between \"@\" and the last dot).\n    - `:top_level_domain` - Validators for the top-level domain part of the email (after the last dot).\n    - `:message` - Custom message returned in case of the validation failure.","ref":"Dsv.Email.html#validate/2-parameters"},{"type":"function","title":"Returns - Dsv.Email.validate/2","doc":"- `:ok` - Represents successful validation, where the email is correct and meets all the specified criteria.\n- `{:error, errors}` - if the data (email) fails validation. The `errors` map provides detailed information about each part of the email that did not pass validation.","ref":"Dsv.Email.html#validate/2-returns"},{"type":"function","title":"Example - Dsv.Email.validate/2","doc":"iex> Dsv.Email.validate(\"mail.test@domain.com\")\n    :ok\n\n    iex> Dsv.Email.validate(\"not-existing-email-address@domain.com\")\n    :ok\n\n    iex> Dsv.Email.validate(\"this is not email address\")\n    {:error, \"Invalid email address.\"}\n\n    iex> Dsv.Email.validate(\"mail.test@domain.com\", top_level_domain: [equal: \"domain\"], username: [format: ~r/m.*t.*/])\n    {:error, %{top_level_domain: [\"Values must be equal\"]}}\n\n    iex> Dsv.Email.validate(\"not-existing-email-address@domain.com\", top_level_domain: [equal: \"domain\"], username: [format: ~r/m.*t.*/])\n    {:error, %{top_level_domain: [\"Values must be equal\"], username: [\"Value not-existing-email-address does not match pattern m.*t.*\"]}}\n\n    iex> Dsv.Email.validate(\"mail.test@domain.com\", [])\n    :ok\n\n    iex> Dsv.Email.validate(\"this_is_not_real_email_address.com\", top_level_domain: [equal: \"com\"])\n    {:error, \"Invalid email address.\"}\n\n    iex> Dsv.Email.validate(\"mail.test@domain.com\", top_level_domain: [equal: \"domain\"], username: [format: ~r/m.*t.*/], message: \"Email address don't meet validation criteria.\")\n    {:error, \"Email address don't meet validation criteria.\"}","ref":"Dsv.Email.html#validate/2-example"},{"type":"function","title":"Dsv.Email.validate/3","doc":"","ref":"Dsv.Email.html#validate/3"},{"type":"function","title":"Dsv.Email.value/2","doc":"","ref":"Dsv.Email.html#value/2"},{"type":"module","title":"Dsv.Equal","doc":"Dsv.Equal module provides functions to determine if two values are equal.","ref":"Dsv.Equal.html"},{"type":"function","title":"Dsv.Equal.valid?/2","doc":"The `valid?/2` function is designed to evaluate whether two given values are equal.\nThis function takes two parameters and compares them to determine if they are equal.\nCheck if the two values are equal.","ref":"Dsv.Equal.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.Equal.valid?/2","doc":"* `data` - The first value to be compared.\n  * `options` - The second value to be compared against the first.","ref":"Dsv.Equal.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.Equal.valid?/2","doc":"A boolean value:\n\n- `true` if `data` is equal to `options`.\n- `false` if `data` is not equal to `options`.","ref":"Dsv.Equal.html#valid?/2-returns"},{"type":"function","title":"Example - Dsv.Equal.valid?/2","doc":"iex> Dsv.Equal.valid?(\"text\", :text)\n    :false\n\n    iex> Dsv.Equal.valid?([], [\"\"])\n    :false\n\n    iex> Dsv.Equal.valid?(\"a\", \"ab\")\n    :false\n\n    iex> Dsv.Equal.valid?(nil, \"\")\n    :false\n\n    iex> Dsv.Equal.valid?(%{}, %{\"a\" => \"b\"})\n    :false\n\n    iex> Dsv.Equal.valid?(%{}, [])\n    :false\n\n    iex> Dsv.Equal.valid?(%{}, true)\n    :false\n\n    iex> Dsv.Equal.valid?([\"elem\"], [\"elem\"])\n    :true\n\n    iex> Dsv.Equal.valid?(\"text\", \"text\")\n    :true\n\n    iex> Dsv.Equal.valid?(36, 36)\n    :true\n\n    iex> Dsv.Equal.valid?(%{a: :b}, %{a: :b})\n    :true\n\n    iex> Dsv.Equal.valid?(~N[2020-10-11 02:12:33], ~N[2020-10-11 02:12:33])\n    :true\n\n    iex> Dsv.Equal.valid?(false, false)\n    :true\n\n    iex> Dsv.Equal.valid?(nil, nil)\n    :true\n\n    iex> Dsv.Equal.valid?(true, true)\n    :true","ref":"Dsv.Equal.html#valid?/2-example"},{"type":"function","title":"Dsv.Equal.valid?/3","doc":"","ref":"Dsv.Equal.html#valid?/3"},{"type":"function","title":"Dsv.Equal.validate/2","doc":"The `validate/2` function is designed to evaluate whether two given values are equal.\nThis function takes two parameters and compares them to determine if they are equal.\nCheck if the two values are equal.","ref":"Dsv.Equal.html#validate/2"},{"type":"function","title":"Parameters - Dsv.Equal.validate/2","doc":"* `data` - The first value to be compared.\n  * `options` - The second value to be compared against the first.","ref":"Dsv.Equal.html#validate/2-parameters"},{"type":"function","title":"Returns - Dsv.Equal.validate/2","doc":"- `:ok` if `data` is equal to `options`.\n- `{:error, \"Values must be equal\"}` if `data` is not equal to `options`.","ref":"Dsv.Equal.html#validate/2-returns"},{"type":"function","title":"Example - Dsv.Equal.validate/2","doc":"iex> Dsv.Equal.validate(\"text\", :text)\n    {:error, \"Values must be equal\"}\n\n    iex> Dsv.Equal.validate([], [\"\"])\n    {:error, \"Values must be equal\"}\n\n    iex> Dsv.Equal.validate(\"a\", \"ab\")\n    {:error, \"Values must be equal\"}\n\n    iex> Dsv.Equal.validate(nil, \"\")\n    {:error, \"Values must be equal\"}\n\n    iex> Dsv.Equal.validate(%{}, %{\"a\" => \"b\"})\n    {:error, \"Values must be equal\"}\n\n    iex> Dsv.Equal.validate(%{}, [])\n    {:error, \"Values must be equal\"}\n\n    iex> Dsv.Equal.validate(%{}, true)\n    {:error, \"Values must be equal\"}\n\n    iex> Dsv.Equal.validate([\"elem\"], [\"elem\"])\n    :ok\n\n    iex> Dsv.Equal.validate(\"text\", \"text\")\n    :ok\n\n    iex> Dsv.Equal.validate(36, 36)\n    :ok\n\n    iex> Dsv.Equal.validate(%{a: :b}, %{a: :b})\n    :ok\n\n    iex> Dsv.Equal.validate(~N[2020-10-11 02:12:33], ~N[2020-10-11 02:12:33])\n    :ok\n\n    iex> Dsv.Equal.validate(false, false)\n    :ok\n\n    iex> Dsv.Equal.validate(nil, nil)\n    :ok\n\n    iex> Dsv.Equal.validate(true, true)\n    :ok","ref":"Dsv.Equal.html#validate/2-example"},{"type":"function","title":"Dsv.Equal.validate/3","doc":"The `validate/3` function is designed to evaluate whether two given values are equal.\nThis function takes three parameters and compares first to the second to determine if they are equal, in case of failure it returns third parameter as an error message.\nCheck if the two values are equal.","ref":"Dsv.Equal.html#validate/3"},{"type":"function","title":"Parameters - Dsv.Equal.validate/3","doc":"* `data` - The first value to be compared.\n  * `options` - The second value to be compared against the first.\n  * `message` - Message that will be returned on failure.","ref":"Dsv.Equal.html#validate/3-parameters"},{"type":"function","title":"Returns - Dsv.Equal.validate/3","doc":"- `:ok` if `data` is equal to `options`.\n- `{:error, \"message\"}` if `data` is not equal to `options`.","ref":"Dsv.Equal.html#validate/3-returns"},{"type":"function","title":"Example - Dsv.Equal.validate/3","doc":"iex> Dsv.Equal.validate(\"text\", :text, \"Value must be equal to `:text`.\")\n    {:error, \"Value must be equal to `:text`.\"}\n\n    iex> Dsv.Equal.validate([], [\"\"], \"Value must be a list with one empty string element.\")\n    {:error, \"Value must be a list with one empty string element.\"}\n\n    iex> Dsv.Equal.validate(\"a\", \"ab\", \"Value must be equal to string `ab`.\")\n    {:error, \"Value must be equal to string `ab`.\"}\n\n    iex> Dsv.Equal.validate(nil, \"\", \"Value must be equal to empty string.\")\n    {:error, \"Value must be equal to empty string.\"}\n\n    iex> Dsv.Equal.validate(%{}, %{\"a\" => \"b\"}, \"Value must be a map that contains key `a` and value `b`.\")\n    {:error, \"Value must be a map that contains key `a` and value `b`.\"}\n\n    iex> Dsv.Equal.validate(%{}, [], \"Value must be equal to empty list.\")\n    {:error, \"Value must be equal to empty list.\"}\n\n    iex> Dsv.Equal.validate(%{}, true, \"Value must be equal to true\")\n    {:error, \"Value must be equal to true\"}\n\n    iex> Dsv.Equal.validate([\"elem\"], [\"elem\"], \"Value must be a list with exactly one element equal to string `eleme`.\")\n    :ok\n\n    iex> Dsv.Equal.validate(\"text\", \"text\", \"Value must be equal to string `text`\")\n    :ok\n\n    iex> Dsv.Equal.validate(36, 36, \"Value must be equal to number `36`\")\n    :ok\n\n    iex> Dsv.Equal.validate(%{a: :b}, %{a: :b}, \"This is not a map I have expected.\")\n    :ok\n\n    iex> Dsv.Equal.validate(~N[2020-10-11 02:12:33], ~N[2020-10-11 02:12:33], \"It isn't the date I was thinking of.\")\n    :ok\n\n    iex> Dsv.Equal.validate(false, false, \"This true is so false.\")\n    :ok\n\n    iex> Dsv.Equal.validate(nil, nil, \"nil would be good\")\n    :ok\n\n    iex> Dsv.Equal.validate(true, true, \"What?\")\n    :ok","ref":"Dsv.Equal.html#validate/3-example"},{"type":"function","title":"Dsv.Equal.value/2","doc":"","ref":"Dsv.Equal.html#value/2"},{"type":"module","title":"Dsv.Exclusion","doc":"Ensure a value is not on the forbidden values list.\nDsv.Exclusion module provides functions to determine if a value is not present in a list.","ref":"Dsv.Exclusion.html"},{"type":"function","title":"Dsv.Exclusion.valid?/2","doc":"The `valid?/2` function evaluates whether a given value is not present in a list.","ref":"Dsv.Exclusion.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.Exclusion.valid?/2","doc":"* `value` - The value to be checked.\n  * `list` - The list in which the presence of the value is checked.","ref":"Dsv.Exclusion.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.Exclusion.valid?/2","doc":"A boolean value:\n\n- `true` if `value` is not present in `list`.\n- `false` if `value` is present in `list`.","ref":"Dsv.Exclusion.html#valid?/2-returns"},{"type":"function","title":"Examples - Dsv.Exclusion.valid?/2","doc":"iex> Dsv.Exclusion.valid?(\"test\", [\"a\", :b, \"c\", %{a: :b}])\n    :true\n\n    iex> Dsv.Exclusion.valid?(\"test\", [\"test\", :b, \"c\", %{a: :b}])\n    :false\n\n    iex> Dsv.Exclusion.valid?(\"test\", [])\n    :true\n\n    iex> Dsv.Exclusion.valid?(nil, [nil, 1, 2, 3])\n    :false\n\n    iex> Dsv.Exclusion.valid?(nil, [1, 2, 3])\n    :true","ref":"Dsv.Exclusion.html#valid?/2-examples"},{"type":"function","title":"Dsv.Exclusion.valid?/3","doc":"","ref":"Dsv.Exclusion.html#valid?/3"},{"type":"function","title":"Dsv.Exclusion.validate/2","doc":"The `validate/2` function evaluates whether a given value is not present in a list.","ref":"Dsv.Exclusion.html#validate/2"},{"type":"function","title":"Parameters - Dsv.Exclusion.validate/2","doc":"* `value` - The value to be checked.\n  * `list` - The list in which the presence of the value is checked.","ref":"Dsv.Exclusion.html#validate/2-parameters"},{"type":"function","title":"Returns - Dsv.Exclusion.validate/2","doc":"- `:ok` if `value` is not present in `list`.\n- `{:error, message}` if `value` is present in `list`.","ref":"Dsv.Exclusion.html#validate/2-returns"},{"type":"function","title":"Examples - Dsv.Exclusion.validate/2","doc":"iex> Dsv.Exclusion.validate(\"test\", [\"a\", :b, \"c\", %{a: :b}])\n    :ok\n\n    iex> Dsv.Exclusion.validate(\"test\", [\"test\", :b, \"c\", %{a: :b}])\n    {:error, ~s(Value \"test\" can't be on the list [\"test\", :b, \"c\", %{a: :b}])}\n\n    iex> Dsv.Exclusion.validate(\"test\", [])\n    :ok\n\n    iex> Dsv.Exclusion.validate(nil, [nil, 1, 2, 3])\n    {:error, ~s(Value nil can't be on the list [nil, 1, 2, 3])}\n\n    iex> Dsv.Exclusion.validate(nil, [nil, 1, 2, 3], message: \"Provided value is not allowed.\")\n    {:error, \"Provided value is not allowed.\"}\n\n    iex> Dsv.Exclusion.validate(nil, [1, 2, 3])\n    :ok","ref":"Dsv.Exclusion.html#validate/2-examples"},{"type":"function","title":"Dsv.Exclusion.validate/3","doc":"The `validate/3` function evaluates whether a given value is not present in a list.","ref":"Dsv.Exclusion.html#validate/3"},{"type":"function","title":"Parameters - Dsv.Exclusion.validate/3","doc":"* `value` - The value to be checked.\n  * `list` - The list in which the presence of the value is checked.\n  * `message` - The message that will be returned in case of failure.","ref":"Dsv.Exclusion.html#validate/3-parameters"},{"type":"function","title":"Returns - Dsv.Exclusion.validate/3","doc":"- `:ok` if `value` is not present in `list`.\n- `{:error, message}` if `value` is present in `list`.","ref":"Dsv.Exclusion.html#validate/3-returns"},{"type":"function","title":"Examples - Dsv.Exclusion.validate/3","doc":"iex> Dsv.Exclusion.validate(nil, [nil, 1, 2, 3], message: \"Provided value is not allowed.\")\n    {:error, \"Provided value is not allowed.\"}\n\n    iex> Dsv.Exclusion.validate(nil, [nil, 1, 2, 3], \"Provided value is not allowed.\")\n    {:error, \"Provided value is not allowed.\"}\n\n    iex> Dsv.Exclusion.validate(nil, [1, 2, 3], \"Provided value is not allowed.\")\n    :ok","ref":"Dsv.Exclusion.html#validate/3-examples"},{"type":"function","title":"Dsv.Exclusion.value/2","doc":"","ref":"Dsv.Exclusion.html#value/2"},{"type":"module","title":"Dsv.Format","doc":"Check if the given `String` matches the regular expression.\nDsv.Format module provides a functions to determine if a string matches a regular expression.","ref":"Dsv.Format.html"},{"type":"function","title":"Dsv.Format.valid?/2","doc":"The `valid?/2` function evaluates whether a given string matches a specified regular expression.","ref":"Dsv.Format.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.Format.valid?/2","doc":"* `data` - The string to be checked against the regular expression.\n  * `format` - The regular expression pattern to be used for matching.","ref":"Dsv.Format.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.Format.valid?/2","doc":"A boolean value:\n\n- `true` if the `string` matches the `regex`.\n- `false` if the `string` does not match the `regex`.","ref":"Dsv.Format.html#valid?/2-returns"},{"type":"function","title":"Examples - Dsv.Format.valid?/2","doc":"iex> Dsv.Format.valid?(\"string to match\", ~r/.* .* .*/)\n    :true\n\n    iex> Dsv.Format.valid?(\"stringtomatch\", ~r/.* .* .*/)\n    :false","ref":"Dsv.Format.html#valid?/2-examples"},{"type":"function","title":"Dsv.Format.valid?/3","doc":"","ref":"Dsv.Format.html#valid?/3"},{"type":"function","title":"Dsv.Format.validate/2","doc":"The `validate/2` function evaluates whether a given string matches a specified regular expression.","ref":"Dsv.Format.html#validate/2"},{"type":"function","title":"Parameters - Dsv.Format.validate/2","doc":"* `data` - The string to be checked against the regular expression.\n  * `format` - The regular expression pattern to be used for matching.","ref":"Dsv.Format.html#validate/2-parameters"},{"type":"function","title":"Returns - Dsv.Format.validate/2","doc":"- `:ok` if the `string` matches the `regex`.\n- `{:error, message}` if the `string` does not match the `regex`.","ref":"Dsv.Format.html#validate/2-returns"},{"type":"function","title":"Examples - Dsv.Format.validate/2","doc":"iex> Dsv.Format.validate(\"string to match\", ~r/.* .* .*/)\n    :ok\n\n    iex> Dsv.Format.validate(\"stringtomatch\", ~r/.* .* .*/)\n    {:error, \"Value stringtomatch does not match pattern .* .* .*\"}","ref":"Dsv.Format.html#validate/2-examples"},{"type":"function","title":"Dsv.Format.validate/3","doc":"The `validate/2` function evaluates whether a given string matches a specified regular expression.","ref":"Dsv.Format.html#validate/3"},{"type":"function","title":"Parameters - Dsv.Format.validate/3","doc":"* `data` - The string to be checked against the regular expression.\n  * `format` - The regular expression pattern to be used for matching.\n  * `message` - An custom error message to be returned in case of failure.","ref":"Dsv.Format.html#validate/3-parameters"},{"type":"function","title":"Returns - Dsv.Format.validate/3","doc":"- `:ok` if the `string` matches the `regex`.\n- `{:error, message}` if the `string` does not match the `regex`.","ref":"Dsv.Format.html#validate/3-returns"},{"type":"function","title":"Examples - Dsv.Format.validate/3","doc":"iex> Dsv.Format.validate(\"string to match\", ~r/.* .* .*/, \"This is wrong.\")\n    :ok\n\n    iex> Dsv.Format.validate(\"stringtomatch\", ~r/.* .* .*/, \"This is wrong.\")\n    {:error, \"This is wrong.\"}\n\n    iex> Dsv.Format.validate(\"stringtomatch\", ~r/.* .* .*/, message: \"This is wrong.\")\n    {:error, \"This is wrong.\"}","ref":"Dsv.Format.html#validate/3-examples"},{"type":"function","title":"Dsv.Format.value/2","doc":"","ref":"Dsv.Format.html#value/2"},{"type":"module","title":"Dsv.Inclusion","doc":"Check if the given value is on the list of possible values.\nDsv.Inclusion module provides a function to determine if a value is present in a list.","ref":"Dsv.Inclusion.html"},{"type":"function","title":"Dsv.Inclusion.valid?/2","doc":"The `valid?/2` function evaluates whether a given value is present in a provided list.","ref":"Dsv.Inclusion.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.Inclusion.valid?/2","doc":"* `value` - The value to be checked.\n  * `list` - The list in which the presence of the value is checked.","ref":"Dsv.Inclusion.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.Inclusion.valid?/2","doc":"A boolean value:\n\n- `true` if `value` is present in `list`.\n- `false` if `value` is not present in `list`.","ref":"Dsv.Inclusion.html#valid?/2-returns"},{"type":"function","title":"Examples - Dsv.Inclusion.valid?/2","doc":"iex> Dsv.Inclusion.valid?(\"test\", [\"a\", :b, \"c\", %{a: :b}])\n    :false\n\n    iex> Dsv.Inclusion.valid?(\"test\", [\"test\", :b, \"c\", %{a: :b}])\n    :true\n\n    iex> Dsv.Inclusion.valid?(%{a: :b}, [\"test\", :b, \"c\", %{a: :b}])\n    :true\n\n    iex> Dsv.Inclusion.valid?(%{a: :b}, [\"test\", :b, \"c\", %{a: :b, c: :d}])\n    :false\n\n    iex> Dsv.Inclusion.valid?(\"test\", [])\n    :false\n\n    iex> Dsv.Inclusion.valid?(nil, [nil, 1, 2, 3])\n    :true\n\n    iex> Dsv.Inclusion.valid?(nil, [1, 2, 3])\n    :false","ref":"Dsv.Inclusion.html#valid?/2-examples"},{"type":"function","title":"Dsv.Inclusion.valid?/3","doc":"","ref":"Dsv.Inclusion.html#valid?/3"},{"type":"function","title":"Dsv.Inclusion.validate/2","doc":"The `validate/2` function evaluates whether a given value is present in a provided list.","ref":"Dsv.Inclusion.html#validate/2"},{"type":"function","title":"Parameters - Dsv.Inclusion.validate/2","doc":"* `value` - The value to be checked.\n  * `list` - The list in which the presence of the value is checked.","ref":"Dsv.Inclusion.html#validate/2-parameters"},{"type":"function","title":"Returns - Dsv.Inclusion.validate/2","doc":"- `:ok` if `value` is present in `list`.\n- `{:error, message}` if `value` is not present in `list`.","ref":"Dsv.Inclusion.html#validate/2-returns"},{"type":"function","title":"Examples - Dsv.Inclusion.validate/2","doc":"iex> Dsv.Inclusion.validate(\"test\", [\"a\", :b, \"c\", %{a: :b}])\n    {:error, ~s(Value \"test\" must be one of the [\"a\", :b, \"c\", %{a: :b}])}\n\n    iex> Dsv.Inclusion.validate(\"test\", [\"test\", :b, \"c\", %{a: :b}])\n    :ok\n\n    iex> Dsv.Inclusion.validate(%{a: :b}, [\"test\", :b, \"c\", %{a: :b}])\n    :ok\n\n    iex> Dsv.Inclusion.validate(%{a: :b}, [\"test\", :b, \"c\", %{a: :b, c: :d}])\n    {:error, ~s(Value %{a: :b} must be one of the [\"test\", :b, \"c\", %{c: :d, a: :b}])}\n\n    iex> Dsv.Inclusion.validate(\"test\", [])\n    {:error, ~s(Value \"test\" must be one of the [])}\n\n    iex> Dsv.Inclusion.validate(nil, [nil, 1, 2, 3])\n    :ok\n\n    iex> Dsv.Inclusion.validate(nil, [1, 2, 3])\n    {:error, ~s(Value nil must be one of the [1, 2, 3])}","ref":"Dsv.Inclusion.html#validate/2-examples"},{"type":"function","title":"Dsv.Inclusion.validate/3","doc":"The `validate/3` function evaluates whether a given value is present in a provided list.","ref":"Dsv.Inclusion.html#validate/3"},{"type":"function","title":"Parameters - Dsv.Inclusion.validate/3","doc":"* `value` - The value to be checked.\n  * `list` - The list in which the presence of the value is checked.\n  * `message` - Custom message that will be returned in case of failure.","ref":"Dsv.Inclusion.html#validate/3-parameters"},{"type":"function","title":"Returns - Dsv.Inclusion.validate/3","doc":"- `:ok` if `value` is present in `list`.\n- `{:error, message}` if `value` is not present in `list`.","ref":"Dsv.Inclusion.html#validate/3-returns"},{"type":"function","title":"Examples - Dsv.Inclusion.validate/3","doc":"iex> Dsv.Inclusion.validate(\"test\", [\"a\", :b, \"c\", %{a: :b}], \"Provided value is not accepted.\")\n    {:error, \"Provided value is not accepted.\"}\n\n    iex> Dsv.Inclusion.validate(\"test\", [\"test\", :b, \"c\", %{a: :b}], \"Provided value is not accepted.\")\n    :ok\n\n    iex> Dsv.Inclusion.validate(%{a: :b}, [\"test\", :b, \"c\", %{a: :b}], \"Provided value is not accepted.\")\n    :ok\n\n    iex> Dsv.Inclusion.validate(%{a: :b}, [\"test\", :b, \"c\", %{a: :b, c: :d}], \"Provided value is not accepted.\")\n    {:error, \"Provided value is not accepted.\"}\n\n    iex> Dsv.Inclusion.validate(\"test\", [], \"Provided value is not accepted.\")\n    {:error, \"Provided value is not accepted.\"}\n\n    iex> Dsv.Inclusion.validate(nil, [nil, 1, 2, 3], \"Provided value is not accepted.\")\n    :ok\n\n    iex> Dsv.Inclusion.validate(nil, [1, 2, 3], \"Provided value is not accepted.\")\n    {:error, \"Provided value is not accepted.\"}","ref":"Dsv.Inclusion.html#validate/3-examples"},{"type":"function","title":"Dsv.Inclusion.value/2","doc":"","ref":"Dsv.Inclusion.html#value/2"},{"type":"module","title":"Dsv.Length","doc":"Dsv.Length module provides a functions to check if the length of an input value meets specified criteria.\nValue can be of the type: `String`, `List`, `Map`, `Tuple`, `Integer`, `Float`. To use values of different type this type need to implement `DataLength` protocol.","ref":"Dsv.Length.html"},{"type":"function","title":"Dsv.Length.valid?/2","doc":"The `valid?/2` function checks if the length of a given input value meets the specified criteria.","ref":"Dsv.Length.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.Length.valid?/2","doc":"* `value` - The input value to be checked for length.\n  * `rules` - A list of validation rules. Each rule is represented as a keyword list containing:\n    - `:min` - The minimum allowed length.\n    - `:max` - The maximum allowed length.\n    - `:range` - A range of acceptable lengths.","ref":"Dsv.Length.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.Length.valid?/2","doc":"A boolean value:\n\n- `true` if the length of `value` meets all of the specified criteria.\n- `false` if the length of `value` does not meet any of the specified criteria.","ref":"Dsv.Length.html#valid?/2-returns"},{"type":"function","title":"Examples - Dsv.Length.valid?/2","doc":"`:min` example:\n    iex> Dsv.Length.valid?(\"abcd\", min: 3)\n    :true\n\n    iex> Dsv.Length.valid?(\"abcd\", min: 4)\n    :true\n\n    iex> Dsv.Length.valid?(\"abcd\", min: 5)\n    :false\n\n`:max` example:\n    iex> Dsv.Length.valid?(\"ab\", max: 3)\n    :true\n\n    iex> Dsv.Length.valid?(\"abcd\", max: 4)\n    :true\n\n    iex> Dsv.Length.valid?(\"abcdef\", max: 5)\n    :false\n\n`:min & :max` example:\n    iex> Dsv.Length.valid?(\"abcd\", min: 3, max: 5)\n    :true\n\n    iex> Dsv.Length.valid?(\"abcd\", min: 4, max: 5)\n    :true\n\n    iex> Dsv.Length.valid?(\"abcd\", min: 2, max: 4)\n    :true\n\n    iex> Dsv.Length.valid?(\"a\", min: 2, max: 4)\n    :false\n\n    iex> Dsv.Length.valid?(\"abcde\", min: 2, max: 4)\n    :false\n\n`:range` example:\n    iex> Dsv.Length.valid?(\"abcd\", range: {3, 5})\n    :true\n\n    iex> Dsv.Length.valid?(\"abcd\", range: {4, 5})\n    :true\n\n    iex> Dsv.Length.valid?(\"abcd\", range: {2, 4})\n    :true\n\n    iex> Dsv.Length.valid?(\"a\", range: {2, 4})\n    :false\n\n    iex> Dsv.Length.valid?(\"abcde\", range: {2, 4})\n    :false\n\n    iex> Dsv.Length.valid?(\"abcde\", range: {4, 2})\n    ** (RuntimeError) Min length (4) can't be greater that max length (2).","ref":"Dsv.Length.html#valid?/2-examples"},{"type":"function","title":"Dsv.Length.valid?/3","doc":"","ref":"Dsv.Length.html#valid?/3"},{"type":"function","title":"Dsv.Length.validate/2","doc":"The `valid?/2` function checks if the length of a given input value meets the specified criteria.","ref":"Dsv.Length.html#validate/2"},{"type":"function","title":"Parameters - Dsv.Length.validate/2","doc":"* `value` - The input value to be checked for length.\n  * `rules` - A list of validation rules. Each rule is represented as a keyword list containing:\n    - `:min` - The minimum allowed length.\n    - `:max` - The maximum allowed length.\n    - `:range` - A range of acceptable lengths.\n  * `message` - An optional options to provide custom message that will be returned on error.","ref":"Dsv.Length.html#validate/2-parameters"},{"type":"function","title":"Returns - Dsv.Length.validate/2","doc":"- `:ok` if the length of `value` meets all of the specified criteria.\n- `{:error, message}` if the length of `value` does not meet any of the specified criteria.","ref":"Dsv.Length.html#validate/2-returns"},{"type":"function","title":"Examples - Dsv.Length.validate/2","doc":"`:min` example:\n    iex> Dsv.Length.validate(\"abcd\", min: 3)\n    :ok\n\n    iex> Dsv.Length.validate(\"abcd\", min: 4)\n    :ok\n\n    iex> Dsv.Length.validate(\"abcd\", min: 5)\n    {:error, ~s(Value \"abcd\" is to short. Minimum lenght is 5)}\n\n    iex> Dsv.Length.validate(\"abcd\", min: 5, message: \"This value should not be shorter than 5.\")\n    {:error, \"This value should not be shorter than 5.\"}\n\n`:max` example:\n    iex> Dsv.Length.validate(\"ab\", max: 3)\n    :ok\n\n    iex> Dsv.Length.validate(\"abcd\", max: 4)\n    :ok\n\n    iex> Dsv.Length.validate(\"abcdef\", max: 5)\n    {:error, ~s(Value \"abcdef\" is to long. Maximum length is 5)}\n\n    iex> Dsv.Length.validate(\"abcdef\", max: 5, message: \"This value should not be longer that 5.\")\n    {:error, \"This value should not be longer that 5.\"}\n\n`:min & :max` example:\n    iex> Dsv.Length.validate(\"abcd\", min: 3, max: 5)\n    :ok\n\n    iex> Dsv.Length.validate(\"abcd\", min: 4, max: 5)\n    :ok\n\n    iex> Dsv.Length.validate(\"abcd\", min: 2, max: 4)\n    :ok\n\n    iex> Dsv.Length.validate(\"a\", min: 2, max: 4)\n    {:error, ~s(Value \"a\" has wrong length. Minimum lenght is 2, maximum length is 4)}\n\n    iex> Dsv.Length.validate(\"abcde\", min: 2, max: 4)\n    {:error, ~s(Value \"abcde\" has wrong length. Minimum lenght is 2, maximum length is 4)}\n\n    iex> Dsv.Length.validate(\"a\", min: 2, max: 4, message: \"This value must have length between 2 and 4.\")\n    {:error, \"This value must have length between 2 and 4.\"}\n\n    iex> Dsv.Length.validate(\"abcde\", min: 2, max: 4, message: \"This value must have length between 2 and 4.\")\n    {:error, \"This value must have length between 2 and 4.\"}\n\n`:range` example:\n    iex> Dsv.Length.validate(\"abcd\", range: {3, 5})\n    :ok\n\n    iex> Dsv.Length.validate(\"abcd\", range: {4, 5})\n    :ok\n\n    iex> Dsv.Length.validate(\"abcd\", range: {2, 4})\n    :ok\n\n    iex> Dsv.Length.validate(\"a\", range: {2, 4})\n    {:error, ~s(Value \"a\" has wrong length. Minimum lenght is 2, maximum length is 4)}\n\n    iex> Dsv.Length.validate(\"abcde\", range: {2, 4})\n    {:error, ~s(Value \"abcde\" has wrong length. Minimum lenght is 2, maximum length is 4)}\n\n    iex> Dsv.Length.validate(\"a\", range: {2, 4}, message: \"This value must have length between 2 and 4.\")\n    {:error, \"This value must have length between 2 and 4.\"}\n\n    iex> Dsv.Length.validate(\"abcde\", range: {2, 4}, message: \"This value must have length between 2 and 4.\")\n    {:error, \"This value must have length between 2 and 4.\"}\n\n    iex> Dsv.Length.validate(\"abcde\", range: {4, 2})\n    ** (RuntimeError) Min length (4) can't be greater that max length (2).","ref":"Dsv.Length.html#validate/2-examples"},{"type":"function","title":"Dsv.Length.validate/3","doc":"","ref":"Dsv.Length.html#validate/3"},{"type":"function","title":"Dsv.Length.value/2","doc":"","ref":"Dsv.Length.html#value/2"},{"type":"module","title":"Dsv.None","doc":"`Dsv.None` validator provides a functions to check that there is no element on the provided `List` or `String` that meets any of the provided criteria.\n\n\n\n> ##","ref":"Dsv.None.html"},{"type":"module","title":"Validate other types of data {: .tip} - Dsv.None","doc":">\n> To validate arguments of other data types `FindAll` protocol must be implemented for this type.","ref":"Dsv.None.html#module-validate-other-types-of-data-tip"},{"type":"function","title":"Dsv.None.valid?/2","doc":"Ensure that none of the validators pass the check for the given data.\n\nFor a string, this validator iterates through all graphemes, checking if none of them pass validation for all provided validators. For a list, it iterates through all elements in the list, verifying that none of them pass the validation for all specified validators.\n\nReturns `:true` if all validators fail for all elements; otherwise, returns `:false.`","ref":"Dsv.None.html#valid?/2"},{"type":"function","title":"Example - Dsv.None.valid?/2","doc":"iex> Dsv.None.valid?(\"abcd\", format: ~r/^[a-z]$/, equal: \"w\")\n    :true\n\n    iex> Dsv.None.valid?(\"abcd\", format: ~r/^[a-z]$/, equal: \"a\")\n    :false\n\n    iex> Dsv.None.valid?(\"abcd\", format: ~r/^[a-d]$/, custom: &is_bitstring/1)\n    :false\n\n    iex> Dsv.None.valid?(\"abcd\", format: ~r/^[a-c]$/, custom: &is_bitstring/1)\n    :false\n\n    iex> Dsv.None.valid?([1, 2, 3, \"hello\"], custom: &is_number/1, number: [gt: 2])\n    :false\n\n    iex> Dsv.None.valid?([1, 2, 3, \"hello\"], number: [gt: 0])\n    :false\n\n    iex> Dsv.None.valid?([1, 2, 3], custom: &is_number/1, number: [gte: 0, lte: 10])\n    :false","ref":"Dsv.None.html#valid?/2-example"},{"type":"function","title":"Dsv.None.valid?/3","doc":"","ref":"Dsv.None.html#valid?/3"},{"type":"function","title":"Dsv.None.validate/2","doc":"Ensure that none of the validators pass the check for the given data and allow for custom error messages.\n\nFor a string, this validator iterates through all graphemes, checking if none of them pass validation for all provided validators. For a list, it iterates through all elements in the list, verifying that none of them pass the validation for all specified validators.\n\nReturns `:ok` if all validators fail for all elements; otherwise, returns error ruple with the error message.","ref":"Dsv.None.html#validate/2"},{"type":"function","title":"Options - Dsv.None.validate/2","doc":"* any validator name like\n    * `:format`\n    * `:custom`\n    * `:number`\n    * `:length`\n    * `:equal`\n    * `:data`\n  * `:message` - (EEx string) message used in error tuple in case of validation failure.","ref":"Dsv.None.html#validate/2-options"},{"type":"function","title":"Example - Dsv.None.validate/2","doc":"iex> Dsv.None.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"w\")\n    :ok\n\n    iex> Dsv.None.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"a\")\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.None.validate(\"abcd\", format: ~r/^[a-d]$/, custom: &is_bitstring/1)\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.None.validate(\"abcd\", format: ~r/^[a-c]$/, custom: &is_bitstring/1)\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.None.validate([1, 2, 3, \"hello\"], custom: &is_number/1, number: [gt: 2])\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.None.validate([1, 2, 3, \"hello\"], number: [gt: 0])\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.None.validate([1, 2, 3], custom: &is_number/1, number: [gte: 0, lte: 10])\n    {:error, \"Not all elements match all validators.\"}\n\n    iex> Dsv.None.validate([1, 2, 3, \"hello\"], number: [gt: 0], format: ~r/^[a-z]$/)\n    {:error, \"Not all elements match all validators.\"}","ref":"Dsv.None.html#validate/2-example"},{"type":"function","title":"Example with custom message - Dsv.None.validate/2","doc":"iex> Dsv.None.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"w\", message: \"All elements must be small letter 'w'.\")\n    :ok\n\n    iex> Dsv.None.validate(\"abcd\", format: ~r/^[a-z]$/, equal: \"a\", message: \"All elements must be small letter 'a'.\")\n    {:error, \"All elements must be small letter 'a'.\"}\n\n    iex> Dsv.None.validate(\"abcd\", format: ~r/^[a-d]$/, custom: &is_bitstring/1, message: \"All elements must be one of the letter: 'a', 'b', 'c', 'd'.\")\n    {:error, \"All elements must be one of the letter: 'a', 'b', 'c', 'd'.\"}\n\n    iex> Dsv.None.validate(\"abcd\", format: ~r/^[a-c]$/, custom: &is_bitstring/1, message: \"All elements must be one of the letter: 'a', 'b', 'c'.\")\n    {:error, \"All elements must be one of the letter: 'a', 'b', 'c'.\"}\n\n    iex> Dsv.None.validate([1, 2, 3, \"hello\"], custom: &is_number/1, number: [gt: 2], message: \"All values must be a number greater that 2.\")\n    {:error, \"All values must be a number greater that 2.\"}\n\n    iex> Dsv.None.validate([1, 2, 3, \"hello\"], number: [gt: 0], message: \"All values must be a number greater that 0.\")\n    {:error, \"All values must be a number greater that 0.\"}\n\n    iex> Dsv.None.validate([1, 2, 3], custom: &is_number/1, number: [gte: 0, lte: 10], message: \"All values must be a number between 0 and 10.\")\n    {:error, \"All values must be a number between 0 and 10.\"}","ref":"Dsv.None.html#validate/2-example-with-custom-message"},{"type":"function","title":"Dsv.None.validate/3","doc":"","ref":"Dsv.None.html#validate/3"},{"type":"function","title":"Dsv.None.value/2","doc":"","ref":"Dsv.None.html#value/2"},{"type":"module","title":"Dsv.NotEmpty","doc":"Ensure a value is not empty (default behavior) or empty if the option is set to `:false`\nEmpty lists, maps, strings, and nil values are considered empty.\n\nDsv.NotEmpty module provides a function to determine if a value is empty or non-empty.","ref":"Dsv.NotEmpty.html"},{"type":"function","title":"Dsv.NotEmpty.valid?/1","doc":"The `valid?/2` function checks if a provided value is non-empty.\n\nThe `valid?/2` function provides a flexible approach to check the emptiness of various data types.\nIt returns true for empty values and false for non-empty values.","ref":"Dsv.NotEmpty.html#valid?/1"},{"type":"function","title":"Parameters - Dsv.NotEmpty.valid?/1","doc":"* `value` - The value to be checked for emptiness. This can be a list, map, string, or nil.","ref":"Dsv.NotEmpty.html#valid?/1-parameters"},{"type":"function","title":"Returns - Dsv.NotEmpty.valid?/1","doc":"A boolean value:\n\n- `true` if `value` is empty according to the defined criteria.\n- `false` if `value` is non-empty according to the defined criteria.","ref":"Dsv.NotEmpty.html#valid?/1-returns"},{"type":"function","title":"Emptiness Criteria - Dsv.NotEmpty.valid?/1","doc":"The `valid?/2` function considers the following conditions to determine emptiness:\n- Lists with a length of 0 are considered empty.\n- Maps without any keys and values are considered empty.\n- Strings with a length of 0 are considered empty.\n- A nil value is considered empty.","ref":"Dsv.NotEmpty.html#valid?/1-emptiness-criteria"},{"type":"function","title":"Example - Dsv.NotEmpty.valid?/1","doc":"iex> Dsv.NotEmpty.valid?([])\n    :false\n\n    iex> Dsv.NotEmpty.valid?(\"\")\n    :false\n\n    iex> Dsv.NotEmpty.valid?(nil)\n    :false\n\n    iex> Dsv.NotEmpty.valid?(%{})\n    :false\n\n    iex> Dsv.NotEmpty.valid?([\"elem\"])\n    :true\n\n    iex> Dsv.NotEmpty.valid?({})\n    :false\n\n    iex> Dsv.NotEmpty.valid?({:a})\n    :true\n\n    iex> Dsv.NotEmpty.valid?(:a)\n    :true\n\n    iex> Dsv.NotEmpty.valid?(\"text\")\n    :true\n\n    iex> Dsv.NotEmpty.valid?(36)\n    :true\n\n    iex> Dsv.NotEmpty.valid?(%{a: :b})\n    :true","ref":"Dsv.NotEmpty.html#valid?/1-example"},{"type":"function","title":"Dsv.NotEmpty.valid?/2","doc":"The `valid?/2` function checks if a provided value is empty or non-empty, with second parameter\nthat can reverse the behavior.\n\nThe `valid?/2` function provides a flexible approach to check the emptiness of various data types.\nIt returns true for empty values and false for non-empty values when second parameter is set to :true. However, you can reverse\nthis behavior by setting the second parameter to :false.","ref":"Dsv.NotEmpty.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.NotEmpty.valid?/2","doc":"* `value` - The value to be checked for emptiness. This can be a list, map, string, or nil.\n  * `:is_empty` - A boolean value that, when set to :false, reverses the function's behavior,\n    returning true for empty values and false for non-empty values.","ref":"Dsv.NotEmpty.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.NotEmpty.valid?/2","doc":"A boolean value:\n\n- `true` if `value` is empty according to the defined criteria.\n- `false` if `value` is non-empty according to the defined criteria.","ref":"Dsv.NotEmpty.html#valid?/2-returns"},{"type":"function","title":"Emptiness Criteria - Dsv.NotEmpty.valid?/2","doc":"The `valid?/2` function considers the following conditions to determine emptiness:\n- Lists with a length of 0 are considered empty.\n- Maps without any keys and values are considered empty.\n- Strings with a length of 0 are considered empty.\n- A nil value is considered empty.","ref":"Dsv.NotEmpty.html#valid?/2-emptiness-criteria"},{"type":"function","title":"Example - Dsv.NotEmpty.valid?/2","doc":"iex> Dsv.NotEmpty.valid?(%{}, true)\n    :false\n\n    iex> Dsv.NotEmpty.valid?({}, :false)\n    :true\n\n    iex> Dsv.NotEmpty.valid?({:a}, :false)\n    :false\n\n    iex> Dsv.NotEmpty.valid?(:a, :false)\n    :false\n\n    iex> Dsv.NotEmpty.valid?(%{a: :b}, true)\n    :true\n\n    iex> Dsv.NotEmpty.valid?([], false)\n    :true\n\n    iex> Dsv.NotEmpty.valid?(nil, false)\n    :true\n\n    iex> Dsv.NotEmpty.valid?(36, false)\n    :false\n\n    iex> Dsv.NotEmpty.valid?(\"text\", false)\n    :false","ref":"Dsv.NotEmpty.html#valid?/2-example"},{"type":"function","title":"Dsv.NotEmpty.valid?/3","doc":"","ref":"Dsv.NotEmpty.html#valid?/3"},{"type":"function","title":"Dsv.NotEmpty.validate/2","doc":"The `validate/2` function checks if a provided value is empty or non-empty, with an optional parameter\nto reverse the behavior.\n\nThe `validate/2` function provides a flexible approach to check the emptiness of various data types.\nBy default, it returns true for empty values and false for non-empty values. However, you can reverse\nthis behavior by setting the second parameter to :false.","ref":"Dsv.NotEmpty.html#validate/2"},{"type":"function","title":"Parameters - Dsv.NotEmpty.validate/2","doc":"* `value` - The value to be checked for emptiness. This can be a list, map, string, or nil.\n  * `:is_empty` (optional, default: true) - A boolean value that, when set to :false, reverses the function's behavior,\n    returning true for empty values and false for non-empty values.","ref":"Dsv.NotEmpty.html#validate/2-parameters"},{"type":"function","title":"Returns - Dsv.NotEmpty.validate/2","doc":"- `:ok` if `value` is empty according to the defined criteria.\n- `{:error, message}` if `value` is non-empty according to the defined criteria.","ref":"Dsv.NotEmpty.html#validate/2-returns"},{"type":"function","title":"Emptiness Criteria - Dsv.NotEmpty.validate/2","doc":"The `valid?/2` function considers the following conditions to determine emptiness:\n- Lists with a length of 0 are considered empty.\n- Maps without any keys and values are considered empty.\n- Strings with a length of 0 are considered empty.\n- A nil value is considered empty.","ref":"Dsv.NotEmpty.html#validate/2-emptiness-criteria"},{"type":"function","title":"Example - Dsv.NotEmpty.validate/2","doc":"iex> Dsv.NotEmpty.validate([])\n    {:error, \"Value must not be empty\"}\n\n    iex> Dsv.NotEmpty.validate(\"\")\n    {:error, \"Value must not be empty\"}\n\n    iex> Dsv.NotEmpty.validate(nil)\n    {:error, \"Value must not be empty\"}\n\n    iex> Dsv.NotEmpty.validate(%{})\n    {:error, \"Value must not be empty\"}\n\n    iex> Dsv.NotEmpty.validate(%{}, true)\n    {:error, \"Value must not be empty\"}\n\n    iex> Dsv.NotEmpty.validate([\"elem\"])\n    :ok\n\n    iex> Dsv.NotEmpty.validate({})\n    {:error, \"Value must not be empty\"}\n\n    iex> Dsv.NotEmpty.validate({:a})\n    :ok\n\n    iex> Dsv.NotEmpty.validate({}, :false)\n    :ok\n\n    iex> Dsv.NotEmpty.validate({:a}, :false)\n    {:error, \"Value must be empty\"}\n\n    iex> Dsv.NotEmpty.validate(:a)\n    :ok\n\n    iex> Dsv.NotEmpty.validate(:a, :false)\n    {:error, \"Value must be empty\"}\n\n    iex> Dsv.NotEmpty.validate(\"text\")\n    :ok\n\n    iex> Dsv.NotEmpty.validate(36)\n    :ok\n\n    iex> Dsv.NotEmpty.validate(%{a: :b})\n    :ok\n\n    iex> Dsv.NotEmpty.validate(%{a: :b}, true)\n    :ok\n\n    iex> Dsv.NotEmpty.validate([], false)\n    :ok\n\n    iex> Dsv.NotEmpty.validate(nil, false)\n    :ok\n\n    iex> Dsv.NotEmpty.validate(36, false)\n    {:error, \"Value must be empty\"}\n\n    iex> Dsv.NotEmpty.validate(\"text\", false)\n    {:error, \"Value must be empty\"}","ref":"Dsv.NotEmpty.html#validate/2-example"},{"type":"function","title":"Dsv.NotEmpty.validate/3","doc":"","ref":"Dsv.NotEmpty.html#validate/3"},{"type":"function","title":"Dsv.NotEmpty.value/2","doc":"","ref":"Dsv.NotEmpty.html#value/2"},{"type":"module","title":"Dsv.Number","doc":"Dsv.Number module provides functions to validate a number based on various options.","ref":"Dsv.Number.html"},{"type":"module","title":"Possible options - Dsv.Number","doc":"- `:gt` - Greater than (value must be greater than the specified number).\n  - `:lt` - Lower than (value must be lower than the specified number).\n  - `:gte` - Greater than or equal (value must be greater than or equal to the specified number).\n  - `:lte` - Lower than or equal (value must be lower than or equal to the specified number).\n  - `:eq` - Equal (value must be equal to the specified number).\n  - `:between` - Between (value must be within the specified range defined by two numbers).\n\nOptions can be combined.","ref":"Dsv.Number.html#module-possible-options"},{"type":"function","title":"Dsv.Number.valid?/2","doc":"The `valid?/2` function checks if a provided number meets specific validation criteria based on the given options (in the form of a keyword list).","ref":"Dsv.Number.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.Number.valid?/2","doc":"* `number` - The number to be validated (integer of float).\n  * `options` - A list of validation options. Each option consists of a keyword followed by the corresponding value.\n    Supported options include:\n    - `:gt` - Greater than (value must be greater than the specified number).\n    - `:lt` - Lower than (value must be lower than the specified number).\n    - `:gte` - Greater than or equal (value must be greater than or equal to the specified number).\n    - `:lte` - Lower than or equal (value must be lower than or equal to the specified number).\n    - `:eq` - Equal (value must be equal to the specified number).\n    - `:between` - Between (value must be within the specified range defined by two numbers).","ref":"Dsv.Number.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.Number.valid?/2","doc":"A boolean value:\n\n- `true` if the `number` meets all the specified validation criteria.\n- `false` if the `number` fails to meet any of the specified criteria.","ref":"Dsv.Number.html#valid?/2-returns"},{"type":"function","title":"Examples - Dsv.Number.valid?/2","doc":"`:gt` example:\n    iex> Dsv.Number.valid?(4, gt: 3)\n    :true\n\n    iex> Dsv.Number.valid?(4, gt: 4)\n    :false\n\n    iex> Dsv.Number.valid?(4, gt: 5)\n    :false\n\n`:gte` example:\n    iex> Dsv.Number.valid?(4, gte: 3)\n    :true\n\n    iex> Dsv.Number.valid?(4, gte: 4)\n    :true\n\n    iex> Dsv.Number.valid?(4, gte: 5)\n    :false\n\n`:lt` example:\n    iex> Dsv.Number.valid?(4, lt: 3)\n    :false\n\n    iex> Dsv.Number.valid?(4, lt: 4)\n    :false\n\n    iex> Dsv.Number.valid?(4, lt: 5)\n    :true\n\n`:lte` example:\n    iex> Dsv.Number.valid?(4, lte: 3)\n    :false\n\n    iex> Dsv.Number.valid?(4, lte: 4)\n    :true\n\n    iex> Dsv.Number.valid?(4, lte: 5)\n    :true\n\n`:gt & :lt` example:\n    iex> Dsv.Number.valid?(4, gt: 3, lt: 5)\n    :true\n\n    iex> Dsv.Number.valid?(4, gt: 4, lt: 5)\n    :false\n\n    iex> Dsv.Number.valid?(4, gt: 2, lt: 4)\n    :false\n\n`:gte & :lt` example:\n    iex> Dsv.Number.valid?(4, gte: 3, lt: 5)\n    :true\n\n    iex> Dsv.Number.valid?(4, gte: 4, lt: 5)\n    :true\n\n    iex> Dsv.Number.valid?(4, gte: 2, lt: 4)\n    :false\n\n`:gt & :lte` example:\n    iex> Dsv.Number.valid?(4, gt: 3, lte: 5)\n    :true\n\n    iex> Dsv.Number.valid?(4, gt: 4, lte: 5)\n    :false\n\n    iex> Dsv.Number.valid?(4, gt: 2, lte: 4)\n    :true\n\n`:gte & :lte` example:\n    iex> Dsv.Number.valid?(4, gte: 3, lte: 5)\n    :true\n\n    iex> Dsv.Number.valid?(4, gte: 4, lte: 5)\n    :true\n\n    iex> Dsv.Number.valid?(4, gte: 2, lte: 4)\n    :true\n\n    iex> Dsv.Number.valid?(1, gte: 2, lte: 4)\n    :false\n\n    iex> Dsv.Number.valid?(5, gte: 2, lte: 4)\n    :false\n\n`:between` example:\n    iex> Dsv.Number.valid?(4, between: {3, 5})\n    :true\n\n    iex> Dsv.Number.valid?(4, between: {4, 5})\n    :true\n\n    iex> Dsv.Number.valid?(4, between: {2, 4})\n    :true\n\n    iex> Dsv.Number.valid?(1, between: {2, 4})\n    :false\n\n    iex> Dsv.Number.valid?(5, between: {2, 4})\n    :false\n\n`:eq` example:\n    iex> Dsv.Number.valid?(4, eq: 4)\n    :true\n\n    iex> Dsv.Number.valid?(4, eq: 5)\n    :false","ref":"Dsv.Number.html#valid?/2-examples"},{"type":"function","title":"Dsv.Number.valid?/3","doc":"","ref":"Dsv.Number.html#valid?/3"},{"type":"function","title":"Dsv.Number.validate/2","doc":"The `validate/2` function checks if a provided number meets specific validation criteria based on the given options.","ref":"Dsv.Number.html#validate/2"},{"type":"function","title":"Parameters - Dsv.Number.validate/2","doc":"* `number` - The number to be validated.\n  * `options` - A list of validation options. Each option consists of a keyword followed by the corresponding value.\n    Supported options include:\n    - `:gt` - Greater than (value must be greater than the specified number).\n    - `:lt` - Lower than (value must be lower than the specified number).\n    - `:gte` - Greater than or equal (value must be greater than or equal to the specified number).\n    - `:lte` - Lower than or equal (value must be lower than or equal to the specified number).\n    - `:eq` - Equal (value must be equal to the specified number).\n    - `:between` - Between (value must be within the specified range defined by two numbers).\n  * `message` (optional) - Custom message returned in case of error.","ref":"Dsv.Number.html#validate/2-parameters"},{"type":"function","title":"Returns - Dsv.Number.validate/2","doc":"- `:ok` if the `number` meets all the specified validation criteria.\n- `{:error, message}` if the `number` fails to meet any of the specified criteria.","ref":"Dsv.Number.html#validate/2-returns"},{"type":"function","title":"Examples - Dsv.Number.validate/2","doc":"`:gt` example:\n    iex> Dsv.Number.validate(4, gt: 3)\n    :ok\n\n    iex> Dsv.Number.validate(4, gt: 4)\n    {:error, \"Value 4 must be greater than: 4\"}\n\n    iex> Dsv.Number.validate(4, gt: 5)\n    {:error, \"Value 4 must be greater than: 5\"}\n\n    iex> Dsv.Number.validate(4, gt: 5, message: \"This number is too small.\")\n    {:error, \"This number is too small.\"}\n\n`:gte` example:\n    iex> Dsv.Number.validate(4, gte: 3)\n    :ok\n\n    iex> Dsv.Number.validate(4, gte: 4)\n    :ok\n\n    iex> Dsv.Number.validate(4, gte: 5)\n    {:error, \"Value 4 must be greater than or equal: 5\"}\n\n    iex> Dsv.Number.validate(2, gte: 5, message: fn\n    ...>  data, _options when data > 3 -> \"This value is too small.\"\n    ...>  _data, _options -> \"This value is way too small.\"\n    ...> end)\n    {:error, \"This value is way too small.\"}\n\n\n`:lt` example:\n    iex> Dsv.Number.validate(4, lt: 3)\n    {:error, \"Value 4 must be lower than: 3\"}\n\n    iex> Dsv.Number.validate(4, lt: 4)\n    {:error, \"Value 4 must be lower than: 4\"}\n\n    iex> Dsv.Number.validate(4, lt: 4, message: \"Please provide smaller number.\")\n    {:error, \"Please provide smaller number.\"}\n\n\n    iex> Dsv.Number.validate(4, lt: 5)\n    :ok\n\n`:lte` example:\n    iex> Dsv.Number.validate(4, lte: 3)\n    {:error, \"Value 4 must be lower than or equal: 3\"}\n\n    iex> Dsv.Number.validate(4, lte: 4)\n    :ok\n\n    iex> Dsv.Number.validate(4, lte: 5)\n    :ok\n\n`:gt & :lt` example:\n    iex> Dsv.Number.validate(4, gt: 3, lt: 5)\n    :ok\n\n    iex> Dsv.Number.validate(4, gt: 4, lt: 5)\n    {:error, \"Value 4 must be lower than: 5 and greater than: 4\"}\n\n    iex> Dsv.Number.validate(4, gt: 2, lt: 4)\n    {:error, \"Value 4 must be lower than: 4 and greater than: 2\"}\n\n`:gte & :lt` example:\n    iex> Dsv.Number.validate(4, gte: 3, lt: 5)\n    :ok\n\n    iex> Dsv.Number.validate(4, gte: 4, lt: 5)\n    :ok\n\n    iex> Dsv.Number.validate(4, gte: 2, lt: 4)\n    {:error, \"Value 4 must be lower than: 4 and greater than or equal: 2\"}\n\n`:gt & :lte` example:\n    iex> Dsv.Number.validate(4, gt: 3, lte: 5)\n    :ok\n\n    iex> Dsv.Number.validate(4, gt: 4, lte: 5)\n    {:error, \"Value 4 must be lower than or equal: 5 and greater than: 4\"}\n\n    iex> Dsv.Number.validate(4, gt: 2, lte: 4)\n    :ok\n\n`:gte & :lte` example:\n    iex> Dsv.Number.validate(4, gte: 3, lte: 5)\n    :ok\n\n    iex> Dsv.Number.validate(4, gte: 4, lte: 5)\n    :ok\n\n    iex> Dsv.Number.validate(4, gte: 2, lte: 4)\n    :ok\n\n    iex> Dsv.Number.validate(1, gte: 2, lte: 4)\n    {:error, \"Value 1 must be lower than or equal: 4 and greater than or equal: 2\"}\n\n    iex> Dsv.Number.validate(5, gte: 2, lte: 4)\n    {:error, \"Value 5 must be lower than or equal: 4 and greater than or equal: 2\"}\n\n`:between` example:\n    iex> Dsv.Number.validate(4, between: {3, 5})\n    :ok\n\n    iex> Dsv.Number.validate(4, between: {4, 5})\n    :ok\n\n    iex> Dsv.Number.validate(4, between: {2, 4})\n    :ok\n\n    iex> Dsv.Number.validate(1, between: {2, 4})\n    {:error, \"Value 1 must be lower than or equal: 4 and greater than or equal: 2\"}\n\n    iex> Dsv.Number.validate(5, between: {2, 4})\n    {:error, \"Value 5 must be lower than or equal: 4 and greater than or equal: 2\"}\n\n`:eq` example:\n    iex> Dsv.Number.validate(4, eq: 4)\n    :ok\n\n    iex> Dsv.Number.validate(4, eq: 5)\n    {:error, \"Value 4 must be equal to value 5\"}\n\n`wrong options` example:\n    iex> Dsv.Number.validate(1, lt: 2, gt: \"not a number\")\n    {:error, \"Value 1 must be lower than: 2 and greater than: not a number\"}\n\n    iex> Dsv.Number.validate(\"wrong data\", between: {1, 5})\n    {:error, ~s(Value \"wrong data\" must be lower than or equal: 5 and greater than or equal: 1)}","ref":"Dsv.Number.html#validate/2-examples"},{"type":"function","title":"Dsv.Number.validate/3","doc":"","ref":"Dsv.Number.html#validate/3"},{"type":"function","title":"Dsv.Number.value/2","doc":"","ref":"Dsv.Number.html#value/2"},{"type":"module","title":"Dsv.Or","doc":"Dsv.Or module provides functions to validate a value against multiple validators.\nThe validation success as soon as at least one of the validators passes.","ref":"Dsv.Or.html"},{"type":"function","title":"Dsv.Or.valid?/2","doc":"The `valid?/2` function checks if a provided value passes any of the specified validators.","ref":"Dsv.Or.html#valid?/2"},{"type":"function","title":"Parameters - Dsv.Or.valid?/2","doc":"* `value` - The value to be validated.\n  * `validators` - A list of validators.","ref":"Dsv.Or.html#valid?/2-parameters"},{"type":"function","title":"Returns - Dsv.Or.valid?/2","doc":"A boolean value:\n\n- `true` if the `value` passes at least one of the specified validators.\n- `false` if the `value` fails all of the specified validators.","ref":"Dsv.Or.html#valid?/2-returns"},{"type":"function","title":"Example - Dsv.Or.valid?/2","doc":"iex> Dsv.Or.valid?(\"testowy\", equal: \"tes\", length: [min: 1, max: 20], format: ~r/t.*wyc/)\n    :true\n\n    iex> Dsv.Or.valid?(\"testowy\", equal: \"tes\", length: [min: 1, max: 2], format: ~r/t.*wyc/)\n    :false\n\n\nThis code will check if value is equal to \"tes\" and has min length of 2 or it has min length of 1, max length of 2 or match regular expression /t.*wyc/.\n\n    iex> Dsv.Or.valid?(\"testowy\", [[equal: \"tes\", length: [min: 2]], length: [min: 1, max: 2], format: ~r/t.*wyc/])\n    :false","ref":"Dsv.Or.html#valid?/2-example"},{"type":"function","title":"Dsv.Or.valid?/3","doc":"","ref":"Dsv.Or.html#valid?/3"},{"type":"function","title":"Dsv.Or.validate/2","doc":"The `validate/2` function checks if a provided value passes any of the specified validators.","ref":"Dsv.Or.html#validate/2"},{"type":"function","title":"Parameters - Dsv.Or.validate/2","doc":"* `value` - The value to be validated.\n  * `validators` - A list of validators.\n  * `message` - An optional custom message that will be returned in case of error.","ref":"Dsv.Or.html#validate/2-parameters"},{"type":"function","title":"Returns - Dsv.Or.validate/2","doc":"- `:ok` if the `value` passes at least one of the specified validators.\n- `{:error, message}` if the `value` fails all of the specified validators.","ref":"Dsv.Or.html#validate/2-returns"},{"type":"function","title":"Example - Dsv.Or.validate/2","doc":"iex> Dsv.Or.validate(\"testowy\", equal: \"tes\", length: [min: 1, max: 20], format: ~r/t.*wyc/)\n    :ok\n\n    iex> Dsv.Or.validate(\"testowy\", equal: \"tes\", length: [min: 1, max: 2], format: ~r/t.*wyc/)\n    {:error, \"At least one validator must be valid.\"}\n\n    iex> Dsv.Or.validate(\"testowy\", [[equal: \"tes\", length: [min: 2]], length: [min: 1, max: 2], format: ~r/t.*wyc/])\n    {:error, \"At least one validator must be valid.\"}\n\n    iex> Dsv.Or.validate(\"testowy\", equal: \"tes\", length: [min: 1, max: 2], format: ~r/t.*wyc/, message: \"You will never provide valid value.\")\n    {:error, \"You will never provide valid value.\"}\n\n\n    iex> Dsv.Or.validate(\"testowy\", [[equal: \"testowy\", length: [min: 2]], length: [min: 1, max: 2], format: ~r/t.*wyc/])\n    :ok\n\n    iex> Dsv.Or.validate(\"testowy\", [[equal: \"testowy\", length: [max: 2]], length: [min: 1, max: 20], format: ~r/t.*wyc/])\n    :ok","ref":"Dsv.Or.html#validate/2-example"},{"type":"function","title":"Dsv.Or.validate/3","doc":"","ref":"Dsv.Or.html#validate/3"},{"type":"function","title":"Dsv.Or.value/2","doc":"","ref":"Dsv.Or.html#value/2"},{"type":"module","title":"Dsv.Type","doc":"Ensure that input data are of the typed defined in the validator.","ref":"Dsv.Type.html"},{"type":"function","title":"Dsv.Type.valid?/2","doc":"Ensure that the data are of the type defined as a second argument.\n\nReturns `:true` if data are of the type defined as a second argument, otherwise, returns `:false`.","ref":"Dsv.Type.html#valid?/2"},{"type":"function","title":"Example - Dsv.Type.valid?/2","doc":"iex> Dsv.Type.valid?(\"abcd\", :string)\n\t:true\n\n\tiex> Dsv.Type.valid?(1, :string)\n\t:false\n\n\tiex> Dsv.Type.valid?(:true, :boolean)\n\t:true\n\n\tiex> Dsv.Type.valid?(:hello, :boolean)\n\t:false\n\n\tiex> Dsv.Type.valid?(:test_atom, :atom)\n\t:true\n\n\tiex> Dsv.Type.valid?(\"test text\", :atom)\n\t:false\n\n\tiex> Dsv.Type.valid?(134, :number)\n\t:true\n\n\tiex> Dsv.Type.valid?(12.4, :number)\n\t:true\n\n\tiex> Dsv.Type.valid?(%{}, :number)\n\t:false\n\n\tiex> Dsv.Type.valid?(2, :integer)\n\t:true\n\n\tiex> Dsv.Type.valid?(3.2, :integer)\n\t:false\n\n\tiex> Dsv.Type.valid?(3.2, :float)\n\t:true\n\n\tiex> Dsv.Type.valid?(3, :float)\n\t:false\n\n\tiex> Dsv.Type.valid?([1, 2, 4], :list)\n\t:true\n\n\tiex> Dsv.Type.valid?({1, 2, 4}, :list)\n\t:false\n\n\tiex> Dsv.Type.valid?({1, 2, 4}, :tuple)\n\t:true\n\n\tiex> Dsv.Type.valid?([1, 2, 4], :tuple)\n\t:false\n\n\tiex> Dsv.Type.valid?(%{a: :b}, :map)\n\t:true\n\n\tiex> Dsv.Type.valid?({:a, :b}, :map)\n\t:false\n\n\tiex> Dsv.Type.valid?([a: 1, b: 2], :keyword_list)\n\t:true\n\n\tiex> Dsv.Type.valid?([:a, :b], :keyword_list)\n\t:false\n\n\tiex> Dsv.Type.valid?(~U[2020-01-01 01:01:01.01Z], :date_time)\n\t:true\n\n\tiex> Dsv.Type.valid?(~D[2020-01-01], :date_time)\n\t:false\n\n\tiex> Dsv.Type.valid?(~D[2020-01-01], :date)\n\t:true\n\n\tiex> Dsv.Type.valid?(~U[2020-01-01 01:01:01.01Z], :date)\n\t:false\n\n\tiex> Dsv.Type.valid?(~T[01:01:01], :time)\n\t:true\n\n\tiex> Dsv.Type.valid?(~D[2020-01-01], :time)\n\t:false\n\n\tiex> Dsv.Type.valid?(~N[2020-01-01 01:01:01], :naive_date_time)\n\t:true\n\n\tiex> Dsv.Type.valid?(~U[2020-01-01 01:01:01.01Z], :naive_date_time)\n\t:false","ref":"Dsv.Type.html#valid?/2-example"},{"type":"function","title":"Dsv.Type.valid?/3","doc":"","ref":"Dsv.Type.html#valid?/3"},{"type":"function","title":"Dsv.Type.validate/2","doc":"Ensure that the data are of the type defined as a second argument.\n\nReturns `:ok` if data are of the type defined as a second argument, otherwise, returns `{:error, message}`.","ref":"Dsv.Type.html#validate/2"},{"type":"function","title":"Example - Dsv.Type.validate/2","doc":"iex> Dsv.Type.validate(\"abcd\", :string)\n\t:ok\n\n\tiex> Dsv.Type.validate(1, :string)\n\t{:error, \"Wrong type. Expected type is string\"}\n\n\tiex> Dsv.Type.validate(:true, :boolean)\n\t:ok\n\n\tiex> Dsv.Type.validate(:hello, :boolean)\n\t{:error, \"Wrong type. Expected type is boolean\"}\n\n\tiex> Dsv.Type.validate(:test_atom, :atom)\n\t:ok\n\n\tiex> Dsv.Type.validate(\"test text\", :atom)\n\t{:error, \"Wrong type. Expected type is atom\"}\n\n\tiex> Dsv.Type.validate(134, :number)\n\t:ok\n\n\tiex> Dsv.Type.validate(12.4, :number)\n\t:ok\n\n\tiex> Dsv.Type.validate(%{}, :number)\n\t{:error, \"Wrong type. Expected type is number\"}\n\n\tiex> Dsv.Type.validate(2, :integer)\n\t:ok\n\n\tiex> Dsv.Type.validate(3.2, :integer)\n\t{:error, \"Wrong type. Expected type is integer\"}\n\n\tiex> Dsv.Type.validate(3.2, :float)\n\t:ok\n\n\tiex> Dsv.Type.validate(3, :float)\n\t{:error, \"Wrong type. Expected type is float\"}\n\n\tiex> Dsv.Type.validate([1, 2, 4], :list)\n\t:ok\n\n\tiex> Dsv.Type.validate({1, 2, 4}, :list)\n\t{:error, \"Wrong type. Expected type is list\"}\n\n\tiex> Dsv.Type.validate({1, 2, 4}, :tuple)\n\t:ok\n\n\tiex> Dsv.Type.validate([1, 2, 4], :tuple)\n\t{:error, \"Wrong type. Expected type is tuple\"}\n\n\tiex> Dsv.Type.validate(%{a: :b}, :map)\n\t:ok\n\n\tiex> Dsv.Type.validate({:a, :b}, :map)\n\t{:error, \"Wrong type. Expected type is map\"}\n\n\tiex> Dsv.Type.validate([a: 1, b: 2], :keyword_list)\n\t:ok\n\n\tiex> Dsv.Type.validate([:a, :b], :keyword_list)\n\t{:error, \"Wrong type. Expected type is keyword_list\"}\n\n\tiex> Dsv.Type.validate(~U[2020-01-01 01:01:01.01Z], :date_time)\n\t:ok\n\n\tiex> Dsv.Type.validate(~D[2020-01-01], :date_time)\n\t{:error, \"Wrong type. Expected type is date_time\"}\n\n\tiex> Dsv.Type.validate(~D[2020-01-01], :date)\n\t:ok\n\n\tiex> Dsv.Type.validate(~U[2020-01-01 01:01:01.01Z], :date)\n\t{:error, \"Wrong type. Expected type is date\"}\n\n\tiex> Dsv.Type.validate(~T[01:01:01], :time)\n\t:ok\n\n\tiex> Dsv.Type.validate(~D[2020-01-01], :time)\n\t{:error, \"Wrong type. Expected type is time\"}\n\n\tiex> Dsv.Type.validate(~N[2020-01-01 01:01:01], :naive_date_time)\n\t:ok\n\n\tiex> Dsv.Type.validate(~U[2020-01-01 01:01:01.01Z], :naive_date_time)\n\t{:error, \"Wrong type. Expected type is naive_date_time\"}","ref":"Dsv.Type.html#validate/2-example"},{"type":"function","title":"Dsv.Type.validate/3","doc":"","ref":"Dsv.Type.html#validate/3"},{"type":"function","title":"Dsv.Type.value/2","doc":"","ref":"Dsv.Type.html#value/2"},{"type":"protocol","title":"DataLength","doc":"DataLength protocol defines a function to retrieve the length of data for various data types.\nImplemenations of this protocol need to implement `len(data)` function.","ref":"DataLength.html"},{"type":"protocol","title":"Default Implementations - DataLength","doc":"The protocol provides default implementations for the following data types:\n  - `String`: Returns the number of graphemes in the string.\n  - `List`: Returns the number of elements in the list.\n  - `Map`: Returns the number of key-value pairs at the first level of the map.\n  - `Tuple`: Returns the number of elements in the tuple.\n  - `Integer`: Returns the number of digits in the integer plus 1 for a minus sign if the number is negative.\n  - `Float`: Returns the number of digits in the float plus 1 for a minus sign if the number is negative and 1 for a \".\".\n\n\n#","ref":"DataLength.html#module-default-implementations"},{"type":"protocol","title":"Default implementations examples: - DataLength","doc":"`BitString` implementation returns the number of graphemes:\n    iex> DataLength.len(\"Hello! Dzień dobry!\")\n    19\n\n`List` implementation returns the number of elements in the list.\n    iex> DataLength.len([\"a\", :b, [\"c\"], %{d: \"d\"}])\n    4\n\n`Map` implementation returns the number of key-value pairs at the first level of the map.:\n    iex> DataLength.len(%{a: 1, b: [2], c: \"hello\"})\n    3\n\n`Tuple` implementation returns the number of elements in the tuple:\n    iex> DataLength.len({\"a\", :b, [\"c\", \"d\"], {\"e\"}, \"f\"})\n    5\n\n`Integer` implementation returns the number of digits in the integer plus 1 for a minus sign if the number is negative:\n    iex> DataLength.len(1234567890)\n    10\n\n    iex> DataLength.len(-1234567890)\n    11\n\n`Float` implemantation returns the number of digits in the float plus 1 for a minus sign if the number is negative and 1 for a \".\". Terminal zeros in fractional part are ignored:\n    iex> DataLength.len(12345.6789)\n    10\n\n    iex> DataLength.len(12345.67890)\n    10\n\n    iex> DataLength.len(12345.67899)\n    11\n\n    iex> DataLength.len(-12345.67890)\n    11\n\n    iex> DataLength.len(-12345.67899)\n    12","ref":"DataLength.html#module-default-implementations-examples"},{"type":"protocol","title":"Custom implementation: - DataLength","doc":"You can implement custom length calculations for other data types by providing specific protocol implementations.\n\n#","ref":"DataLength.html#module-custom-implementation"},{"type":"protocol","title":"Example - DataLength","doc":"defmodule MyStruct do\n        defstruct data: [1, 2, 3]\n      end\n\n      defimpl Length, for: MyStruct do\n        def len(%MyStruct{data: data}), do: length(data)\n      end","ref":"DataLength.html#module-example"},{"type":"function","title":"DataLength.len/1","doc":"Retrieves the length of the input data.","ref":"DataLength.html#len/1"},{"type":"function","title":"Parameters - DataLength.len/1","doc":"* `data` - The data for which to calculate the length.","ref":"DataLength.html#len/1-parameters"},{"type":"function","title":"Returns - DataLength.len/1","doc":"An integer representing the length of the data.","ref":"DataLength.html#len/1-returns"},{"type":"type","title":"DataLength.t/0","doc":"All the types that implement this protocol.","ref":"DataLength.html#t:t/0"},{"type":"protocol","title":"Dsv.Empty","doc":"A protocol to check if a provided value is empty.\nThis protocol is used by the `Dsv.NotEmpty` validator.","ref":"Dsv.Empty.html"},{"type":"protocol","title":"Default Implementations - Dsv.Empty","doc":"The protocol provides default implementations for the following data types:\n- `List`: Checks if the list has a length of 0.\n- `Map`: Checks if the map has no key-value pairs.\n- `Tuple`: Checks if the tuple has no elements.\n- `String`: Checks if the string has a length of 0.\n- `Integer`: Always return :false, integer can not be empty.\n- `Float`: Always return :false, float can not be empty.\n- `Date`: Always return :false, date can not be empty.\n- `DateTime`: Always return :false, date time can not be empty.\n- `NaiveDateTime`: Always return :false, naive date time can not be empty.\n- `Time`: Always return :false, time can not be empty.\n- `Atom`: Checks if the atom is `:nil` or `:false`.\n\nYou can implement custom emptiness checks for other data types by providing specific protocol implementations.","ref":"Dsv.Empty.html#module-default-implementations"},{"type":"protocol","title":"Example - Dsv.Empty","doc":"defmodule MyStruct do\n      defstruct data: [:a, :b, :c]\n    end\n\n    defimpl Empty, for: MyStruct do\n      def empty?(%MyStruct{data: data}), do: length(data) == 0\n    end","ref":"Dsv.Empty.html#module-example"},{"type":"function","title":"Dsv.Empty.empty?/1","doc":"Return `:true` if the value is empty. Otherwise, return `:false`.\n\n\nFor `List`, it will return `:true` only for the `[]` list. In other cases, it will return `:false`.\n\nFor `Map`, it will return `:true` only for the `%{}` map. In other cases, it will return `:false`.\n\nFor `Tuple`, it will return `:true` only for `{}` tuple. In other cases, it will return `:false`.\n\nFor `String`, it will return `:true` only for `\"\"` string. In other cases, it will return `:false`.\n\nFor `Integer`, `Float`, `Date`, `DateTime`, `NaiveDateTime`, and `Time`, it will always return `:false` as if value of this type exists, it cannot be empty.\n\nFor `Atom`, it will return `:true` for values `:false` and `nil`. In other cases, it will return `:false`.","ref":"Dsv.Empty.html#empty?/1"},{"type":"function","title":"Example - Dsv.Empty.empty?/1","doc":"iex> Dsv.Empty.empty?(:false)\n    :true\n\n    iex> Dsv.Empty.empty?(\"\")\n    :true\n\n    iex> Dsv.Empty.empty?(\"not empty\")\n    :false\n\n    iex> Dsv.Empty.empty?([])\n    :true\n\n    iex> Dsv.Empty.empty?(:true)\n    :false\n\n    iex> Dsv.Empty.empty?(1.1)\n    :false\n\n    iex> Dsv.Empty.empty?(~D[2000-01-01])\n    :false\n\n    iex> Dsv.Empty.empty?(~U[2000-01-01 11:11:11Z])\n    :false\n\n    iex> Dsv.Empty.empty?(~N[2000-01-01 11:11:11])\n    :false\n\n    iex> Dsv.Empty.empty?(~T[11:11:11])\n    :false","ref":"Dsv.Empty.html#empty?/1-example"},{"type":"type","title":"Dsv.Empty.t/0","doc":"All the types that implement this protocol.","ref":"Dsv.Empty.html#t:t/0"},{"type":"protocol","title":"Dsv.EqualProto","doc":"A protocol to check if two values are equal.\nThis protocol is used by the `Dsv.Equal` validator.\n\n\nYou can implement custom equality checks for other data types by providing specific protocol implementations.","ref":"Dsv.EqualProto.html"},{"type":"protocol","title":"Example - Dsv.EqualProto","doc":"defmodule User do\n      defstruct [:first_name, :last_name, :age, :id_number]\n    end\n\n    defimpl Equal, for: User do\n      def equal?(%User{first_name: first_name1, last_name: last_name1, age: age1, id_number: id_number1}, %User{first_name: first_name2, last_name: last_name2, age: age2, id_number: id_number2}) do\n        first_name1 === first_name2 and last_name1 === last_name2 and age1 === age2 and id_number1 === id_number2\n      end\n    end","ref":"Dsv.EqualProto.html#module-example"},{"type":"function","title":"Dsv.EqualProto.is_equal/2","doc":"Checks if two values are equal.","ref":"Dsv.EqualProto.html#is_equal/2"},{"type":"function","title":"Parameters - Dsv.EqualProto.is_equal/2","doc":"* `a` - The first value to compare.\n* `b` - The second value to compare.","ref":"Dsv.EqualProto.html#is_equal/2-parameters"},{"type":"function","title":"Returns - Dsv.EqualProto.is_equal/2","doc":"- `true` if `a` and `b` are considered equal.\n- `false` otherwise.","ref":"Dsv.EqualProto.html#is_equal/2-returns"},{"type":"type","title":"Dsv.EqualProto.t/0","doc":"All the types that implement this protocol.","ref":"Dsv.EqualProto.html#t:t/0"},{"type":"protocol","title":"Iterable","doc":"Protocol used to iterate over elements of the iterable. This protocol is used in `Dsv.Any`, `Dsv.All` and `Dsv.None` validators.\n\nThe first parameter is the data we want to go through.\nThe second is the functions that receives one element of the iterable at a time and return one of the value:\n  * :cont - the iteration should be continue\n  * :done - the iteration should be halt immediately\n\n\nThere are default implementations for: `List`, `BitString` and `Tuple`.\n\n\n`BitString` implementation goes through graphemes:\n    iex> Iterable.iterate(\"Hello! Dzień dobry!\", :cont, fn elem -> if elem == \"D\", do: :halt, else: :cont end)\n    {:done, :not_empty}\n\n    iex> Iterable.iterate(\"Hello! Dzień dobry!\", :cont, fn elem -> if elem == \"e\", do: :halt, else: :cont end)\n    {:done, :not_empty}\n\n    iex> Iterable.iterate(\"Hello! Dzień dobry!\", :cont, fn elem -> if elem == \"h\", do: :halt, else: :cont end)\n    {:done, :empty}\n\n\n`List`  implementation goes through all elements of the list:\n    iex> Iterable.iterate([\"a\", :b, [\"c\"], %{d: \"d\"}], :cont, fn elem -> if elem == [\"c\"], do: :halt, else: :cont end)\n    {:done, :not_empty}\n\n    iex> Iterable.iterate([\"a\", :b, [\"c\"], %{d: \"d\"}], :cont, fn elem -> if elem == %{d: \"d\"}, do: :halt, else: :cont end)\n    {:done, :not_empty}\n\n`Tuple` implementation goes throught all elements of the tuple:\n    iex> Iterable.iterate({\"a\", :b, [\"c\"], %{d: \"d\"}}, :cont, fn elem -> if elem == [\"c\"], do: :halt, else: :cont end)\n    {:done, :not_empty}\n\n    iex> Iterable.iterate({\"a\", :b, [\"c\"], %{d: \"d\"}}, :cont, fn elem -> if elem == %{d: \"d\"}, do: :halt, else: :cont end)\n    {:done, :not_empty}\n\n    iex> Iterable.iterate({:a, :b, :c}, :cont, fn elem -> if elem == :d, do: :halt, else: :cont end)\n    {:done, :empty}\n\n`Map` implementation goes throught all elements of the map:\n    iex> Iterable.iterate(%{a: :b, c: :d}, :cont, fn {_key, value} -> if value == :d, do: :halt, else: :cont end)\n    {:done, :not_empty}\n\n    iex> Iterable.iterate(%{a: :b, c: :d}, :cont, fn {_key, value} -> if value == :e, do: :halt, else: :cont end)\n    {:done, :empty}\n\n    iex> Iterable.iterate(%{}, :cont, fn {_key, value} -> if value == :e, do: :halt, else: :cont end)\n    {:done, :empty}","ref":"Iterable.html"},{"type":"function","title":"Iterable.iterate/3","doc":"Go through every element and provide that element to the `t:next/0` function.","ref":"Iterable.html#iterate/3"},{"type":"type","title":"Iterable.element/0","doc":"The value for each step.","ref":"Iterable.html#t:element/0"},{"type":"type","title":"Iterable.next/0","doc":"The next function","ref":"Iterable.html#t:next/0"},{"type":"type","title":"Iterable.result/0","doc":"The result value for iterate function.","ref":"Iterable.html#t:result/0"},{"type":"type","title":"Iterable.t/0","doc":"All the types that implement this protocol.","ref":"Iterable.html#t:t/0"},{"type":"type","title":"Iterable.tag/0","doc":"It must be a value that is one of the following \"tags\":\n\n  * `:cont`    - the iteration should continue\n  * `:halt`    - the iteration should halt immediately","ref":"Iterable.html#t:tag/0"},{"type":"protocol","title":"ValueAt","doc":"A protocol to retrive an element at a specific position within a data structure.","ref":"ValueAt.html"},{"type":"protocol","title":"Default Implementations - ValueAt","doc":"The protocol provides default implementations for common data types:\n- `BitString`: Retrieves the grapheme at the specified position.\n- `List`: Retrieves the element at the specified index.\n\nPosition start from zero:\n    iex> ValueAt.at(\"Hello! Dzień dobry!\", 5)\n    \"!\"\n\nImplementation return value at the given position:\n    iex> ValueAt.at([\"a\", :b, [\"c\"], %{d: \"d\"}], 3)\n    %{d: \"d\"}\n\nIn both implementations, `List` and `BitString`, when an index is out of bounds, `nil` is returned.\n    iex> ValueAt.at(\"Hello\", 10)\n    nil","ref":"ValueAt.html#module-default-implementations"},{"type":"protocol","title":"Custom implementation example - ValueAt","doc":"Implementation of this protocl require a `ValueAt.at(data, position)` function.\n\n    defmodule MyStruct do\n      defstruct data: [1, 2, 3]\n    end\n\n    defimpl At, for: MyStruct do\n      def at(%MyStruct{data: data}, index) when is_integer(index) and index >= 0 do\n        Enum.at(data, index)\n      end\n    end","ref":"ValueAt.html#module-custom-implementation-example"},{"type":"function","title":"ValueAt.at/2","doc":"Retrieves an element at the specified position within the data structure.","ref":"ValueAt.html#at/2"},{"type":"function","title":"Parameters - ValueAt.at/2","doc":"* `data` - The data structure from which to retrieve the element.\n* `index` - The position (index) at which to retrieve the element.","ref":"ValueAt.html#at/2-parameters"},{"type":"function","title":"Returns - ValueAt.at/2","doc":"- The element at the specified position within the data structure.\n- If position is out of bound, `nil` should be returned.","ref":"ValueAt.html#at/2-returns"},{"type":"type","title":"ValueAt.t/0","doc":"All the types that implement this protocol.","ref":"ValueAt.html#t:t/0"},{"type":"behaviour","title":"Dsv.Comparator","doc":"Specify the function that needs to be implemented by any validator used for comparing two elements from the validator input.","ref":"Dsv.Comparator.html"},{"type":"callback","title":"Dsv.Comparator.to_comparator/2","doc":"Create validator options from value and options.","ref":"Dsv.Comparator.html#c:to_comparator/2"},{"type":"behaviour","title":"Dsv.Validator","doc":"Validate user data\n\nA behavior module for implementing a validator.\n\nA Dsv.Validator is a module that defines a few useful macros that help when implementing a validator.\nA Dsv.Validator provides `validate/2` function that will validate the input based on the function `valid?/2` that needs to be provided by the `Dsv.Validator` implementation.\nA Dsv.Validator behavior abstracts the common validation functions and creation of error messages (in line with validator options provided by the user - more on that later).\n\nA `Dsv.Validator` is a convenient way to create new validators which will handle error response generation and are a way to create validators that can be used with `Dsv.validate` function.","ref":"Dsv.Validator.html"},{"type":"behaviour","title":"Example - Dsv.Validator","doc":"To create a simple validator, we need to create a new module that will use our `Dsv.Validator` behavior.\nThe only required thing to be implemented is the `valid?/2` function, which will get data to validate as a first argument and validator options as a second argument.\n\n    iex> defmodule Dsv.IsTrue do\n    ...>   use Dsv.Validator\n    ...>\n    ...>   def valid?(data, options \\\\ []), do: if data == :true, do: :true, else: :false\n    ...> end\n\nThis is the smallest working example of a new validator.\n\nThis can be used in a few ways.\n\nThe first way is to check if the value is valid using the `Dsv.IsTrue.valid?/2` function. This will return `:true` if `data` is `:true` and `:false` otherwise.\n\n    iex> Dsv.IsTrue.valid?(:true)\n    :true\n\n    iex> Dsv.IsTrue.valid?(:false)\n    :false\n\n    iex> Dsv.IsTrue.valid?(\"this is not true\")\n    :false\n\nUsing `Dsv.Validator` provides another useful function, `Dsv.IsTrue.validate/2`. In this case, the result will be `:ok` in case the data is valid (`:true`) and `{:error, message}` in case the data is invalid (`:false`).\n\n    iex> Dsv.IsTrue.validate(:true)\n    :ok\n\n    iex> Dsv.IsTrue.validate(:false)\n    {:error, \"Unknown validation error\"}\n\nOn validation failure, we get a tuple with `:error` as the first element and the default error message \"Unknown validation error\". This error message is not very helpful if we are interested in the reason for the failure.\nWe can change it by defining error messages specific to the validator options.\n\nWe do not use validator options in the above example, so the error message definition will look like this (it will not be specific to validation options).\n\n\n    iex> defmodule Dsv.IsTrue do\n    ...>   use Dsv.Validator\n    ...>\n    ...>   message \"Provided value is invalid. Accepted value is ':true'\"\n    ...>\n    ...>   def valid?(data, options \\\\ []), do: if data == :true, do: :true, else: :false\n    ...> end\n\n    iex> Dsv.IsTrue.validate(:false)\n    {:error, \"Provided value is invalid. Accepted value is ':true'\"}\n\n\nWhen we define a validator that will get data to validate and validate options, we can create a message for each possible options combination.\n\n    iex> defmodule Dsv.StringLength do\n    ...>   use Dsv.Validator\n    ...>\n    ...>   message {:min, \"Value is too short\"}\n    ...>   message {:max, \"Value is too long\"}\n    ...>   message {[:min, :max], \"Value is too short or too long\"}\n    ...>\n    ...>   def valid?(data, [{:min, min_length}]), do: String.length(data) > min_length\n    ...>   def valid?(data, [{:max, max_length}]), do: String.length(data)     def valid?(data, [{:min, min_length}, {:max, max_length}]), do: valid?(data, min: min_length) and valid?(data, max: max_length)\n    ...> end\n\nIn this case, when we use `Dsv.StringLength.validate/2` function, we will get a message related to what we want to validate (options we provided).\n\n    iex> Dsv.StringLength.validate(\"test\", min: 6)\n    {:error, \"Value is too short\"}\n\n    iex> Dsv.StringLength.validate(\"test\", max: 2)\n    {:error, \"Value is too long\"}\n\n    iex> Dsv.StringLength.validate(\"test\", min: 10, max: 100)\n    {:error, \"Value is too short or too long\"}\n\n##","ref":"Dsv.Validator.html#module-example"},{"type":"behaviour","title":"Defining messages with data and options information. - Dsv.Validator","doc":"Besides simple messages like the above, we can create messages containing data under validation and options used during validation.\nWe can use EEx (Embedded Elixir) to create such a message.\nWe can use `data`, which contains data provided as the first argument to the `validate/2` function, and `options`, which contains data provided as the second argument to the `validate/2` function.\n\n\n    iex> defmodule Dsv.StringLength do\n    ...>   use Dsv.Validator\n    ...>\n    ...>   message {:min, \"Value '<%= data %>' is shorter than expected <%= options[:min] %> characters.\"}\n    ...>   message {:max, \"Value '<%= data %>' is longer than expected <%= options[:max] %> characters.\"}\n    ...>   message {[:min, :max], \"Value '<%= data %>' is shorter than expected <%= options[:min] %> or longer than expected <%= options[:max] %> characters.\"}\n    ...>\n    ...>   def valid?(data, [{:min, min_length}]), do: String.length(data) > min_length\n    ...>   def valid?(data, [{:max, max_length}]), do: String.length(data)     def valid?(data, [{:min, min_length}, {:max, max_length}]), do: valid?(data, min: min_length) and valid?(data, max: max_length)\n    ...> end\n\n    iex> Dsv.StringLength.validate(\"test\", min: 6)\n    {:error, \"Value 'test' is shorter than expected 6 characters.\"}\n\n    iex> Dsv.StringLength.validate(\"test\", max: 2)\n    {:error, \"Value 'test' is longer than expected 2 characters.\"}\n\n    iex> Dsv.StringLength.validate(\"test\", min: 10, max: 100)\n    {:error, \"Value 'test' is shorter than expected 10 or longer than expected 100 characters.\"}\n\n\nWhen we expect the raise of an Exception during validation, we can catch that Exception and change it to the error message.\nWe must define the error message using macro `e_message/1` to do that.\n\n\n    iex> defmodule Dsv.StringLength do\n    ...>   use Dsv.Validator\n    ...>\n    ...>   message {:min, \"Value '<%= data %>' is shorter than expected <%= options[:min] %> characters.\"}\n    ...>   message {:max, \"Value '<%= data %>' is longer than expected <%= options[:max] %> characters.\"}\n    ...>   message {[:min, :max], \"Value '<%= data %>' is shorter than expected <%= options[:min] %> or longer than expected <%= options[:max] %> characters.\"}\n    ...>\n    ...>   e_message \"Unexpected error occured during validation. The data or options you provided are incorrect.\"\n    ...>\n    ...>   def valid?(data, [{:min, min_length}]), do: String.length(data) > min_length\n    ...>   def valid?(data, [{:max, max_length}]), do: String.length(data)     def valid?(data, [{:min, min_length}, {:max, max_length}]), do: valid?(data, min: min_length) and valid?(data, max: max_length)\n    ...> end\n\n    iex> Dsv.StringLength.validate(4443432, min: 6)\n    {:error, \"Unexpected error occured during validation. The data or options you provided are incorrect.\"}\n\nIf the `e_message` macro is not used to define an error message, then `FunctionClauseError` will be returned in this example.\n\n    iex> defmodule Dsv.StringLength do\n    ...>   use Dsv.Validator\n    ...>\n    ...>   message {:min, \"Value '<%= data %>' is shorter than expected <%= options[:min] %> characters.\"}\n    ...>   message {:max, \"Value '<%= data %>' is longer than expected <%= options[:max] %> characters.\"}\n    ...>   message {[:min, :max], \"Value '<%= data %>' is shorter than expected <%= options[:min] %> or longer than expected <%= options[:max] %> characters.\"}\n    ...>\n    ...>\n    ...>   def valid?(data, [{:min, min_length}]), do: String.length(data) > min_length\n    ...>   def valid?(data, [{:max, max_length}]), do: String.length(data)     def valid?(data, [{:min, min_length}, {:max, max_length}]), do: valid?(data, min: min_length) and valid?(data, max: max_length)\n    ...> end\n\n    iex> Dsv.StringLength.validate(4443432, min: 6)\n    ** (FunctionClauseError) no function clasue matching in String.length/1","ref":"Dsv.Validator.html#module-defining-messages-with-data-and-options-information"},{"type":"behaviour","title":"Different ways of defining messages. - Dsv.Validator","doc":"You can define validation messages in three different ways:\n\n- Using message macro (as seen above) with the options value and the message as EEx string.\n- Using function which will receive four arguments\n\nSee more in the `message/1` documentation.\n\n##","ref":"Dsv.Validator.html#module-different-ways-of-defining-messages"},{"type":"behaviour","title":"Map options to the message keys - Dsv.Validator","doc":"###","ref":"Dsv.Validator.html#module-map-options-to-the-message-keys"},{"type":"behaviour","title":"Default `message_key_mapper` implementation. - Dsv.Validator","doc":"By default, `:message_key_mapper` will return a list of the option's keys.\nThe order of the options in the `validate` function is not essential.\n\n\nThis is used with a combination of `message` macro. Let's assume you defined a few messages like that:","ref":"Dsv.Validator.html#module-default-message_key_mapper-implementation"},{"type":"behaviour","title":"Example - Dsv.Validator","doc":"iex> defmodule Dsv.Validator.MessageExample do\n    ...>   use Dsv.Validator\n    ...>\n    ...>   message {:first, \"This is the message produced when validator is call with option `:first`\"}\n    ...>   message {:second, \"This is the message produced when validator is call with option `:second`\"}\n    ...>   message {:third, \"This is the message produced when validator is call with option `:third`\"}\n    ...>   message {[:first, :second, :third], \"This is the message produced when validator is call with all of the options `:first`, `:second`, `:third`\"}\n    ...>\n    ...>   def valid?(data, options), do: if data == :fail, do: :false, else: :true\n    ...> end\n\nWhen you call validator\n    iex> DSV.Validator.MessageExample(:fail, first: \"first constraint\")\n\nand the validation will fail, then the produced message will be:\n> This is the message produced when validator is call with option `:first`\nas this is the message defined for the `:first` option\n\nIf validator is called with all options like that:\n    iex> DSV.Validator.MessageExample(:fail, first: \"first constraint\", second: \"second constraint\", third: \"third constraint\")\n\nand the validation will fail, then the produced message will be:\n>This is the message produced when validator is call with all of the options `:first`, `:second`, `:third`\nas this is the message defined for the options: [:first, :second, :third]\n\n###","ref":"Dsv.Validator.html#module-example"},{"type":"behaviour","title":"Custom `:message_key_mapper` implementation - Dsv.Validator","doc":"There is a possibility to define a custom mapping from validation options to the message keys. By default, validation options are mapped to message keys without changes. This behavior can be changed by defining the `:message_key_mapper` option of the `Dsv.Validator` module. `:message_key_mapper` must be a function that receives options as the only argument and returns a list of keys that are defined in the `message` macros.","ref":"Dsv.Validator.html#module-custom-message_key_mapper-implementation"},{"type":"behaviour","title":"Example - Dsv.Validator","doc":"iex> defmodule Dsv.Validator.MessageExample do\n      ...>  use Dsv.Validator, message_key_mapper: fn options -> if length(options) == 1, do: [:one_option_one_error], else: [:many_options] end\n      ...>\n      ...>  message {:one_option_one_error, \"Validation for only one options failed.\"}\n      ...>  message {:many_options, \"There are so many options. I do not know what has failed.\"}\n      ...>\n      ...>  def valid?(data, options \\\\ [])\n      ...>  def valid?(data, options), do: if data == :fail, do: :false, else: :true\n      ...> end\n\n\nIn this example, we assigned the function\n    fn options -> if length(options) == 1, do: [:one_option_one_error], else: [:many_options] end\nto the `:message_key_mapper` option.\nWhen you run the `validate` function on the `Dsv.Validator.MessageExample` module, and there is a failure (data parameter equal to `:fail`), the function assigned to the `:message_key_mapper` will be run with all options that you pass to the `validate` function.\nOur function will check the number of options you pass to the `validate` function and will return `:one_option_one_error` in case there is exactly one option in the method call or `:many_options` otherwise.\nThe value returned from the function assigned to the `:message_key_mapper` will be used to choose the error message from messages defined in the `message` macro.\n\n      iex> Dsv.Validator.MessageExample.validate(:fail, one_option: false)\n      {:error, \"Validation for only one options failed.\"}\n\n      iex> Dsv.Validator.MessageExample.validate(:fail, option_one: 1, option_two: 2)\n      {:error, \"There are so many options. I do not know what has failed.\"}\n\n#","ref":"Dsv.Validator.html#module-example"},{"type":"behaviour","title":"Complex validators - Dsv.Validator","doc":"Complex vaidator will gather all error messages from all the validators used for data validation.\n\n\nComplex validators allow to transform/split input data to different form/parts based on the validation options.\nTransformation can be something like getting part of the input as in the `Dsv.Email` or calling function on the data like in the below `Dsv.Path` example.\nTransformation need to be done by implementing `get_element/2` function if user will not provide own implementation then default one will be used.\nDefault implementation of `get_element/2` is just `Map.get/2` function.\n\nTo use above functionallity you need to set `:complex` option to `:true`.","ref":"Dsv.Validator.html#module-complex-validators"},{"type":"behaviour","title":"Example - Dsv.Validator","doc":"defmodule Dsv.Path do\n      use Dsv.Validator, complex: :true\n\n\n      message {:basename, \"Wrong basename.\"}\n      message {:dirname, \"Wrong dirname.\"}\n      message {:rootname, \"Wrong rootname.\"}\n      message {[:basename, :rootname, :dirname], \"hohoho\"}\n      message fn data, _options, errors -> \"Path \" <> data <> \" doesn't meet criteria \" <> (errors |> Map.values |> Enum.join(\" and \")) end\n\n      def get_element(path, :basename), do: Path.basename(path)\n      def get_element(path, :dirname), do: Path.dirname(path)\n      def get_element(path, :rootname), do: Path.rootname(path)\n    end\n\nNow we can use `Dsv.Path` to validate basic path info.\n\n    iex> Dsv.Path.validate(\"/a/b/c/d.e\", basename: [equal: \"d.e\"], dirname: [length: [min: 2, max: 20]])\n    :ok\n\n    iex> Dsv.Path.validate(\"/a/b/c/d.e\", basename: [equal: \"d.e\"], dirname: [length: [min: 2, max: 4]])\n    {:error, \"Path /a/b/c/d.e doesn't meet criteria Values must be equal and Value \"/a/b/c\" has wrong length. Minimum lenght is 2, maximum length is 4\"}\n\nIn the last example you can see that error message contain messages from `Dsv.Equal` and `Dsv.Length` validators combine together.\nThis is possible because when you define your validator with `:complex` option set to `true`, you can use in your custom error message any error from any validator used during validation of any element.","ref":"Dsv.Validator.html#module-example"},{"type":"macro","title":"Dsv.Validator.e_message/1","doc":"The `e_message` macro allow to define the message that the validator will return in case of an exception. There are two possible ways to define a message:\n  * Provide a plain string or string in a EEx format\n  * Provide a function that will receive three arguments:\n    * data - data for validation\n    * options - validation options\n    * exception - exception returned by the validator","ref":"Dsv.Validator.html#e_message/1"},{"type":"macro","title":"Example - Dsv.Validator.e_message/1","doc":"#","ref":"Dsv.Validator.html#e_message/1-example"},{"type":"macro","title":"Plain string - Dsv.Validator.e_message/1","doc":"defmodule Dsv.StringLength do\n      use Dsv.Validator\n   \n      message {:min, \"Value '<%= data %>' is shorter than expected <%= options[:min] %> characters.\"}\n      message {:max, \"Value '<%= data %>' is longer than expected <%= options[:max] %> characters.\"}\n      message {[:min, :max], \"Value '<%= data %>' is shorter than expected <%= options[:min] %> or longer than expected <%= options[:max] %> characters.\"}\n   \n      e_message \"Unexpected error occured during validation. The data or options you provided are incorrect.\"\n   \n      def valid?(data, [{:min, min_length}]), do: String.length(data) > min_length\n      def valid?(data, [{:max, max_length}]), do: String.length(data)   Dsv.StringLength.validate(4443432, min: 6)\n    {:error, \"Unexpected error occured during validation. The data or options you provided are incorrect.\"}\n\n#","ref":"Dsv.Validator.html#e_message/1-plain-string"},{"type":"macro","title":"String in EEx format - Dsv.Validator.e_message/1","doc":"The message can contain information from fields such as data, options, and exceptions, where data is the data passed to the validator, options are the options passed to the validator, and an exception is an exception thrown during validation.\n\n    defmodule Dsv.StringLength do\n      use Dsv.Validator\n   \n      message {:min, \"Value '<%= data %>' is shorter than expected <%= options[:min] %> characters.\"}\n      message {:max, \"Value '<%= data %>' is longer than expected <%= options[:max] %> characters.\"}\n      message {[:min, :max], \"Value '<%= data %>' is shorter than expected <%= options[:min] %> or longer than expected <%= options[:max] %> characters.\"}\n   \n      e_message \"Unexpected error <%= inspect exception %> occured during validation with data: [<%= inspect data %>] and options: [<%= inspect options %>].\"\n   \n      def valid?(data, [{:min, min_length}]), do: String.length(data) > min_length\n      def valid?(data, [{:max, max_length}]), do: String.length(data)   Dsv.StringLength.validate(4443432, min: 6)\n    {:error, \"Unexpected error %FunctionClauseError{args: nil, arity: 1, clauses: nil, function: :length, kind: nil, module: String} occured during validation with data: [4443432] and options: [[min: 6]].\"}\n\n\n#","ref":"Dsv.Validator.html#e_message/1-string-in-eex-format"},{"type":"macro","title":"Function - Dsv.Validator.e_message/1","doc":"The function will receive three arguments that we can use to create an error message:\n  * data - data for validation\n  * options - validation options\n  * exception - exception returned by the validator\n\n##","ref":"Dsv.Validator.html#e_message/1-function"},{"type":"macro","title":"Example - Dsv.Validator.e_message/1","doc":"iex> defmodule Dsv.StringLength do\n    ...>   use Dsv.Validator\n    ...>\n    ...>   message {:min, \"Value '<%= data %>' is shorter than expected <%= options[:min] %> characters.\"}\n    ...>   message {:max, \"Value '<%= data %>' is longer than expected <%= options[:max] %> characters.\"}\n    ...>   message {[:min, :max], \"Value '<%= data %>' is shorter than expected <%= options[:min] %> or longer than expected <%= options[:max] %> characters.\"}\n    ...>\n    ...>   e_message fn data, options, exception -> \"Unexpected error occured during '#{data} validation with options #{options}. The data or options you provided are incorrect. Exception: #{exception}.\" end\n    ...>\n    ...>   def valid?(data, [{:min, min_length}]), do: String.length(data) > min_length\n    ...>   def valid?(data, [{:max, max_length}]), do: String.length(data)     def valid?(data, [{:min, min_length}, {:max, max_length}]), do: valid?(data, min: min_length) and valid?(data, max: max_length)\n    ...> end\n\n    iex> Dsv.StringLength.validate(4443432, min: 6)\n    {:error, \"Unexpected error occured during '4443432 validation with options [min: 6]. The data or options you provided are incorrect. Exception: %FunctionClauseError{args: nil, arity: 1, clauses: nil, function: :length, kind: nil, module: String}.\"}","ref":"Dsv.Validator.html#e_message/1-example"},{"type":"macro","title":"Dsv.Validator.message/1","doc":"The `message` macro allows defining the message that the validator will return in case of a validation failure.\n\nThere are a few ways of defining a message for a validator:\n  * Provide a plain string\n  * Provide a function that returns a string\n  * Provide a tuple with keys as the first element and a message string as the second element.\n  * Provide a tuple with keys as the first element and a function that return a string as the second element.\n\n#","ref":"Dsv.Validator.html#message/1"},{"type":"macro","title":"Plain string - Dsv.Validator.message/1","doc":"Provide message for any combination of options or no options at all.\nIt can be used as a default message if there is no message specified for particular options.\nThis version of `message` should be used as a last message definition in the module otherwise it will be always returned even if there will be message better adjusted to the options.\n\n##","ref":"Dsv.Validator.html#message/1-plain-string"},{"type":"macro","title":"Example - Dsv.Validator.message/1","doc":"iex> defmodule Dsv.IsTrue do\n      ...>   use Dsv.Validator\n      ...>\n      ...>   message \"Provided value is invalid. Accepted value is ':true'\"\n      ...>\n      ...>   def valid?(data, options \\\\ []), do: if data == :true, do: :true, else: :false\n      ...> end\n      iex> Dsv.IsTrue.validate(:false)\n      {:error, \"Provided value is invalid. Accepted value is ':true'\"}\n\n#","ref":"Dsv.Validator.html#message/1-example"},{"type":"macro","title":"Function that returns a string - Dsv.Validator.message/1","doc":"Provide message for any combination of options or no options at all.\nIt can be used as a default message if there is no message specified for particular options.\nThis version of `message` should be used as a last message definition in the module otherwise it will be always returned even if there will be message better adjusted to the options.\n\nThe function will receive two (three in case of :complex option set to true) that can be used to create error message:\n  * data - input data\n  * options - validation options\n  * errors - map of errors - avaliable only when [:complex](#module-complex-validators) is set to true\n\n##","ref":"Dsv.Validator.html#message/1-function-that-returns-a-string"},{"type":"macro","title":"Example - Dsv.Validator.message/1","doc":"iex> defmodule Dsv.IsTrue do\n      ...>   use Dsv.Validator\n      ...>\n      ...>   message fn _, _ -> \"Provided value is invalid. Accepted value is ':true'\"\n      ...>\n      ...>   def valid?(data, options \\\\ []), do: if data == :true, do: :true, else: :false\n      ...> end\n      iex> Dsv.IsTrue.validate(:false)\n      {:error, \"Provided value is invalid. Accepted value is ':true'\"}\n\n#","ref":"Dsv.Validator.html#message/1-example"},{"type":"macro","title":"Tuple with keys and message - Dsv.Validator.message/1","doc":"When the `message` macro is used with the tuple, the first element decides which message should be used on validation failure.\nThe second element of the tuple is the message that will be returned on failure.\nBy default message is connected with validation options.\nIt is possible to change this behaviour by passing `message_key_mapper` option to the `Dsv.Validator` module.\n\n##","ref":"Dsv.Validator.html#message/1-tuple-with-keys-and-message"},{"type":"macro","title":"Example - Dsv.Validator.message/1","doc":"defmodule Dsv.IsTrue do\n        use Dsv.Validator, message_key_mapper: fn\n           :g -> [:general]\n           :a -> [:additional_info]\n           _ -> []\n        end\n     \n        message {:general, \"Provided value is invalid. Accepted value is ':true'\"}\n        message {:additional_info, \"Provided value '<%= inspect data %>' is invalid. The only accepted value is ':true'\"}\n        message \"This will be used in case options doesn't match any previous value\"\n     \n        def valid?(data, options \\\\ []), do: if data == :true, do: :true, else: :false\n      end\n      iex> Dsv.IsTrue.validate(:false, :g)\n      {:error, \"Provided value is invalid. Accepted value is ':true'\"}\n      iex> Dsv.IsTrue.validate(:false, :a)\n      {:error, \"Provided value 'false' is invalid. The only accepted value is ':true'\"}\n      iex> Dsv.IsTrue.validate(:false, :anything_else)\n      {:error, \"This will be used in case options doesn't match any previous value\"}\n      iex> Dsv.IsTrue.validate(:false)\n      {:error, \"This will be used in case options doesn't match any previous value\"}\n\nTo associate message with multiple options, provide options in the list as the first argument of the tuple pass to `message` macro.\nMessage associated with multiple options will be returned no matter in what order user will use those options in the validate function.\n\n##","ref":"Dsv.Validator.html#message/1-example"},{"type":"macro","title":"Example - validator with `:min` and `:max` options. - Dsv.Validator.message/1","doc":"In the case of a validator that accepts two options (`:min` and `:max` in this case) we can create messages for each option combination (`:min`, `:max` and `[:min, :max]` - in this case, it doesn't matter in what order we provide `[:min, :max]` or `[:max, :min]` options to the message macro as long as validator accepts them in any order).\n\n      iex> defmodule Dsv.StringLength do\n      ...>   use Dsv.Validator\n      ...>\n      ...>   message {:min, \"Value is too short\"}\n      ...>   message {:max, \"Value is too long\"}\n      ...>   message {[:min, :max], \"Value is too short or too long\"}\n      ...>\n      ...>   def valid?(data, [{:min, min_length}]), do: String.length(data) > min_length\n      ...>   def valid?(data, [{:max, max_length}]), do: String.length(data)     def valid?(data, [{:min, min_length}, {:max, max_length}]), do: valid?(data, min: min_length) and valid?(data, max: max_length)\n      ...>   def valid?(data, [{:max, max_length}, {:min, min_length}]), do: valid?(data, min: min_length) and valid?(data, max: max_length)\n      ...> end\n      iex> Dsv.StringLength.validate(\"Test\", min: 10)\n      {:error, \"Value is too short\"}\n      iex> Dsv.StringLength.validate(\"Test\", max: 3)\n      {:error, \"Value is too long\"}\n      iex> Dsv.StringLength.validate(\"Test\", min: 3, max: 4)\n      {:error, \"Value is too short or too long\"}\n      iex> Dsv.StringLength.validate(\"Test\", max: 4, min: 3)\n      {:error, \"Value is too short or too long\"}\n\n\n#","ref":"Dsv.Validator.html#message/1-example-validator-with-min-and-max-options"},{"type":"macro","title":"Order of the message definition - Dsv.Validator.message/1","doc":"Order of message definition is important. In case of overlapping definitions, the first message that matches will be used.\n\n#","ref":"Dsv.Validator.html#message/1-order-of-the-message-definition"},{"type":"macro","title":"Default message - Dsv.Validator.message/1","doc":"If there is no definition of the message, the default value (\"Unknown validation error\") will be returned in case of any validation error.\n\n#","ref":"Dsv.Validator.html#message/1-default-message"},{"type":"macro","title":"Additional information in the error message - Dsv.Validator.message/1","doc":"Defining messages with data and options information. Besides simple messages like the above, we can create messages that will contain data under validation as well as options used during validation.\nThere are two ways of creating such message:\n  * EEx (Embedded Elixir) - the EEx string can use `data`, `options`, and `errors` values.\n  * Function\n    * with two arguments:\n      * data\n      * options\n    * with three arguments:\n      * data\n      * options\n      * errors\n\nIn the case of EEx as well as function data are the data that are under validation, options are the options provided to the validator and errors are the errors from sub validator in the case of `:complex` validator.\n\n\n      defmodule Dsv.StringLength do\n        use Dsv.Validator\n     \n        message {:min, \"Value '<%= data %>' is shorter than expected <%= options[:min] %> characters.\"}\n        message {:max, \"Value '<%= data %>' is longer than expected <%= options[:max] %> characters.\"}\n        message {[:min, :max], fn data, options -> \"Value '#{data}' is shorter than expected #{options[:min]} or longer than expected #{options[:max]} characters.\" end}\n     \n        def valid?(data, [{:min, min_length}]), do: String.length(data) > min_length\n        def valid?(data, [{:max, max_length}]), do: String.length(data)   Dsv.StringLength.validate(\"test\", min: 6)\n      {:error, \"Value 'test' is shorter than expected 6 characters.\"}\n\n      iex> Dsv.StringLength.validate(\"test\", max: 2)\n      {:error, \"Value 'test' is longer than expected 2 characters.\"}\n\n      iex> Dsv.StringLength.validate(\"test\", min: 10, max: 100)\n      {:error, \"Value 'test' is shorter than expected 10 or longer than expected 100 characters.\"}\n\n\n      In case of validation failure, we get a tuple with `:error` as the first element and the error message as the second element.","ref":"Dsv.Validator.html#message/1-additional-information-in-the-error-message"},{"type":"callback","title":"Dsv.Validator.valid?/2","doc":"This is a function that need to be implemented in a validator that use `Dsv.Validator` module.\nThis is basic function to validated input data.","ref":"Dsv.Validator.html#c:valid?/2"},{"type":"callback","title":"Dsv.Validator.validate/2","doc":"This is function that has default implementation and don't need to be implemented.\nWhen you will leave default implementation it will use `valid?` function to check if input data are valid and return `:ok` or `{:error, error}` result.\nThe error value in the error result will contain default message or the message defined in the validator by the `Dsv.Validator.message/1` macro.","ref":"Dsv.Validator.html#c:validate/2"},{"type":"type","title":"Dsv.Validator.input_data/0","doc":"","ref":"Dsv.Validator.html#t:input_data/0"},{"type":"extras","title":"Elixir Validator Library","doc":"# Elixir Validator Library\n\n**Simple validator for Elixir**\n\n![Elixir Version](https://img.shields.io/badge/elixir-v1.16.0%2B-blue.svg)\n![License](https://img.shields.io/badge/license-MIT-green.svg)\n\nA versatile and extensible validation library for Elixir. Easily validate data for different data types and scenarios.","ref":"readme.html"},{"type":"extras","title":"Features - Elixir Validator Library","doc":"- Provides a set of validators out of the box.\n- Allows for easy extension with custom validation rules.\n- Supports various data types, including numbers, strings, dates, and more.\n- Flexible and user-friendly API for integrating validation into your Elixir applications.","ref":"readme.html#features"},{"type":"extras","title":"Installation - Elixir Validator Library","doc":"Add the `validator` package to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:dsv, \"~> 0.2.1\"}\n  ]\nend\n```\n\nThen, run:\n\n```bash\n$ mix deps.get\n```","ref":"readme.html#installation"},{"type":"extras","title":"Basic usage - Elixir Validator Library","doc":"To use the `Dsv` validator library to validate your data, use `Dsv.validate/2` function:\n\n\n```elixir\niex> Dsv.validate(\"Test name\", length: [min: 3])\n:ok\n```\n\nThis is an example of checking if a string has minimum 3 graphemes.\n\n\nFull description with examples is available in the function documentation `Dsv.validate/2`.","ref":"readme.html#basic-usage"},{"type":"extras","title":"Documentation - Elixir Validator Library","doc":"For detailed documentation and examples, please refer to the [library documentation](`Dsv`).","ref":"readme.html#documentation"},{"type":"extras","title":"Contributing - Elixir Validator Library","doc":"Contributions are welcome! If you have suggestions, bug reports, or want to add new validation rules, please open an issue or submit a pull request.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Elixir Validator Library","doc":"This library is released under the [MIT License](LICENSE).","ref":"readme.html#license"},{"type":"extras","title":"Dsv Validator Documentation","doc":"# Dsv Validator Documentation\n\nWelcome to the guide on validating data with the `Dsv` Validator module. \n`Dsv` provides multiple ways to validate data, catering to different user preferences. \nThis document outlines the various methods available for validating data.","ref":"how_to_validate.html"},{"type":"extras","title":"Overview of Validation Approaches - Dsv Validator Documentation","doc":"Dsv offers different ways to validate data:\n\n1. [**Basic Validation**](#basic-validations): Perform validation on a single value using a keywrod list validators description.\n2. [**Pipeline Validation**](#pipeline-validations): Define validation rules for each field in a structured, readable manner.\n3. [**Mirror Data Structure Validation**](#mirror-data-structure): Validate complex data structures by mirroring the structure.\n4. [**List of Paths and Validators**](#list-of-paths-and-validators): Use a list of paths and associated validators for nested data structures.","ref":"how_to_validate.html#overview-of-validation-approaches"},{"type":"extras","title":"Basic Validation - Dsv Validator Documentation","doc":"- [single value](#single-value-one-validator)\n- [single value multiple validators](#single-value-multiple-validators)\n\nBasic validation allows you to validate individual values quickly using the `Dsv.validate/2` function. \nPass the value to validate as the first argument and specify the validator in the second argument.\n\n#","ref":"how_to_validate.html#basic-validation"},{"type":"extras","title":"Single value one validator - Dsv Validator Documentation","doc":"To validate simple data like string, number etc. use `Dsv.validate/2` function.\n\n##","ref":"how_to_validate.html#single-value-one-validator"},{"type":"extras","title":"Example single value validation with single validator: - Dsv Validator Documentation","doc":"iex> Dsv.validate(\"Test name\", length: [min: 3])\n    :ok\n\n\nThis code will check if `\"Test name\"` value is at least of length of 3. \nAs the validator passed, result is `:ok`\n\n#","ref":"how_to_validate.html#example-single-value-validation-with-single-validator"},{"type":"extras","title":"Single value multiple validators - Dsv Validator Documentation","doc":"You can combine multiple validators, and validation passes only if all of them succeed. When the value is valid, `Dsv.validate/2` returns `:ok`. If invalid, it returns a tuple with a list of errors.\n\n##","ref":"how_to_validate.html#single-value-multiple-validators"},{"type":"extras","title":"Example - basic validation, multiple validators - Dsv Validator Documentation","doc":"Check if the value has a minimum length of 3 and starts with \"Test\".\n\n    iex> Dsv.validate(\"Test name\", length: [min: 3], format: ~r/^Test.*/)\n    :ok\n\n##","ref":"how_to_validate.html#example-basic-validation-multiple-validators"},{"type":"extras","title":"Example - basic validation, multiple validators, failed validation - Dsv Validator Documentation","doc":"Check if the value has a minimum length of 3, maximum length of 4 and starts with \"Test\".\n\n    iex> Dsv.validate(\"Wrong name\", length: [min: 3, max: 4], format: ~r/^Test.*/)\n    {\n      :error,\n      [\n          \"Value \\\"Wrong name\\\" has wrong length. Minimum lenght is 3, maximum length is 4\",\n          \"Value Wrong name does not match pattern ^Test.*\"\n      ]\n    }","ref":"how_to_validate.html#example-basic-validation-multiple-validators-failed-validation"},{"type":"extras","title":"Pipeline validations - Dsv Validator Documentation","doc":"Pipeline validation allows for structured and readable approach to complex data structure validation. \nThis approach consists of three steps: creating validation, adding validators, and validating data.\n\n\n- [Create validation](#create-validation)\n- [Add validators](#add-validators)\n- [Validate data](#validate-data)\n- [Use field from data as validation rule](#two-fields-from-validated-data-in-one-validator)\n\n#","ref":"how_to_validate.html#pipeline-validations"},{"type":"extras","title":"Create validation - Dsv Validator Documentation","doc":"To use pipeline validations, `Dsv` needs to create a data structure that gathers all the information about the validation rules. \n`Dsv.validation/1` allows you to create this structure. To do so, call this function with the data to validate.\n\n\n##","ref":"how_to_validate.html#create-validation"},{"type":"extras","title":"create validation example - Dsv Validator Documentation","doc":"validation = Dsv.validation(%{\n\t\t\t\"first\" => \"This is the first element to validate\", \n\t\t\t\"second\" => %{\n\t\t\t\t\"third\" => 42, \"fourth\" => [-1, 0, 1]\n\t\t\t}\n\t\t})\n\nThis is the first step. The result of this step can the be pipelined into the `Dsv.add_validator/3` function.\n\n#","ref":"how_to_validate.html#create-validation-example"},{"type":"extras","title":"Add validators - Dsv Validator Documentation","doc":"After creating the validation, the next step is to add all the desirable validators.\nFor each field that will be validated, you should call `Dsv.add_validator/3`. \nThis function will add a list of validators to the field specified in the path.\nThe path is in the form of a list, where each element of the list represents a part of the path to the validated element.\n\n##","ref":"how_to_validate.html#add-validators"},{"type":"extras","title":"Add validators example - Dsv Validator Documentation","doc":"validation = validation\n\t\t|> Dsv.add_validator([\"first\"], length: [min: 1])\n\t\t|> Dsv.add_validator([\"second\", \"fourth\"], length: [max: 20], position: [\"1\": [equal: 0]])\n\n#","ref":"how_to_validate.html#add-validators-example"},{"type":"extras","title":"Validate data - Dsv Validator Documentation","doc":"The final step in the pipeline is to initiate the validation process by calling the `Dsv.validate` function.\n\n##","ref":"how_to_validate.html#validate-data"},{"type":"extras","title":"validate data example - Dsv Validator Documentation","doc":"validation |> Dsv.validate\n\n##","ref":"how_to_validate.html#validate-data-example"},{"type":"extras","title":"Pipeline validation - full example - Dsv Validator Documentation","doc":"%{\"first\" => \"This is the first element to validate\", \"second\" => %{\"third\" => 42, \"fourth\" => [-1, 0, 1]}}\n\t\t|> Dsv.add_validator([\"first\"], length: [min: 1])\n\t\t|> Dsv.add_validator([\"second\", \"fourth\"], length: [max: 20], position: [\"1\": [equal: 0]])\n\t\t|> Dsv.validate\n\n##","ref":"how_to_validate.html#pipeline-validation-full-example"},{"type":"extras","title":"Pipeline validation - full example - data to validate are provided at the end - Dsv Validator Documentation","doc":"Dsv.validation\n\t\t|> Dsv.add_validator([\"first\"], length: [min: 1])\n\t\t|> Dsv.add_validator([\"second\", \"fourth\"], length: [max: 20], position: [\"1\": [equal: 0]])\n\t\t|> Dsv.validate(%{\"first\" => \"This is the first element to validate\", \"second\" => %{\"third\" => 42, \"fourth\" => [-1, 0, 1]}})\n\nThis approach allow to use the same validator multiple times with different data.\n\n    simple_validator = Dsv.validation\n\t\t|> Dsv.add_validator([\"first\"], length: [min: 1])\n\t\t|> Dsv.add_validator([\"second\", \"fourth\"], length: [max: 20], position: [\"1\": [equal: 0]])\n\n    simple_validator\n\t\t|> Dsv.validate(%{\"first\" => \"This is the first element to validate\", \"second\" => %{\"third\" => 42, \"fourth\" => [-1, 0, 1]}})\n    \n    simple_validator\n\t\t|> Dsv.validate(%{\"first\" => \"Different data to validate \", \"second\" => %{\"third\" => 2, \"fourth\" => [1, 0, 1]}})\n\n    simple_validator\n    |> Dsv.validate(%{\"first\" => \"Different data to validate \", \"second\" => %{\"third\" => 2, \"fourth\" => [1, 1, 1]}})\n    {:error, %{\"second\" => %{\"fourth\" => [%{\"1\": [\"Values must be equal\"]}]}}}\n  \n\n\n#","ref":"how_to_validate.html#pipeline-validation-full-example-data-to-validate-are-provided-at-the-end"},{"type":"extras","title":"Two fields from validated data in one validator - Dsv Validator Documentation","doc":"Sometimes there is a need to use two different fields from input data in a single validator.\nA good example is when a user sends a new password and a password confirmation.\nIn such cases, the validator needs to check if both values are equal.\nTo accomplish this, the `Dsv.bind/3` function is designed.\nIt is intended to accept path to the binded value and the name for the binded value.\nNext whenewer binded value is needed `Dsv.binded/1` function need to be used with the name of the binded value.\n\n##","ref":"how_to_validate.html#two-fields-from-validated-data-in-one-validator"},{"type":"extras","title":"Two fields from validated data in one validator - example - Dsv Validator Documentation","doc":"%{user: %{password: \"test\", password_confirmation: \"test_confirmation\"}}\n\t\t|> Dsv.validation\n\t\t|> Dsv.bind([:user, :password_confirmation], :pass_confirmation)\n\t\t|> Dsv.add_validator([:user, :password], length: [min: 8, max: 40])\n\t\t|> Dsv.add_validator([:user, :password], equal: Dsv.binded(:pass_confirmation))\n\t\t|> Dsv.validate\n\nThis will give the same result as:\n\n\t\t%{user: %{password: \"test\", password_confirmation: \"test_confirmation\"}}\n\t\t|> Dsv.validation\n\t\t|> Dsv.add_validator([:user, :password], length: [min: 8, max: 40])\n\t\t|> Dsv.add_validator([:user, :password], equal: \"test_confirmation\")\n\t\t|> Dsv.validate\n\n##","ref":"how_to_validate.html#two-fields-from-validated-data-in-one-validator-example"},{"type":"extras","title":"Two fields from validate data in one validator - number example - Dsv Validator Documentation","doc":"%{min_members: 10, max_members: 100, actual_members: 30}\n\t\t|> Dsv.validation\n\t\t|> Dsv.bind([:min_members], :min_members)\n\t\t|> Dsv.bind([:max_members], :max_members)\n\t\t|> Dsv.add_validator([:actual_members], number: [gte: Dsv.binded(:min_members), lte: Dsv.binded(:max_members))\n\t\t|> Dsv.validate\n\nThis can be translated to:\n\n\t\t%{min_members: 10, max_members: 100, actual_members: 30}\n\t\t|> Dsv.validation\n\t\t|> Dsv.add_validator([:actual_members], number: [gte: 10, lte: 100])\n\t\t|> Dsv.validate\n\n##","ref":"how_to_validate.html#two-fields-from-validate-data-in-one-validator-number-example"},{"type":"extras","title":"Struct example - Dsv Validator Documentation","doc":"It is possible to use pipelines to validate structs.\n\nLets define Address struct and the Person struct which will contain address.\n\n    defmodule Address do\n      defstruct [:street, :city, :country, :postal_code]\n    end\n\n    defmodule Person do\n      defstruct [:name, :surname, :age, :address]\n    end\n\nNow we can validate those data\n\n    %Person{name: \"Martin\", surname: \"Smith\", age: 43, address: %Address{street: \"St Marks Ave\", city: \"New York\", country: \"USA\", postal_code: \"11213\"}}\n    |> Dsv.validation\n    |> Dsv.add_validator([:name], length: [min: 2, max: 30])\n    |> Dsv.add_validator([:surname], length: [min: 2, max: 30])\n    |> Dsv.add_validator([:age], number: [gt: 0, lt: 140])\n    |> Dsv.add_validator([:address, :street], length: [min: 1, max: 100])\n    |> Dsv.add_validator([:address, :country], in: [\"USA\", \"Canada\", \"Mexico\"])\n    |> Dsv.validate","ref":"how_to_validate.html#struct-example"},{"type":"extras","title":"Mirror data structure - Dsv Validator Documentation","doc":"This approach involves validating complex data structures by mirroring the structure in the validation rules, to do that use `Dsv.validate/2` function.\n\n#","ref":"how_to_validate.html#mirror-data-structure"},{"type":"extras","title":"Basic success example - Dsv Validator Documentation","doc":"Data to validate\n\n     user_info = %{\n          \"name\" => \"Martin\",\n          \"age\" => 45,\n          \"programming_languages\" => [\"PHP\", \"Python\", \"Java\", \"Go\", \"Elixir\", \"Kotlin\"],\n          \"company_data\" => %{\n            \"name\" => \"company name\",\n            \"address\" => %{\n              \"city\" => \"New York\",\n              \"zip code\" => \"10001\"\n            }\n          },\n          \"skills\" => %{\n            :programming => 100,\n            :cooking => 60\n          }\n     }\n\nValidate user info data\n\n    Dsv.validate(user_info, %{\n       \"name\" => [length: [min: 2, max: 50], format: ~r/^[A-Z][a-z]+/],\n       \"age\" => [number: [gte: 21, lte: 120]],\n       \"company_data\" => %{\n         \"name\" => [length: [min: 2]],\n         \"address\" => %{\n           \"city\" => [length: [min: 2]],\n           \"zip code\" => [format: ~r/[0-9]{1,5}/]\n         }\n       }\n    })\nIn this example input data meets validation createria, so the result of the validation will be :ok.\n\n#","ref":"how_to_validate.html#basic-success-example"},{"type":"extras","title":"Basic failure example - Dsv Validator Documentation","doc":"Map with data to validate.\n\n     user_info = %{\n        \"name\" => \"M\",\n        \"age\" => 145,\n        \"programming_languages\" => [\"PHP\", \"Python\", \"Java\", \"Go\", \"Elixir\", \"Kotlin\"],\n        \"company_data\" => %{\n          \"name\" => \"\",\n          \"address\" => %{\n            \"city\" => \"New York\",\n            \"zip code\" => \"a10001\"\n          }\n        },\n        \"skills\" => %{\n          :programming => 100,\n          :cooking => 60\n        }\n     }\n\nValidate data\n\n    Dsv.validate(user_info, %{\n       \"name\" => [length: [min: 2, max: 50], format: ~r/^[A-Z][a-z]+/],\n       \"age\" => [number: [gte: 21, lte: 120]],\n       \"company_data\" => %{\n         \"name\" => [length: [min: 2]],\n         \"address\" => %{\n           \"city\" => [length: [min: 2]],\n           \"zip code\" => [format: ~r/[0-9]{1,5}/]\n         }\n       }\n    })\n\nIn this case validator will return a tuple with a map of errors:\n\n    {:error,\n     %{\n       \"age\" => [\"Value 145 must be lower than or equal: 120 and greater than or equal: 21\"],\n       \"company_data\" => %{\n         \"name\" => [\"Value \\\"\\\" is to short. Minimum lenght is 2\"]\n       },\n       \"name\" => [\n          \"Value \\\"M\\\" has wrong length. Minimum lenght is 2, maximum length is 50\",\n          \"Value M does not match pattern ^[A-Z][a-z]+\"\n        ]\n     }}\n\n\n\n#","ref":"how_to_validate.html#basic-failure-example"},{"type":"extras","title":"Use input data as validator argument - Dsv Validator Documentation","doc":"Sometimes there is a need to use two different fields from input data in a single validator.\nIn this example, we want to compare the levels of two different skills.\nTo achieve this, the `Dsv.validate/3` function is designed.\nThis function operates similarly to `Dsv.validate/2` but with the addition of a map with binded values.\nIt is meant to accept a map where key is the binded value name and value is a path to the binded value.\nTo use binded value in validators `Dsv.binded/1` function need to be used with binded value name as a parameter.\n\n##","ref":"how_to_validate.html#use-input-data-as-validator-argument"},{"type":"extras","title":"Success example - Dsv Validator Documentation","doc":"user_info = %{\n          \"name\" => \"Martin\",\n          \"age\" => 45,\n          \"programming_languages\" => [\"PHP\", \"Python\", \"Java\", \"Go\", \"Elixir\", \"Kotlin\"],\n          \"company_data\" => %{\n            \"name\" => \"Company name\",\n            \"address\" => %{\n              \"city\" => \"New York\",\n              \"zip code\" => \"10001\"\n            }\n          },\n          \"skills\" => %{\n            :programming => 100,\n            :cooking => 60\n          }\n       }\n\n      binded_values = %{\n        cooking_skills: [\"skills\", :cooking]\n      }\n\n      Dsv.validate(user_info, %{\n        \"name\" => [length: [min: 2, max: 50], format: ~r/^[A-Z][a-z]+/],\n        \"age\" => [number: [gte: 21, lte: 120]],\n        \"skills\" => %{\n          :programming => [number: [gt: Dsv.binded(:cooking_skills)]]\n        },\n        \"company_data\" => %{\n          \"name\" => [length: [min: 2]],\n          \"address\" => %{\n            \"city\" => [length: [min: 2]],\n            \"zip code\" => [format: ~r/[0-9]{1,5}/]\n          }\n        }\n      }, binded_values: binded_values)\n      :ok\n\n##","ref":"how_to_validate.html#success-example"},{"type":"extras","title":"Failure example - Dsv Validator Documentation","doc":"user_info = %{\n          \"name\" => \"Martin\",\n          \"age\" => 45,\n          \"programming_languages\" => [\"PHP\", \"Python\", \"Java\", \"Go\", \"Elixir\", \"Kotlin\"],\n          \"company_data\" => %{\n            \"name\" => \"Company name\",\n            \"address\" => %{\n              \"city\" => \"New York\",\n              \"zip code\" => \"10001\"\n            }\n          },\n          \"skills\" => %{\n            :programming => 100,\n            :cooking => 60\n          }\n       }\n\n      binded_values = %{\n        cooking_skills: [\"skills\", :cooking]\n      }\n\n      Dsv.validate(user_info, %{\n        \"name\" => [length: [min: 2, max: 50], format: ~r/^[A-Z][a-z]+/],\n        \"age\" => [number: [gte: 21, lte: 120]],\n        \"skills\" => %{\n          :programming => [equal: Dsv.binded(:cooking_skills)]\n        },\n        \"company_data\" => %{\n          \"name\" => [length: [min: 2]],\n          \"address\" => %{\n            \"city\" => [length: [min: 2]],\n            \"zip code\" => [format: ~r/[0-9]{1,5}/]\n          }\n        }\n      }, binded_values: binded_values)\n      {:error, %{\"skills\" => %{programming: [\"Values must be equal\"]}}}\n\n\n\n##","ref":"how_to_validate.html#failure-example"},{"type":"extras","title":"Struct example - Dsv Validator Documentation","doc":"As in other approches it is possible to use map with validators to validate structs.\n\nLets define Address struct and the Person struct which will contain address.\n\n    defmodule Address do\n      defstruct [:street, :city, :country, :postal_code]\n    end\n\n    defmodule Person do\n      defstruct [:name, :surname, :age, :address]\n    end\n\nNow we can validate those data\n\n    person = %Person{name: \"Martin\", surname: \"Smith\", age: 43, address: %Address{street: \"St Marks Ave\", city: \"New York\", country: \"USA\", postal_code: \"11213\"}}\n    Dsv.validate(person, %{\n      name: [length: [min: 2, max: 30]],\n      surname: [length: [min: 2, max: 30]],\n      age: [number: [gt: 0, lt: 140]],\n      address: %{\n        street: [length: [min: 1, max: 100]],\n        country: [in: [\"USA\", \"Canada\", \"Mexico\"]]\n      }\n    })","ref":"how_to_validate.html#struct-example"},{"type":"extras","title":"List of paths and validators - Dsv Validator Documentation","doc":"This approach involves using a list of keyword lists, where the `path` key is used to define the path to the validated field, and other keys represent the names of the validators.\n\n#","ref":"how_to_validate.html#list-of-paths-and-validators"},{"type":"extras","title":"List of paths and validators - example - Dsv Validator Documentation","doc":"user_info = %{\n      \"name\" => \"Martin\",\n      \"age\" => 45,\n      \"programming_languages\" => [\"PHP\", \"Python\", \"Java\", \"Go\", \"Elixir\", \"Kotlin\"],\n      \"company_data\" => %{\n        \"name\" => \"company name\",\n        \"address\" => %{\n          \"city\" => \"New York\",\n          \"zip code\" => \"10001\"\n        }\n      },\n      \"skills\" => %{\n        :programming => 100,\n        :cooking => 60\n      }\n    }\n\n    Dsv.validate(user_info, paths([\n      [path: [\"name\"], length: [min: 2, max: 50], format: ~r/^[A-Z][a-z]+/],\n      [path: [\"age\"], number: [gte: 21, lte: 120]],\n      [path: [\"company_data\", \"name\"], length: [min: 2]],\n      [path: [\"company_data\", \"address\", \"city\"], length: [min: 2]],\n      [path: [\"company_data\", \"address\", \"zip code\"], format: ~r/[0-9]{1,5}/]\n    ]))\n\n#","ref":"how_to_validate.html#list-of-paths-and-validators-example"},{"type":"extras","title":"List of paths - error example - Dsv Validator Documentation","doc":"user_info = %{\n      \"name\" => \"M\",\n      \"age\" => 145,\n      \"programming_languages\" => [\"PHP\", \"Python\", \"Java\", \"Go\", \"Elixir\", \"Kotlin\"],\n      \"company_data\" => %{\n        \"name\" => \"\",\n        \"address\" => %{\n          \"city\" => \"New York\",\n          \"zip code\" => \"a10001\"\n        }\n      },\n      \"skills\" => %{\n        :programming => 100,\n        :cooking => 60\n      }\n    }\n\n\n    Dsv.validate(user_info, Dsv.paths([\n      [path: [\"name\"], length: [min: 2, max: 50], format: ~r/^[A-Z][a-z]+/],\n      [path: [\"age\"], number: [gte: 21, lte: 120]],\n      [path: [\"company_data\", \"name\"], length: [min: 2]],\n      [path: [\"company_data\", \"address\", \"city\"], length: [min: 2]],\n      [path: [\"company_data\", \"address\", \"zip code\"], format: ~r/[0-9]{1,5}/]\n    ]))\n    {:error,\n     %{\n       \"age\" => [\"Value 145 must be lower than or equal: 120 and greater than or equal: 21\"],\n       \"company_data\" => %{\n         \"name\" => [\"Value \\\"\\\" is to short. Minimum lenght is 2\"]\n       },\n       \"name\" => [\n          \"Value \\\"M\\\" has wrong length. Minimum lenght is 2, maximum length is 50\",\n          \"Value M does not match pattern ^[A-Z][a-z]+\"\n       ]\n     }}\n\n##","ref":"how_to_validate.html#list-of-paths-error-example"},{"type":"extras","title":"Struct example - Dsv Validator Documentation","doc":"As in other approches it is possible to use list of paths with validators to validate structs.\n\nLets define Address struct and the Person struct which will contain address.\n\n    defmodule Address do\n      defstruct [:street, :city, :country, :postal_code]\n    end\n\n    defmodule Person do\n      defstruct [:name, :surname, :age, :address]\n    end\n\nNow we can validate those data\n\n    person = %Person{name: \"Martin\", surname: \"Smith\", age: 43, address: %Address{street: \"St Marks Ave\", city: \"New York\", country: \"USA\", postal_code: \"11213\"}}\n    Dsv.validate(person, Dsv.paths([\n      [path: [:name], length: [min: 2, max: 30]],\n      [path: [:surname], length: [min: 2, max: 30]],\n      [path: [:age], number: [gt: 0, lt: 140]],\n      [path: [:address, :street], length: [min: 1, max: 100]],\n      [path: [:address, :country],  in: [\"USA\", \"Canada\", \"Mexico\"]]\n    ]))","ref":"how_to_validate.html#struct-example"},{"type":"extras","title":"Changelog","doc":"# 0.2.1\n* Add changelog for 0.2.0\n\n# 0.2.0\n\n#","ref":"changelog.html"},{"type":"extras","title":"New features: - Changelog","doc":"* Binded values - possibility to bind values from validated data and use themn in the validator options.\n* Dsv.Type validator - validate type of the input data\n* Reusable pipeline validators - allow to create pipeline validator and then validate different data with the same pipeline validator.\n\n# 0.1.1\n\n#","ref":"changelog.html#new-features"},{"type":"extras","title":"Fixes: - Changelog","doc":"* Iterable protocol, default map implementation finish iteration with proper return value when no elemnts in map left\n* Type `validators` spec\n* Type spec for Dsv.Date.validate/2 function\n* Dsv.Validator.generate_message function implementation with default errors argument for message function\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Tests: - Changelog","doc":"* Dsv.Comparator implementation for Number\n* Dsv.Empty - DateTime, Date, NaiveDateTime, Time, Float, empty List\n* Iterable protocol - Map iteration, Tuple - element not found\n* Permutations - test for permutations module\n\n# 0.1.0\n* Initial release","ref":"changelog.html#tests"}],"content_type":"text/markdown"}